import logging

import numpy as np

from deploy.server.base.inference_server import InferenceServer
from misc_util.network_utils import receive_image, send_image
from misc_util.timer import log_time


def get_ndarray_diagnostics(data):
    assert (isinstance(data, np.ndarray))
    diagnostics_string = "Shape {} | MIN {} | MAX {} | dtype {}".format(data.shape, np.min(data), np.max(data),
                                                                        data.dtype)
    return diagnostics_string


class KeypointInferenceServer(InferenceServer):
    def __init__(self,
                 inference_provider=None,
                 open_port=None,
                 name="inference server",
                 compressed_send=False,
                 do_logging=False):
        super().__init__(open_ports=[open_port],
                         name=name,
                         do_logging=do_logging)

        self.compressed_send = compressed_send
        self.inference_provider = inference_provider
        self.logger.setLevel(logging.WARNING)
        return

    @log_time
    def _predict(self, image):
        return self.inference_provider.predict_single_image(image)

    def _run_inner_loop(self):
        msg, image = receive_image(self._get_open_port_socket())
        self.logger.debug(get_ndarray_diagnostics(image))
        self.logger.debug("Received image from {}".format(msg))

        result = self._predict(image)[0].numpy()
        print(get_ndarray_diagnostics(result))
        send_image(self._get_open_port_socket(),
                   result[0, :, :, 0],
                   message=self.name,
                   compressed_send=self.compressed_send)
        return True
