import argparse
import os

from config.server_config import ServerConfig
from deploy.saved_model_inferrer import SavedModelInferrer
from deploy.server.base.keypoint_inference_server import KeypointInferenceServer
from misc_util.tf_util import limit_gpu_memory_usage


def parse_args():
    parser = argparse.ArgumentParser(description="Keypoint Inference Server")
    parser.add_argument('--config_yaml', dest='config_yaml', required=True, help='')
    return parser.parse_args()


if __name__ == '__main__':
    args = parse_args()
    server_config = ServerConfig(args.config_yaml)
    listen_address = 'tcp://*:' + str(server_config.get("port"))

    dtype = server_config.get('dtype')

    # configure the tf session
    os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
    os.environ["CUDA_VISIBLE_DEVICES"] = str(server_config.get("gpu"))  # choose from GPUs 0-3
    # only use as much VRAM as needed
    limit_gpu_memory_usage()

    # get the inference parameters ready
    model_dir = server_config.get("model_dir")

    input_params = {
        "image_filenames": "",
        "output_dir": "",
        "model_dir": model_dir,
        "annotations": None,
        "batch_size": 1
    }
    inferrer = SavedModelInferrer(input_params)

    # start the server
    server = KeypointInferenceServer(inferrer,
                                     listen_address,
                                     name=server_config.get("name"),
                                     compressed_send=False,
                                     do_logging=True)
    server.run()
