import argparse

import tensorflow as tf
from tensorflow.keras.models import load_model
from tensorflow.python.compiler.tensorrt import trt_convert as trt


def convert_saved_model_tf2(saved_model_dir, output_dir, precision='float32'):
    if precision == 'float32':
        conversion_params = trt.DEFAULT_TRT_CONVERSION_PARAMS._replace(
            precision_mode=trt.TrtPrecisionMode.FP32)
    else:
        conversion_params = trt.DEFAULT_TRT_CONVERSION_PARAMS._replace(
            precision_mode=trt.TrtPrecisionMode.FP16)

    conversion_params = conversion_params._replace(minimum_segment_size=17)
    conversion_params = conversion_params._replace(precision_mode=trt.TrtPrecisionMode.FP32)

    converter = trt.TrtGraphConverterV2(
        input_saved_model_dir=saved_model_dir,
        conversion_params=conversion_params)
    converter.convert()

    converter.save(output_dir)
    return


def h5_to_saved_model(h5_path, saved_model_dir):
    model = load_model(h5_path)
    model.save(saved_model_dir, save_format='tf')


def parse_args():
    parser = argparse.ArgumentParser(description="TRT Utils")
    parser.add_argument('--convert_trt', action='store_true', help='')
    parser.add_argument('--convert_h5', action='store_true', help='')
    parser.add_argument('--sm_dir')
    parser.add_argument('--h5')
    parser.add_argument('--output_dir')
    return parser.parse_args()


if __name__ == '__main__':
    tf.keras.backend.set_learning_phase(0)

    args = parse_args()

    input_saved_model_dir = args.sm_dir
    input_h5 = args.h5
    output_dir = args.output_dir

    if args.convert_trt:
        convert_saved_model_tf2(input_saved_model_dir, output_dir)
    elif args.convert_h5:
        h5_to_saved_model(input_h5, output_dir)
