import tensorflow as tf

from research.ablation_studies.train.keypoint_tf import KeypointHelper


# we expect keypoints to be in image coordinates
# for correct heatmaps we need to transform them to heatmap coordinates
def create_heatmaps(image_width,
                    image_height,
                    heatmap_width,
                    heatmap_height,
                    std_dev,
                    keypoints,
                    keypoint_helper):
    num_keypoints = tf.shape(keypoints)[0]

    def _create_gaussian_around(center_x, center_y, standard_dev, width_, height_):
        y = tf.range(0, width_, 1, dtype=tf.float32)
        x = tf.range(0, height_, 1, dtype=tf.float32)[:, tf.newaxis]
        exp_parameter = -(((x - center_x) ** 2 + (y - center_y) ** 2) / (2 * standard_dev ** 2))
        return tf.math.exp(exp_parameter)

    def create_single_heatmap(keypoint, single_heatmap_shape, kp_helper):
        # scale to heatmap size
        scaled_keypoint = kp_helper.rescale(keypoint,
                                            image_width,
                                            image_height,
                                            single_heatmap_shape[1],
                                            single_heatmap_shape[0])

        heatmap = _create_gaussian_around(kp_helper.get_y(scaled_keypoint),
                                          kp_helper.get_x(scaled_keypoint),
                                          std_dev, single_heatmap_shape[1], single_heatmap_shape[0])

        heatmap = tf.cond(KeypointHelper.is_none_keypoint(keypoint),
                          lambda: tf.zeros(shape=single_heatmap_shape, dtype=tf.float32),
                          lambda: tf.reshape(heatmap, shape=single_heatmap_shape),
                          name='map_none_keypoint_to_zeros')

        return heatmap

    heatmaps = tf.map_fn(lambda kp:
                         create_single_heatmap(kp, (heatmap_height, heatmap_width, 1), keypoint_helper),
                         keypoints, name='map_heatmaps_to_keypoints')

    # switch the dimensions so we have channels last
    heatmaps = tf.transpose(heatmaps, perm=[3, 1, 2, 0])
    heatmaps = tf.reshape(heatmaps, shape=(heatmap_height, heatmap_width, num_keypoints))
    return heatmaps
