import logging
import sys


class LoggedObject:
    def __init__(self, do_logging=False):
        self.do_logging = do_logging
        self.logger = logging.getLogger(self.__class__.__name__ + ":" + str(id(self)))
        if self.logger.handlers:
            self.logger.handlers = []
        self._init_logger()
        self.logger.setLevel(logging.DEBUG)

        if not do_logging:
            self.logger.disabled = True
            # logging.disable(logging.CRITICAL)

    # from https://realpython.com/python-logging/
    def _init_logger(self):
        filename = self.__class__.__name__ + "_" + str(id(self)) + '.log'
        file_handler = logging.FileHandler(filename)
        file_handler.setLevel(logging.INFO)

        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setLevel(logging.DEBUG)

        c_format = logging.Formatter('%(name)s - %(levelname)s - %(message)s')
        f_format = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        console_handler.setFormatter(c_format)
        file_handler.setFormatter(f_format)

        # self.logger.addHandler(file_handler)
        self.logger.addHandler(console_handler)
        return
