import tensorflow as tf


def limit_gpu_memory_usage():
    gpus = tf.config.experimental.list_physical_devices('GPU')
    if gpus:
        try:
            # Currently, memory growth needs to be the same across GPUs
            for gpu in gpus:
                tf.config.experimental.set_memory_growth(gpu, True)
            logical_gpus = tf.config.experimental.list_logical_devices('GPU')
            print(len(gpus), "Physical GPUs,", len(logical_gpus), "Logical GPUs")
        except RuntimeError as e:
            # Memory growth must be set before GPUs have been initialized
            print(e)


def get_random_stateless_tf_bool():
    return tf.cast(tf.floor(tf.random.uniform([]) + 0.5), tf.bool)


def get_random_tf_bool(random_seed=None):
    # return tf.cast(tf.floor(tf.random.uniform([]) + 0.5), tf.bool)
    return tf.cast(tf.floor(tf.add(tf.random.stateless_uniform([], seed=(random_seed, random_seed)), 0.5)), tf.bool)


# from https://github.com/tensorflow/tpu/blob/06ac8c2af4c196161260d45ba5aa0be923c15a3c/models/official/efficientnet/autoaugment.py#L467
def randomly_negate_tensor(tensor, random_seed=None):
    """With 50% prob turn the tensor negative."""
    do_negate = get_random_tf_bool(random_seed=random_seed)
    final_tensor = tf.cond(do_negate, lambda: -tensor, lambda: tensor)
    return final_tensor
