import os

from tensorflow.python.keras.models import load_model

from research.ablation_studies.dataset.dataset_utils import gather_tfrecords, sequence_image_feature_description
from research.ablation_studies.dataset.pipeline_creator import ClassificationSequenceInputMap, SequencePipelineCreator, \
    ClassificationKeypointSequenceInputMap
from research.ablation_studies.model.create_classifier_models import compile_created_model_classifier
from research.ablation_studies.model.create_combo_models import create_combo_sequence_classifier, \
    create_combo_sequence_keypoint_classifier
from research.ablation_studies.model.create_keypoint_models import get_preprocess_func, compile_created_model_keypoint
from research.ablation_studies.train.model_classification_trainer import ModelClassificationTrainer

if __name__ == '__main__':
    contine_training = False
    # continue_file = "/home/florin/diss_models/effnetb0_classify_rnn_kp_diss/rnn_kp_ts3_shift3_lstm128_bnf_adam_upc/1_chkpt/rnn_kp_ts3_shift3_lstm128_bnf_adam_upc_best.087.h5"

    arch = "effnetb0"
    config_id = "{}_combo_diss".format(arch)

    combo_type = "kp"  # "kp" "cls"

    convlstm_size = 64

    constant_lr = None  # 1e-5
    initial_lr = 1e-3
    batch_size = 4
    time_steps = 30
    shift = time_steps
    sequence_length = 60

    unfreeze_blocks = (7,)
    freeze_bns = True

    do_basic_imagenet = False

    input_shape = (224, 224, 3)
    output_shape = (166, 166)

    train_prefix_id = "{}_combo_ts{}_shift{}_convlstm{}_7tbnf_normlr3_aug".format(combo_type, time_steps, shift,
                                                                                  convlstm_size)

    train_item_size = 11160 / time_steps
    test_item_size = 2880 / time_steps

    ##########################################################################

    # classifier_model_weights = "/data/work/sequence_analysis_models/convlstm-30-64.h5"
    # keypoint_model_weights = "/data/work/sequence_analysis_models/effnetb0_detail_u4_ff_ra9_final.h5"

    classifier_model_weights = "/home/florin/diss_models/final_weights/sequence_analysis/convlstm-30-64.h5"
    keypoint_model_weights = "/home/florin/diss_models/final_weights/sequence_analysis/effnetb0_detail_u4_ff_ra9_final.h5"

    # tfrecord_dir = "/data/work/sequence/"
    tfrecord_dir = "/home/florin/data/sequence/"

    tfrecords = gather_tfrecords(tfrecord_dir=tfrecord_dir)

    # get datasets train/test
    if combo_type == "cls":
        input_mapper = ClassificationSequenceInputMap(
            input_shape,
            (0, 0),
            6,
            downscale_ratio=1,
            input_prep_func=get_preprocess_func(arch),
            use_augmentation=True)
    else:
        input_mapper = ClassificationKeypointSequenceInputMap(
            input_shape,
            output_shape,
            6,
            downscale_ratio=1,
            input_prep_func=get_preprocess_func(arch),
            use_augmentation=True)

    pipeline = SequencePipelineCreator(
        input_mapper,
        batch_size,
        tfrecords,
        image_features=sequence_image_feature_description,
        timesteps=time_steps,
        shift=shift)
    ds_train, ds_test = pipeline.get_datasets()

    if not contine_training:
        if combo_type == "kp":
            my_model = create_combo_sequence_keypoint_classifier(
                keypoint_model_weights,
                classifier_model_weights,
                input_shape,
                time_steps,
                convlstm_size,
                unfreeze_last_classifier_block=True,
                unfreeze_last_keypoint_block=False)
        elif combo_type == "cls":
            my_model = create_combo_sequence_classifier(
                keypoint_model_weights,
                classifier_model_weights,
                input_shape,
                time_steps,
                unfreeze_last_classifier_block=True,
                unfreeze_last_keypoint_block=False)
    else:
        my_model = load_model(continue_file, compile=False)

    if combo_type == "cls":
        my_model = compile_created_model_classifier(my_model)
    else:
        my_model = compile_created_model_keypoint(my_model)

    ##########################################################################

    # model_save_path = "/data/work/diss_models/{}/{}/1/".format(config_id, train_prefix_id)
    model_save_path = "/home/florin/diss_models/final_weights/sequence_analysis/{}/{}/1/".format(config_id,
                                                                                                 train_prefix_id)
    os.makedirs(model_save_path, exist_ok=True)
    print("Storing model in {}".format(model_save_path))

    # checkpoint_save_path = "/data/work/diss_models/{}/{}/1_chkpt/".format(config_id, train_prefix_id)
    checkpoint_save_path = "/home/florin/diss_models/final_weights/sequence_analysis/{}/{}/1_chkpt/".format(config_id,
                                                                                                            train_prefix_id)
    os.makedirs(checkpoint_save_path, exist_ok=True)
    print("Storing checkpoints in {}".format(checkpoint_save_path))

    ##########################################################################

    trainer = ModelClassificationTrainer(
        ds_train, ds_test,
        batch_size,
        my_model,
        train_prefix_id,
        checkpoint_save_path,
        train_item_size=train_item_size,
        test_item_size=test_item_size,
        train_neg_weight=0.7492,
        constant_lr=constant_lr,
        initial_lr=initial_lr)

    print("Starting training!")
    trainer.train(200)

    print("Saving model ...")

    my_model.save(model_save_path, save_format='tf')
