import os

from tensorflow.keras.models import Model

from research.ablation_studies.dataset.dataset_utils import gather_tfrecords, sequence_image_feature_description
from research.ablation_studies.dataset.pipeline_creator import PipelineCreator, ClassifcationInputMap
from research.ablation_studies.model.create_classifier_models import create_detail_effnetB0_classifier_no_memory, \
    compile_created_model_classifier
from research.ablation_studies.model.create_keypoint_models import get_preprocess_func
from research.ablation_studies.train.model_classification_trainer import ModelClassificationTrainer

if __name__ == '__main__':
    # tf.debugging.experimental.enable_dump_debug_info(
    #    dump_root="/home/florin/tb_db_logs",
    #    tensor_debug_mode="FULL_HEALTH",
    #    circular_buffer_size=-1)
    arch = "effnetb0"

    config_id = "{}_classify_final".format(arch)
    train_prefix_id = "agnostic_xdense1_weighted_ls0.1_m6_finelr3_nobn"
    constant_lr = None
    initial_lr = 1e-3
    fine_tune_weights = "/home/florin/diss_models/effnetb0_classify_final/agnostic_no_xdense_weighted_ls0.1_m9_frozen/1_chkpt/agnostic_no_xdense_weighted_ls0.1_m9_frozen_best.003.h5"
    effnet_trainable = True
    use_extra_dense1 = True
    use_extra_dense2 = False
    magnitude = 6
    batch_size = 32
    input_shape = (224, 224, 3)

    tfrecord_dir = "/home/florin/data/sequence/"
    tfrecords = gather_tfrecords(tfrecord_dir=tfrecord_dir)

    # get datasets train/test
    input_mapper = ClassifcationInputMap(
        input_shape,
        (0, 0),
        magnitude,
        downscale_ratio=1,
        input_prep_func=get_preprocess_func(arch))

    pipeline = PipelineCreator(input_mapper, batch_size, tfrecords, image_features=sequence_image_feature_description)
    ds_train, ds_test = pipeline.get_datasets()

    # create model
    effnet_input, effnet_output = create_detail_effnetB0_classifier_no_memory(
        input_shape=input_shape,
        effnet_trainable=effnet_trainable,
        use_extra_dense1=use_extra_dense1,
        use_extra_dense2=use_extra_dense2,
        fine_tune_weights=fine_tune_weights)

    my_model = Model(effnet_input, effnet_output,
                     name=config_id)

    my_model = compile_created_model_classifier(my_model)

    model_save_path = "/home/florin/diss_models/{}/{}/1/".format(config_id, train_prefix_id)
    os.makedirs(model_save_path, exist_ok=True)
    print("Storing model in {}".format(model_save_path))

    checkpoint_save_path = "/home/florin/diss_models/{}/{}/1_chkpt/".format(config_id, train_prefix_id)
    os.makedirs(checkpoint_save_path, exist_ok=True)
    print("Storing checkpoints in {}".format(checkpoint_save_path))

    trainer = ModelClassificationTrainer(
        ds_train, ds_test,
        batch_size,
        my_model,
        train_prefix_id,
        checkpoint_save_path,
        train_item_size=11160,
        test_item_size=2880,
        train_neg_weight=0.7492,
        constant_lr=constant_lr,
        initial_lr=initial_lr)

    print("Starting training!")
    trainer.train(200, use_class_weights=True)

    print("Saving model ...")

    my_model.save(model_save_path, save_format='tf')
