from ast import literal_eval

import numpy as np
import pandas as pd
import tensorflow as tf

from research.ablation_studies.dataset.dataset_utils import image_to_tfexample, write_tfrecord_with_examples
from research.ablation_studies.helpers import rescale_keypoint


def create_tfrecords_for_large_dataset():
    all_annos = pd.read_csv('D:/thesis/zen-phd-bobcat/data/annotations/final_consistent_annos.csv', index_col=0,
                            dtype={'subject': np.int32,
                                   'task': np.int32,
                                   'frame': np.int32},
                            converters={
                                'ptip': literal_eval,
                                'ptail': literal_eval})

    print("Dataframe has {} rows.".format(len(all_annos.index)))

    counter = 0
    file_counter = 0
    interval = 500
    examples = []
    for index, row in all_annos.iterrows():
        filename = row['full_path']
        ptip = row['ptip']
        ptail = row['ptail']
        subject_id = row['subject']
        task_id = row['task']
        frame_id = row['frame']
        print("Processing sample: subject {} | task {} | frame {} | ptip {} | ptail {} ".format(subject_id, task_id,
                                                                                                frame_id, ptip, ptail))

        img_bytes = tf.io.read_file(filename)
        img = tf.io.decode_jpeg(img_bytes)
        img = tf.image.convert_image_dtype(img, tf.float32)
        resized_img = tf.image.resize(img, size=(1080, 1920), preserve_aspect_ratio=True)
        print("Resized image from {} to {}".format(img.shape, resized_img.shape))
        resized_img = tf.image.convert_image_dtype(resized_img, tf.uint8)

        resized_img_bytes = tf.io.encode_jpeg(resized_img, format="rgb", chroma_downsampling=False, optimize_size=True)

        ptip_keypoint = rescale_keypoint(ptip, 3840, 2160, 1920, 1080)
        ptail_keypoint = rescale_keypoint(ptail, 3840, 2160, 1920, 1080)

        example = image_to_tfexample(resized_img_bytes,
                                     b'jpg',
                                     subject_id,
                                     frame_id,
                                     # ERROR! We created the large ds with task and frame switched; now fixed in this code
                                     task_id,
                                     1080, 1920,
                                     ptip_keypoint[0], ptip_keypoint[1],
                                     ptail_keypoint[0], ptail_keypoint[1])

        examples.append(example)

        if (counter % interval == 0) and counter > 0:
            tfrecord_filename = "large_dataset_{}.tfrecord".format(file_counter)
            print("Writing tfrecord {}".format(tfrecord_filename))
            write_tfrecord_with_examples(tfrecord_filename, examples)
            examples.clear()

            file_counter += 1

        counter += 1

    if len(examples) > 0:
        tfrecord_filename = "large_dataset_{}.tfrecord".format(file_counter)
        write_tfrecord_with_examples(tfrecord_filename, examples)
        examples.clear()


if __name__ == '__main__':
    create_tfrecords_for_large_dataset()
