import numpy as np
from scipy.interpolate import interp1d

from research.ablation_studies.gestures.dollar import Recognizer, RotateToZero, ScaleToSquare, \
    TranslateToOrigin, NumPoints, SquareSize, DistanceAtBestAngle, AngleRange, \
    AnglePrecision, HalfDiagonal, Result
from research.ablation_studies.gestures.path_distances_extractor import PathDistancesExtractor


def recognize_gesture(keypoint_path):
    distances, unique_path = PathDistancesExtractor(keypoint_path, cumulative=True, unique=True, normalized=True).run()
    points = np.array(unique_path)

    alpha = np.linspace(0, 1, NumPoints)
    interpolator = interp1d(distances, unique_path, kind='slinear', axis=0)
    interpolated_points = interpolator(alpha)

    dollar_points = RotateToZero(interpolated_points.tolist())
    dollar_points = ScaleToSquare(dollar_points, SquareSize)
    dollar_points = TranslateToOrigin(dollar_points)

    recognizer = Recognizer()
    b = float('infinity')
    for temp in recognizer.Templates:
        d = DistanceAtBestAngle(dollar_points, temp, -AngleRange, +AngleRange, AnglePrecision)
        if d < b:
            b = d
            t = temp

    if 't' not in locals():
        return Result("None", 0.0)

    score = 1.0 - (b / HalfDiagonal)
    return Result(t.Name, score)
