# Von Jürgen Hahn bekommen

# $1 Recognizer in Python by Shaun Kane
# Original by Jacob O. Wobbrock, Andrew D. Wilson, Yang Li
# http://depts.washington.edu/aimgroup/proj/dollar/

import math

import numpy as np
# constants
from scipy.interpolate import interp1d

from research.ablation_studies.gestures.path_distances_extractor import PathDistancesExtractor

NumTemplates = 4
NumPoints = 64
# SquareSize = 250.0
SquareSize = 250.0
HalfDiagonal = 0.5 * math.sqrt(SquareSize * SquareSize + SquareSize * SquareSize)
AngleRange = 45.0
AnglePrecision = 2.0
Phi = 0.5 * (-1.0 + math.sqrt(5.0))  # Golden Ratio
minPoints = 60


class Template:
    def __init__(self, name2, points2):
        self.Name = name2
        self.Points = points2

        self.Points = BetterResample(self.Points, NumPoints)
        self.Points = RotateToZero(self.Points)
        # print("BBox before scale {} {}".format(name2, BoundingBox(self.Points)))
        self.Points = ScaleToSquare(self.Points, SquareSize)
        self.Points = TranslateToOrigin(self.Points)


class Result:
    def __init__(self, name2, score2):
        self.Name = name2
        self.Score = score2


class Recognizer:
    def __init__(self):
        # predefined templates
        self.Templates = []
        self.Templates.append(Template('line',
                                       # [(0, 100), (10, 100), (20, 100), (30, 100), (40, 100), (50, 100), (300, 100)]))
                                       [(69, 217), (72, 217), (78, 217),
                                        (82, 217), (85, 217), (86, 217), (87, 217), (90, 217), (91, 217), (92, 217),
                                        (95, 217), (97, 217), (100, 217), (101, 217), (103, 217), (105, 217),
                                        (106, 217), (107, 217), (109, 217), (112, 217), (114, 217), (116, 217),
                                        (118, 217), (119, 217), (120, 217), (122, 217), (124, 217), (126, 217),
                                        (129, 217), (130, 217), (132, 217), (134, 217), (136, 217), (137, 217),
                                        (139, 217), (140, 217), (142, 217), (143, 217), (145, 217), (147, 217),
                                        (148, 217), (150, 217), (154, 217), (157, 217), (159, 217), (160, 217),
                                        (161, 217), (164, 217), (166, 217), (168, 217), (170, 217), (172, 217),
                                        (173, 217), (175, 217), (177, 217), (178, 217), (182, 217), (183, 217),
                                        (184, 217), (186, 217), (187, 217), (189, 217), (190, 217), (192, 217),
                                        (193, 217), (195, 217), (196, 217), (197, 217), (198, 217), (199, 217),
                                        (200, 217), (201, 217), (202, 217), (204, 217), (205, 217), (206, 217),
                                        (208, 217), (209, 217), (210, 217), (211, 217), (212, 217), (214, 217),
                                        (215, 217)]))

        self.Templates.append(Template('square',
                                       [(68, 102), (68, 106), (68, 108), (68, 109), (68, 110), (68, 111), (68, 113),
                                        (68, 114), (68, 115), (68, 116), (68, 117), (68, 118), (68, 119), (68, 120),
                                        (68, 121), (68, 122), (68, 123), (68, 125), (68, 126), (68, 127), (68, 129),
                                        (68, 130), (68, 132), (68, 134), (68, 135), (68, 136), (68, 137), (68, 139),
                                        (68, 141), (68, 142), (68, 143), (68, 145), (68, 146), (68, 148), (68, 149),
                                        (68, 151), (68, 152), (68, 153), (68, 155), (68, 156), (68, 157), (68, 159),
                                        (68, 160), (68, 162), (68, 163), (67, 165), (67, 166), (67, 168), (67, 170),
                                        (67, 172), (67, 173), (67, 176), (67, 178), (67, 180), (67, 182), (67, 183),
                                        (67, 185), (67, 187), (67, 188), (67, 189), (67, 191), (67, 192), (67, 193),
                                        (68, 195), (68, 196), (68, 198), (68, 199), (69, 200), (69, 202), (69, 203),
                                        (69, 205), (69, 207), (69, 208), (69, 209), (69, 210), (69, 211), (69, 212),
                                        (69, 213), (69, 214), (69, 215), (69, 216), (69, 217), (72, 217), (78, 217),
                                        (82, 217), (85, 217), (86, 217), (87, 217), (90, 217), (91, 217), (92, 217),
                                        (95, 217), (97, 217), (100, 217), (101, 217), (103, 217), (105, 217),
                                        (106, 217), (107, 217), (109, 217), (112, 217), (114, 217), (116, 217),
                                        (118, 217), (119, 217), (120, 217), (122, 217), (124, 217), (126, 217),
                                        (129, 217), (130, 217), (132, 217), (134, 217), (136, 217), (137, 217),
                                        (139, 217), (140, 217), (142, 217), (143, 217), (145, 217), (147, 217),
                                        (148, 217), (150, 217), (154, 217), (157, 217), (159, 217), (160, 217),
                                        (161, 217), (164, 217), (166, 217), (168, 217), (170, 217), (172, 217),
                                        (173, 217), (175, 217), (177, 217), (178, 217), (182, 217), (183, 217),
                                        (184, 217), (186, 217), (187, 217), (189, 217), (190, 217), (192, 217),
                                        (193, 217), (195, 217), (196, 217), (197, 217), (198, 217), (199, 217),
                                        (200, 217), (201, 217), (202, 217), (204, 217), (205, 217), (206, 217),
                                        (208, 217), (209, 217), (210, 217), (211, 217), (212, 217), (214, 217),
                                        (215, 217), (215, 216), (215, 215), (215, 213), (215, 212), (215, 211),
                                        (215, 209), (215, 208), (215, 206), (215, 204), (215, 203), (215, 202),
                                        (215, 200), (215, 198), (215, 196), (215, 195), (215, 193), (215, 190),
                                        (215, 188), (215, 184), (215, 183), (215, 178), (215, 177), (215, 175),
                                        (215, 173), (215, 171), (215, 170), (215, 168), (215, 166), (215, 164),
                                        (215, 162), (215, 160), (215, 157), (216, 153), (217, 149), (217, 146),
                                        (217, 143), (217, 141), (217, 139), (217, 137), (218, 136), (218, 134),
                                        (218, 133), (219, 131), (219, 130), (219, 127), (219, 125), (219, 123),
                                        (219, 121), (219, 120), (220, 118), (220, 116), (220, 115), (220, 113),
                                        (220, 112), (220, 111), (220, 109), (220, 107), (220, 106), (220, 105),
                                        (220, 104), (220, 102), (220, 101), (220, 100), (220, 99), (220, 97), (220, 96),
                                        (220, 95), (220, 92), (220, 91), (220, 89), (220, 88), (220, 87), (220, 86),
                                        (220, 85), (220, 83), (220, 82), (218, 82), (214, 82), (211, 82), (207, 82),
                                        (206, 82), (204, 82), (201, 82), (197, 82), (195, 82), (192, 82), (191, 82),
                                        (189, 82), (187, 82), (184, 82), (182, 82), (181, 82), (177, 82), (175, 82),
                                        (173, 82), (171, 82), (170, 82), (168, 82), (166, 82), (164, 82), (163, 82),
                                        (162, 82), (161, 82), (160, 82), (158, 82), (157, 82), (154, 82), (152, 82),
                                        (151, 82), (149, 82), (147, 82), (145, 82), (144, 82), (143, 82), (140, 82),
                                        (138, 82), (136, 82), (133, 82), (131, 82), (127, 82), (125, 82), (123, 82),
                                        (122, 82), (120, 82), (118, 82), (116, 82), (114, 82), (112, 82), (110, 82),
                                        (109, 82), (108, 82), (106, 82), (105, 82), (103, 82), (102, 82), (101, 82),
                                        (100, 82), (99, 82), (97, 82), (96, 82), (95, 82), (93, 82), (92, 82), (90, 82),
                                        (88, 82), (86, 82), (85, 82), (84, 82), (83, 82), (82, 82), (81, 82), (80, 82),
                                        (79, 82), (77, 82), (76, 82), (75, 82), (74, 82), (73, 82), (70, 82),
                                        (69, 82), ]))

        self.Templates.append(Template('triangle',
                                       [(46, 199), (46, 198), (47, 197), (48, 196), (48, 195), (49, 195), (49, 194),
                                        (49, 193), (50, 193), (50, 192), (51, 192), (51, 191), (52, 191), (52, 190),
                                        (53, 189), (53, 188), (54, 187), (55, 187), (56, 186), (56, 185), (56, 184),
                                        (57, 184), (57, 183), (58, 181), (59, 180), (60, 179), (61, 178), (62, 177),
                                        (63, 175), (63, 174), (65, 173), (66, 171), (67, 170), (68, 168), (69, 167),
                                        (70, 166), (71, 164), (73, 162), (75, 160), (77, 159), (78, 157), (79, 156),
                                        (80, 154), (82, 152), (84, 150), (86, 148), (86, 147), (88, 146), (89, 144),
                                        (90, 142), (92, 140), (92, 138), (94, 137), (95, 135), (95, 134), (97, 133),
                                        (98, 131), (99, 130), (100, 129), (101, 128), (103, 127), (104, 126),
                                        (106, 123), (108, 122), (110, 121), (111, 119), (113, 118), (114, 116),
                                        (115, 115), (116, 113), (117, 112), (119, 110), (120, 108), (121, 107),
                                        (123, 104), (124, 104), (125, 102), (125, 101), (126, 100), (127, 99),
                                        (128, 98), (128, 97), (129, 95), (130, 94), (130, 93), (131, 91), (132, 90),
                                        (133, 88), (134, 87), (135, 86), (135, 85), (136, 84), (137, 83), (139, 81),
                                        (139, 79), (140, 78), (142, 76), (144, 73), (145, 71), (146, 70), (146, 68),
                                        (147, 68), (148, 67), (149, 67), (151, 69), (153, 71), (154, 72), (155, 74),
                                        (156, 75), (156, 76), (158, 77), (158, 78), (159, 78), (160, 79), (161, 80),
                                        (162, 81), (163, 82), (164, 83), (164, 84), (165, 84), (166, 86), (167, 87),
                                        (168, 88), (169, 90), (170, 91), (171, 91), (172, 92), (172, 93), (173, 93),
                                        (174, 94), (175, 95), (176, 95), (176, 96), (177, 97), (178, 98), (179, 99),
                                        (181, 102), (182, 103), (182, 105), (184, 105), (184, 107), (185, 108),
                                        (187, 110), (188, 111), (189, 112), (190, 113), (191, 115), (192, 116),
                                        (193, 118), (194, 120), (195, 121), (197, 123), (198, 125), (199, 127),
                                        (201, 129), (201, 130), (203, 132), (205, 134), (206, 136), (208, 138),
                                        (210, 140), (211, 141), (212, 143), (213, 144), (214, 146), (215, 147),
                                        (216, 148), (218, 150), (219, 152), (220, 153), (221, 155), (222, 156),
                                        (224, 158), (225, 159), (226, 161), (227, 161), (228, 162), (229, 163),
                                        (230, 164), (231, 165), (232, 166), (234, 168), (235, 169), (236, 170),
                                        (238, 172), (239, 173), (241, 174), (242, 176), (242, 177), (244, 179),
                                        (246, 180), (247, 182), (249, 183), (250, 184), (250, 185), (252, 186),
                                        (253, 187), (253, 189), (254, 190), (255, 191), (256, 191), (256, 193),
                                        (257, 194), (258, 195), (258, 196), (260, 197), (260, 198), (261, 199),
                                        (262, 200), (260, 200), (258, 200), (256, 200), (255, 200), (253, 200),
                                        (251, 200), (250, 200), (249, 200), (247, 200), (246, 200), (245, 199),
                                        (243, 199), (241, 199), (239, 199), (238, 199), (237, 199), (235, 199),
                                        (234, 198), (232, 198), (231, 198), (230, 198), (227, 198), (225, 198),
                                        (224, 198), (221, 198), (220, 198), (218, 198), (216, 198), (214, 198),
                                        (212, 198), (211, 198), (209, 198), (207, 198), (205, 198), (202, 198),
                                        (199, 198), (197, 198), (194, 198), (192, 198), (188, 198), (186, 198),
                                        (184, 199), (181, 199), (178, 199), (175, 199), (171, 199), (167, 199),
                                        (165, 200), (164, 200), (161, 200), (158, 200), (155, 200), (154, 200),
                                        (150, 200), (148, 201), (145, 201), (142, 202), (140, 202), (136, 202),
                                        (134, 202), (132, 202), (131, 202), (128, 202), (126, 202), (125, 202),
                                        (122, 202), (120, 202), (118, 202), (117, 202), (116, 202), (113, 202),
                                        (111, 202), (110, 202), (109, 202), (107, 203), (106, 203), (104, 203),
                                        (103, 203), (101, 203), (99, 203), (97, 203), (96, 203), (94, 203), (93, 203),
                                        (92, 203), (90, 203), (89, 203), (87, 203), (86, 203), (85, 203), (84, 203),
                                        (82, 203), (80, 203), (79, 203), (78, 203), (77, 203), (76, 203), (75, 203),
                                        (70, 203), (64, 203), (61, 203), (60, 203), (58, 203), (57, 203), (56, 203),
                                        (55, 203), (54, 203), (53, 203), (52, 203), (51, 203), (50, 203), ]))

        self.Templates.append(Template('circle',
                                       [(62, 153), (62, 152), (62, 150), (62, 149), (62, 148), (62, 146), (62, 145),
                                        (62, 144), (62, 143), (62, 142), (62, 140), (62, 139), (62, 138), (62, 137),
                                        (62, 136), (62, 135), (62, 134), (62, 133), (62, 132), (62, 131), (63, 130),
                                        (64, 129), (64, 128), (64, 127), (65, 126), (65, 124), (66, 123), (66, 122),
                                        (66, 121), (67, 120), (68, 119), (68, 117), (69, 117), (69, 115), (69, 114),
                                        (69, 113), (70, 113), (71, 111), (71, 110), (71, 109), (72, 108), (73, 107),
                                        (73, 106), (74, 106), (75, 104), (76, 103), (76, 102), (77, 101), (78, 101),
                                        (78, 100), (79, 99), (80, 98), (80, 97), (82, 96), (83, 95), (84, 94), (84, 93),
                                        (86, 93), (86, 92), (87, 92), (89, 91), (90, 90), (92, 89), (93, 88), (94, 88),
                                        (95, 87), (97, 86), (99, 85), (100, 85), (101, 84), (103, 84), (104, 83),
                                        (105, 83), (109, 81), (110, 81), (112, 80), (113, 79), (114, 79), (116, 79),
                                        (117, 79), (118, 79), (119, 78), (120, 78), (122, 78), (123, 78), (125, 77),
                                        (126, 77), (127, 77), (128, 76), (129, 76), (130, 76), (131, 76), (132, 76),
                                        (133, 76), (134, 76), (135, 76), (136, 76), (137, 76), (138, 76), (139, 76),
                                        (140, 76), (141, 76), (142, 76), (143, 76), (145, 76), (146, 76), (148, 76),
                                        (150, 76), (152, 76), (153, 76), (154, 76), (155, 76), (156, 76), (157, 76),
                                        (158, 76), (159, 77), (160, 77), (161, 77), (162, 77), (164, 77), (165, 77),
                                        (166, 77), (167, 78), (168, 78), (170, 78), (171, 78), (171, 79), (172, 79),
                                        (173, 80), (174, 80), (175, 81), (176, 81), (177, 81), (178, 81), (179, 82),
                                        (180, 83), (181, 83), (182, 83), (182, 85), (183, 85), (184, 85), (185, 86),
                                        (186, 86), (187, 86), (187, 87), (188, 87), (189, 88), (190, 89), (191, 90),
                                        (192, 90), (192, 91), (193, 91), (193, 93), (194, 94), (195, 94), (196, 96),
                                        (196, 97), (197, 97), (198, 98), (199, 99), (200, 100), (201, 101), (202, 102),
                                        (202, 103), (203, 103), (204, 105), (205, 106), (206, 107), (207, 108),
                                        (208, 109), (209, 110), (209, 111), (211, 112), (211, 113), (212, 114),
                                        (213, 116), (214, 117), (214, 118), (214, 119), (215, 120), (216, 121),
                                        (216, 123), (216, 124), (217, 125), (218, 126), (218, 128), (218, 129),
                                        (218, 131), (218, 132), (219, 133), (219, 134), (219, 135), (219, 136),
                                        (219, 137), (219, 138), (219, 139), (219, 140), (219, 141), (219, 142),
                                        (219, 143), (219, 145), (219, 146), (219, 147), (219, 149), (219, 150),
                                        (219, 152), (219, 154), (219, 155), (219, 156), (219, 157), (219, 158),
                                        (219, 159), (219, 160), (219, 162), (219, 163), (218, 164), (217, 166),
                                        (217, 168), (217, 170), (216, 171), (216, 172), (216, 173), (215, 175),
                                        (214, 176), (214, 178), (213, 179), (213, 180), (212, 181), (210, 184),
                                        (210, 185), (209, 186), (208, 187), (207, 188), (207, 189), (206, 190),
                                        (206, 191), (205, 192), (204, 193), (203, 194), (202, 195), (201, 197),
                                        (200, 197), (199, 198), (198, 199), (197, 200), (196, 200), (196, 201),
                                        (194, 202), (193, 203), (191, 204), (187, 206), (184, 206), (183, 207),
                                        (179, 208), (177, 209), (174, 210), (171, 211), (170, 212), (168, 212),
                                        (167, 214), (165, 214), (163, 214), (162, 215), (160, 215), (159, 215),
                                        (157, 216), (156, 216), (155, 217), (154, 217), (153, 217), (152, 217),
                                        (150, 217), (149, 217), (146, 217), (145, 217), (143, 217), (142, 217),
                                        (140, 217), (138, 217), (137, 217), (136, 217), (135, 217), (133, 217),
                                        (131, 217), (130, 217), (128, 217), (127, 217), (126, 217), (125, 217),
                                        (123, 216), (122, 216), (121, 216), (120, 216), (119, 216), (118, 215),
                                        (117, 215), (115, 214), (114, 213), (112, 212), (111, 212), (110, 211),
                                        (109, 211), (108, 209), (107, 209), (106, 209), (104, 208), (103, 207),
                                        (102, 207), (101, 206), (100, 206), (99, 205), (98, 205), (97, 204), (95, 204),
                                        (94, 203), (92, 203), (91, 202), (90, 201), (89, 201), (88, 200), (87, 200),
                                        (85, 199), (84, 199), (83, 199), (83, 198), (81, 197), (80, 196), (79, 196),
                                        (78, 195), (77, 195), (77, 194), (76, 193), (75, 193), (74, 192), (73, 190),
                                        (72, 190), (72, 189), (71, 189), (70, 188), (70, 187), (69, 186), (69, 185),
                                        (69, 184), (68, 183), (68, 182), (68, 181), (67, 180), (66, 179), (66, 178),
                                        (66, 177), (66, 176), (65, 176), (65, 175), (65, 174), (64, 173), (64, 172),
                                        (64, 170), (64, 169), (63, 169), (63, 168), (63, 167), (63, 166), (62, 166),
                                        (62, 165), (62, 164), (62, 163), (62, 162), (62, 160), (62, 159), ]))

    # self.Templates.append(Template('circle', [(34,149), (34,147), (34,145), (33,142), (33,138), (34,133), (34,130), (36,124), (37,119), (40,114), (41,109), (43,106), (46,101), (49,96), (51,93), (54,88), (57,85), (60,82), (66,77), (69,74), (74,71), (79,68), (80,67), (86,63), (91,60), (96,58), (99,57), (105,55), (112,53), (120,51), (126,49), (132,48), (136,47), (142,46), (147,46), (153,46), (159,46), (161,46), (166,46), (171,46), (175,48), (182,50), (188,52), (191,54), (197,57), (201,59), (206,61), (209,64), (211,67), (215,72), (220,77), (224,82), (226,85), (233,94), (234,95), (237,102), (240,108), (241,114), (242,118), (243,125), (244,133), (245,142), (245,150), (245,155), (245,162), (245,169), (245,173), (243,181), (242,186), (240,190), (236,198), (232,204), (226,212), (218,220), (215,223), (207,230), (200,235), (192,242), (184,246), (177,248), (172,250), (164,253), (155,254), (146,254), (141,255), (132,255), (121,254), (113,252), (104,249), (94,245), (89,242), (83,236), (76,231), (68,225), (62,220), (59,216), (53,211), (48,208), (44,203), (42,200), (38,195), (34,190), (31,186), (29,184), (28,180), (27,176), (26,174), (25,171), (25,170),]))
    # The $1 Gesture Recognizer API begins here -- 3 methods
    def Recognize(self, points):
        if PathLength(points) == 0.0:
            return Result("None", 0.0)

        points = Resample(points, NumPoints)
        points = RotateToZero(points)
        points = ScaleToSquare(points, SquareSize)
        points = TranslateToOrigin(points)

        if len(points) == 0:
            return Result("None", 0.0)

        b = float('infinity')
        for temp in self.Templates:
            d = DistanceAtBestAngle(points, temp, -AngleRange, +AngleRange, AnglePrecision)
            if d < b:
                b = d
                t = temp

        score = 1.0 - (b / HalfDiagonal)
        return Result(t.Name, score)

    def AddTemplate(self, name, points):
        self.Templates.append(Template(name, points))  # append new template
        num = 0
        for temp in self.Templates:
            if temp.Name == name:
                num = num + 1
        return num

    def DeleteUserTemplates(self):
        self.Templates = self.Templates[0:NumTemplates]  # clear any beyond the original set
        return NumTemplates


# Helper functions from this point down
def Resample(points, n):
    I = PathLength(points) / (n - 1)  # interval length
    D = 0.0
    newpoints = [points[0]]
    for i in range(1, len(points)):
        d = Distance(points[i - 1], points[i])
        if (D + d) >= I:
            qx = points[i - 1][0] + ((I - D) / d) * (points[i][0] - points[i - 1][0])
            qy = points[i - 1][1] + ((I - D) / d) * (points[i][1] - points[i - 1][1])
            q = (qx, qy)
            newpoints.append(q)  # append new point 'q'
            points.insert(i, q)  # insert 'q' at position i in points s.t. 'q' will be the next i
            D = 0.0
        else:
            D += d

    # somtimes we fall a rounding-error short of adding the last point, so add it if so
    if len(newpoints) == n - 1:
        newpoints.append(points[-1])
    return newpoints


def BetterResample(points, n):
    distances, unique_path = PathDistancesExtractor(points, cumulative=True, unique=True, normalized=True).run()
    points = np.array(unique_path)
    alpha = np.linspace(0, 1, NumPoints)
    interpolator = interp1d(distances, unique_path, kind='slinear', axis=0)
    interpolated_points = interpolator(alpha)
    return interpolated_points.tolist()


def RotateToZero(points):
    c = Centroid(points)
    theta = math.atan2(c[1] - points[0][1], c[0] - points[0][0])
    return RotateBy(points, -theta)


def RotateBy(points, theta):
    c = Centroid(points)
    cos = math.cos(theta)
    sin = math.sin(theta)

    newpoints = []
    for p in points:
        qx = (p[0] - c[0]) * cos - (p[1] - c[1]) * sin + c[0]
        qy = (p[0] - c[0]) * sin + (p[1] - c[1]) * cos + c[1]
        newpoints.append((qx, qy))
    return newpoints


def ScaleToSquare(points, size):
    B = BoundingBox(points)
    newpoints = []
    for p in points:
        qx = p[0] * (size / B[1][0])
        if B[1][1] < 1e-2:
            qy = 0
        else:
            qy = p[1] * (size / B[1][1])
        newpoints.append((qx, qy))
    return newpoints


def TranslateToOrigin(points):
    c = Centroid(points)
    newpoints = []
    for p in points:
        qx = p[0] - c[0]
        qy = p[1] - c[1]
        newpoints.append((qx, qy))
    return newpoints


def DistanceAtBestAngle(points, T, a, b, threshold):
    x1 = Phi * a + (1.0 - Phi) * b
    f1 = DistanceAtAngle(points, T, x1)
    x2 = (1.0 - Phi) * a + Phi * b
    f2 = DistanceAtAngle(points, T, x2)
    while abs(b - a) > threshold:
        if f1 < f2:
            b = x2
            x2 = x1
            f2 = f1
            x1 = Phi * a + (1.0 - Phi) * b
            f1 = DistanceAtAngle(points, T, x1)
        else:
            a = x1
            x1 = x2
            f1 = f2
            x2 = (1.0 - Phi) * a + Phi * b
            f2 = DistanceAtAngle(points, T, x2)

    return min(f1, f2)


def DistanceAtAngle(points, T, theta):
    newpoints = RotateBy(points, theta)
    return PathDistance(newpoints, T.Points)


def Centroid(points):
    x = 0.0
    y = 0.0
    for p in points:
        x += p[0]
        y += p[1]

    x /= len(points)
    y /= len(points)
    return (x, y)


def BoundingBox(points):
    minX = float('infinity')
    maxX = float('-infinity')
    minY = float('infinity')
    maxY = float('-infinity')

    for p in points:
        if (p[0] < minX):
            minX = p[0]
        if (p[0] > maxX):
            maxX = p[0]
        if (p[1] < minY):
            minY = p[1]
        if (p[1] > maxY):
            maxY = p[1]

    # return [(X,Y),(width,height)]
    return [(minX, minY), (maxX - minX, maxY - minY)]


def PathDistance(pts1, pts2):
    d = 0.0
    min_length = min(len(pts1), len(pts2))
    for i in range(0, min_length):  # assumes pts1.length == pts2.length
        d += Distance(pts1[i], pts2[i])
    return d / min_length


def PathLength(points):
    d = 0.0
    for i in range(1, len(points)):
        d += Distance(points[i - 1], points[i])
    return d


def Distance(p1, p2):
    dx = p2[0] - p1[0]
    dy = p2[1] - p1[1]
    return math.sqrt(dx * dx + dy * dy)
