import os

import matplotlib
import numpy as np
import tensorflow as tf
from tensorflow.python.keras.models import load_model

from misc_util.imgproc_utils import demean_heatmap
from research.ablation_studies.sequence.video_streamer import VideoFrameStreamer

matplotlib.use('TkAgg')
import matplotlib.pyplot as plt


class GestureViewer:
    def __init__(self, video_file):
        model_weights = "/home/florin/diss_models/final_weights/kp/roi/effnet-b0/effnetb0_roi_u4_ff_ra9_best.178.h5"
        self.model = load_model(model_weights)
        self.image_counter = 0
        self.streamer = VideoFrameStreamer(video_file)

        self.current_frame = self.streamer.get()

        self.fig, self.axes = plt.subplots(ncols=2, nrows=1, squeeze=False)
        self.fig.tight_layout(pad=2.0)
        self.fig.canvas.mpl_connect('key_press_event', self._on_key)
        return

    def _on_key(self, event):
        if event.key == 'e':
            self._plot_batch()
        elif event.key == 'w':
            self._plot_batch()
        self.show()
        return

    def _plot_batch(self):
        preprocessed_frame = np.array(self.current_frame, np.float32) / 255.0
        # preprocessed_frame = tf.image.crop_to_bounding_box(preprocessed_frame, 140, 0, 568, 480)
        preprocessed_frame = tf.image.resize(preprocessed_frame, (224, 224))
        self.axes[0][0].imshow(preprocessed_frame)

        inference = self.model([preprocessed_frame])
        # output_heatmap = tf.image.convert_image_dtype(inference[0, :, :, 0], tf.uint8)
        output = inference[0, :, :, 0]
        filter_bin_hm = demean_heatmap(output)
        filter_bin_hm = filter_bin_hm / np.std(filter_bin_hm)
        mask_3sigma = np.where(filter_bin_hm < 3.0)
        result = filter_bin_hm.numpy()
        result[mask_3sigma] = 0

        self.axes[0][1].imshow(result)

        self.current_frame = self.streamer.get()
        return

    def _clear_axes(self):
        for ax in self.axes.flatten():
            ax.cla()
        return

    def show(self):
        self._clear_axes()
        self._plot_batch()
        plt.show()
        return


if __name__ == '__main__':
    video_dir = "/home/florin/data/gestures/"
    file_id = "2_tip_print_circle_0.mp4"

    viewer = GestureViewer(os.path.join(video_dir, file_id))
    viewer.show()
