import os

import matplotlib
import numpy as np
from tensorflow.python.keras.models import load_model

from research.ablation_studies.gestures.classify_gesture_videos import get_path_from_video_csv
from research.ablation_studies.gestures.dollar import Recognizer, RotateToZero, ScaleToSquare, \
    TranslateToOrigin, NumPoints, SquareSize, DistanceAtBestAngle, AngleRange, \
    AnglePrecision, HalfDiagonal
from research.ablation_studies.gestures.path_distances_extractor import PathDistancesExtractor

matplotlib.use('TkAgg')
import matplotlib.pyplot as plt

from scipy.interpolate import interp1d


def filter_duplicates_from_path(keypoint_path):
    last_kp = keypoint_path[0]
    result_path = [last_kp]
    for keypoint in keypoint_path[1:]:
        if np.count_nonzero(keypoint == last_kp) == 2:
            continue
        else:
            result_path.append(keypoint)
            last_kp = keypoint
    return result_path


def display_csv_file_path_analysis(full_csv_path, kp_type):
    raw_path = np.array(get_path_from_video_csv(full_csv_path, kp_type, 0.01))
    plt.plot(raw_path[:, 0], raw_path[:, 1])

    distances, unique_path = PathDistancesExtractor(raw_path, cumulative=True, unique=True, normalized=True).run()
    points = np.array(unique_path)
    plt.plot(points[:, 0], points[:, 1], 'gx')

    alpha = np.linspace(0, 1, NumPoints)
    interpolator = interp1d(distances, unique_path, kind='slinear', axis=0)
    interpolated_points = interpolator(alpha)

    dollar_points = RotateToZero(interpolated_points.tolist())
    dollar_points = ScaleToSquare(dollar_points, SquareSize)
    dollar_points = TranslateToOrigin(dollar_points)

    recognizer = Recognizer()

    b = float('infinity')
    for temp in recognizer.Templates:
        d = DistanceAtBestAngle(dollar_points, temp, -AngleRange, +AngleRange, AnglePrecision)
        if d < b:
            b = d
            t = temp

    score = 1.0 - (b / HalfDiagonal)
    print(t.Name)
    print(score)

    dollar_points = np.array(dollar_points)
    plt.plot(dollar_points[:, 0], dollar_points[:, 1], 'o')

    return


def vis_heatmaps_from_video(video_file):
    model_weights = "/home/florin/diss_models/final_weights/kp/roi/effnet-b0/effnetb0_roi_u4_ff_ra9_best.178.h5"
    my_model = load_model(model_weights)
    return


if __name__ == '__main__':
    csv_dir = "/home/florin/data/gestures/raw_csv/"

    file_id = "1_tip_print_circle_0"

    analysis_path = os.path.join(csv_dir, file_id + '.csv')

    display_csv_file_path_analysis(analysis_path, "tip")
