import numpy as np


class PathDistancesExtractor:
    '''
    points: column-major
    unique: if distances should be unique, e.g. no duplicate points
    '''

    def __init__(self, points: np.ndarray, cumulative=True, normalized=True, unique=True):
        self.points = points
        self.unique = unique
        self.cumulative = cumulative
        self.normalized = normalized

    def _get_linear_length_along_the_path(self, cumulative, normalized):
        if len(self.points) > 1:
            distances = np.sqrt(np.sum(np.diff(self.points, axis=0) ** 2, axis=1))
            if cumulative:
                distances = np.cumsum(distances)
            if normalized:
                distances = np.insert(distances, 0, 0) / distances[-1]
        else:
            return []
        return distances

    def _get_unique_distances_and_points(self, distance_array):
        distance, unique_indices = np.unique(distance_array, return_index=True)
        new_points = np.take(self.points, unique_indices, axis=0)
        return distance, new_points

    def run(self):
        distance_array = self._get_linear_length_along_the_path(self.cumulative, self.normalized)
        points = self.points
        if self.unique:
            distance_array, points = self._get_unique_distances_and_points(distance_array)
        return distance_array, points
