import os

import numpy as np
import tensorflow as tf

from research.ablation_studies.helpers import is_none_keypoint
from research.ablation_studies.train.image_transformer import ImageTransformer

IMG_SIZE = 224


def save_img(img, name, format="rgb"):
    uint8_img = tf.image.convert_image_dtype(img, dtype=tf.uint8)
    img_bytes = tf.io.encode_jpeg(uint8_img, format=format)
    tf.io.write_file(name, img_bytes)


def create_bboxes_keypoint(keypoint):
    boxes = tf.stack(
        [keypoint[1] / IMG_SIZE, keypoint[0] / IMG_SIZE, keypoint[1] / IMG_SIZE + 0.03, keypoint[0] / IMG_SIZE + 0.03],
        axis=-1)
    boxes = tf.reshape(boxes, shape=[1, 1, 4])
    return boxes


base_dir = "/home/florin/data/patches_224/train/"
ex_dir = "/home/florin/glowing-keypoint-firefly-dep/tmp_out/"

img1_path = "__4.jpg"
img2_path = "__83.jpg"
img3_path = "_0_119.jpg"
img4_path = "_1_2_340.jpg"
img5_path = "_2_751.jpg"

img1 = tf.io.decode_jpeg(tf.io.read_file(os.path.join(base_dir, img1_path)), channels=3)
img2 = tf.io.decode_jpeg(tf.io.read_file(os.path.join(base_dir, img2_path)), channels=3)
img3 = tf.io.decode_jpeg(tf.io.read_file(os.path.join(base_dir, img3_path)), channels=3)
img4 = tf.io.decode_jpeg(tf.io.read_file(os.path.join(base_dir, img4_path)), channels=3)
img5 = tf.io.decode_jpeg(tf.io.read_file(os.path.join(base_dir, img5_path)), channels=3)

kp1 = [173.5, 126.0]
kp2 = [87.5, 68.0]
kp3 = [134.5, 23.5]
kp4 = [-1.0, -1.0]
kp5 = [-1.0, -1.0]

colors = np.array([[1.0, 0.3, 0.3]])

imgs = [img1, img2, img3, img4, img5]
kps = [kp1, kp2, kp3, kp4, kp5]

# counter = 0
# for img, kp in zip(imgs, kps):
#    boxes = create_bboxes_keypoint(kp)

#    if not is_none_keypoint(kp):
#        kp_img = tf.image.draw_bounding_boxes([tf.image.convert_image_dtype(img, dtype=tf.float32)], boxes, colors=colors)[0]
#    else:
#        kp_img = img

#    save_img(kp_img, os.path.join(ex_dir, "{}_orig.jpg".format(counter)))
#    counter += 1

#
# only random crops and flips -> zero layers
#

# transformer = ImageTransformer(0, ImageTransformer.MAX_MAGNITUDE, image_dtype=tf.uint8, keypoint_row_major=False)
#
# counter = 0
# for img, kp in zip(imgs, kps):
#     aug_img, aug_kp = transformer.transform(tf.image.convert_image_dtype(img, dtype=tf.float32), kp)
#
#     boxes = create_bboxes_keypoint(aug_kp)
#
#     if not is_none_keypoint(aug_kp):
#         aug_img = tf.image.draw_bounding_boxes([aug_img], boxes, colors=colors)[0]
#     else:
#         print("Image {} has none keypoint.".format(counter))
#
#     save_img(aug_img, os.path.join(ex_dir, "{}_flip_rcrop.jpg".format(counter)))
#     counter += 1

#
# one layer, without flip and random crop
#


transformer = ImageTransformer(2, 9, image_dtype=tf.uint8, keypoint_row_major=False)

counter = 0
for img, kp in zip(imgs, kps):
    aug_img, aug_kp = transformer.transform(tf.image.convert_image_dtype(img, dtype=tf.float32), tf.constant(kp),
                                            do_preprocessing=True)

    boxes = create_bboxes_keypoint(aug_kp)

    if not is_none_keypoint(aug_kp):
        aug_img = tf.image.draw_bounding_boxes([aug_img], boxes, colors=colors)[0]
        hm_img = transformer.create_heatmap(211, 211, aug_kp, 3)
        tf.print(tf.shape(hm_img))
        save_img(hm_img, os.path.join(ex_dir, "{}_kp.jpg".format(counter)), format="grayscale")
    else:
        print("Image {} has none keypoint.".format(counter))

    save_img(aug_img, os.path.join(ex_dir, "{}_one_layer.jpg".format(counter)))
    counter += 1
