import os

from research.ablation_studies.dataset.dataset_utils import sequence_image_feature_description, \
    prepare_dataset, clone_sequence_example_for_disk, write_tfrecord_with_examples

examples_to_fix = {
    0: [(4, 620, 1)],
    1: [(2, 513, 0), (2, 514, 0), (2, 531, 1)],
    2: [(1, 495, -1), (1, 514, 1)],
    3: [(1, 369, 1), (1, 378, 1), (1, 739, 1)],
    5: [(2, 270, 1), (2, 271, 1), (2, 282, 1)],
    7: [(4, 83, 1), (4, 86, 1), (4, 993, 1), (4, 1008, 1)],
    8: [(2, 674, 1)],
    9: [(2, 164, 1), (2, 177, 1), (2, 184, 0), (2, 1390, 1)],
    10: [(2, 917, 1)],
    12: [(1, 1149, 1)],
    13: [(3, 139, 1), (3, 900, 1)],
    14: [(4, 1061, 1)],
    15: [(2, 754, 1), (2, 778, 1)],
    18: [(2, 99, 0), (2, 100, 0)],
    20: [(1, 129, 1), (1, 907, 1), (1, 936, 1)],
    21: [(3, 292, 1), (3, 293, 1), (3, 535, 1), (3, 856, 1)],
    22: [(3, 1020, 1)],
    24: [(4, 1629, 1), (4, 1630, 1)],
    25: [(2, 801, 1), (2, 802, 1)],
    26: [(4, 383, 1), (4, 423, 0)],
    27: [(1, 180, 1)],
    29: [(2, 133, 1), (2, 1146, 1)],
    30: [(1, 343, 1)],
    31: [(2, 668, 1), (2, 669, 1), (2, 1376, 1)],
    32: [(3, 276, 1), (3, 292, 1), (3, 747, 1), (3, 1081, 1)],
    35: [(4, 1200, 1)],
    36: [(1, 992, 1)],
    38: [(3, 184, 1)],
}


def find_sequence_file(subject, task, sequence_dir):
    files = os.listdir(sequence_dir)
    sequence_files = [file for file in files if '.tfrecord' in file]
    result = None
    for file in sequence_files:
        if "_s" + str(subject) + "_" in file:
            if "_t" + str(task) + "_" in file:
                result = file
                break
    return result


def fix_examples_up_down(sequence_dir, examples_to_fix):
    for example_subject in examples_to_fix:
        example_task = examples_to_fix[example_subject][0][0]
        sequence_file = find_sequence_file(example_subject, example_task, sequence_dir)
        if sequence_file is not None:
            fixed_examples = []
            dataset_in = iter(
                prepare_dataset([os.path.join(sequence_dir, sequence_file)], sequence_image_feature_description))

            for item in dataset_in:
                frame_id = int(item['image/frame_id'])
                fix_frames = examples_to_fix[example_subject]
                for frame in fix_frames:
                    if frame[1] == frame_id:
                        if frame[2] == -1:
                            item['image/ptip_x'] = -1.0
                            item['image/ptip_y'] = -1.0
                        else:
                            item['image/pen_down'] = frame[2]
                        print("Changed frame {} for example {}".format(int(item['image/frame_id']), frame))

                fixed_examples.append(clone_sequence_example_for_disk(item))

            tfrecord_filename = "/home/florin/data/sequence/fixed/merged_s{}_t{}_sequences.tfrecord".format(
                example_subject, example_task)
            print("Writing tfrecord {}".format(tfrecord_filename))
            write_tfrecord_with_examples(tfrecord_filename, fixed_examples)
    return


if __name__ == '__main__':
    tfrecord_dir = "/home/florin/data/sequence/"
    fix_examples_up_down(tfrecord_dir, examples_to_fix)
