import argparse

from research.ablation_studies.dataset.dataset_utils import prepare_dataset, sequence_image_feature_description, \
    write_tfrecord_with_examples, clone_sequence_example_for_disk


def parse_args():
    parser = argparse.ArgumentParser(
        description="Merge sequence annotations for single subjects. First gets overwritten with second.")
    parser.add_argument('--in1', help='Base sequence annotations.')
    parser.add_argument('--in2', help='New annotations that overwrite the ones in in1')
    parser.add_argument('--outfile', help='Output file.')
    return parser.parse_args()


def keypoints_equal(example1, example2):
    if example1['image/ptip_x'] == example2['image/ptip_x']:
        if example1['image/ptip_y'] == example2['image/ptip_y']:
            return True
    return False


def updown_equal(example1, example2):
    if example1['image/pen_down'] == example2['image/pen_down']:
        return True
    return False


def copy_second_example_over_first(example1, example2):

    example1['image/pen_down'] = example2['image/pen_down']
    example1['image/ptip_x'] = example2['image/ptip_x']
    example1['image/ptip_y'] = example2['image/ptip_y']
    return


if __name__ == '__main__':
    subject = 7
    task = 0

    in1 = "/home/florin/glowing-keypoint-firefly-dep/research/ablation_studies/sequence/val1_s{}_t{}_sequences.tfrecord".format(
        subject, task)
    # in1 = "/home/florin/data/val1/sequence/merged/merged_s{}_t{}_sequences.tfrecord".format(subject, task)
    in2 = "/home/florin/data/val1/sequence/val1_s{}_t{}_sequences.tfrecord".format(subject, task)

    dataset_in1 = iter(prepare_dataset(in1, sequence_image_feature_description))
    dataset_in2 = iter(prepare_dataset(in2, sequence_image_feature_description))

    finished = False
    merged_examples = []

    current_in2 = next(dataset_in2)

    current1_counter = 0
    current2_counter = 0

    for current_in1 in dataset_in1:
        if current_in2 is not None:
            if current_in1['image/task_id'] == current_in2['image/task_id']:
                if current_in1['image/sequence_id'] == current_in2['image/sequence_id']:
                    if current_in1['image/frame_id'] == current_in2['image/frame_id']:
                        if keypoints_equal(current_in1, current_in2) and updown_equal(current_in1, current_in2):
                            current_in2 = next(dataset_in2, None)
                            current2_counter += 1
                        else:
                            copy_second_example_over_first(current_in1, current_in2)
                            current_in2 = next(dataset_in2, None)
                            current2_counter += 1
        merged_examples.append(clone_sequence_example_for_disk(current_in1))
        current1_counter += 1

    print("In1: {} elements / In2: {} elements".format(current1_counter, current2_counter))
    print("Merged {} examples.".format(len(merged_examples)))
    tfrecord_filename = "/home/florin/data/val1/sequence/merged/merged_s{}_t{}_sequences.tfrecord".format(subject, task)
    print("Writing tfrecord {}".format(tfrecord_filename))
    write_tfrecord_with_examples(tfrecord_filename, merged_examples)
