import cv2


class VideoFrameStreamer:
    def __init__(self, filename):
        self.filename = filename
        self.capture = cv2.VideoCapture()
        self.capture.open(filename)
        self._check_capture()

    def _check_capture(self):
        if not self.capture.isOpened():
            print("Couldn't open camera capture.")
            raise IOError
        return

    def get(self):
        ret, frame = self.capture.read()
        if not ret:
            print("No frame received")
            return None
        return cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)

    def seek_to_and_get(self, index):
        self.capture.set(cv2.CAP_PROP_POS_FRAMES, index)
        return self.get()

    def get_frame_count(self):
        return self.capture.get(cv2.CAP_PROP_FRAME_COUNT)

    def get_fps(self):
        return self.capture.get(cv2.CAP_PROP_FPS)

    def __del__(self):
        self.capture.release()
