import pytest
import tensorflow as tf
from numpy import random
from skimage.transform import resize

from misc_util.imgproc_utils import resize_image_array_pil_antialias


def resize_image_scikit(image, to_width, to_height):
    resize(image, (to_height, to_width), anti_aliasing=True, order=1)
    return


def resize_image_pil(image, to_width, to_height):
    resize_image_array_pil_antialias(image, (to_width, to_height))
    return


def resize_image_tf(image, to_width, to_height):
    tf.image.resize(image, (to_height, to_width), antialias=True)
    return


@pytest.mark.parametrize("from_width,from_height,to_width,to_height",
                         [(3840, 2560, 224, 224), (1920, 1080, 224, 224), (1280, 720, 224, 224)])
def test_resize_image(benchmark, from_width, from_height, to_width, to_height):
    image = (random.random((from_height, from_width, 3)) * 255).astype('uint8')
    result = benchmark(resize_image_scikit, image, to_width, to_height)


@pytest.mark.parametrize("from_width,from_height,to_width,to_height",
                         [(3840, 2560, 224, 224), (1920, 1080, 224, 224), (1280, 720, 224, 224)])
def test_resize_image_pil(benchmark, from_width, from_height, to_width, to_height):
    image = (random.random((from_height, from_width, 3)) * 255).astype('uint8')
    result = benchmark(resize_image_pil, image, to_width, to_height)


@pytest.mark.parametrize("from_width,from_height,to_width,to_height",
                         [(3840, 2560, 224, 224), (1920, 1080, 224, 224), (1280, 720, 224, 224)])
def test_resize_image_tf(benchmark, from_width, from_height, to_width, to_height):
    image = (random.random((from_height, from_width, 3)) * 255).astype('uint8')
    result = benchmark(resize_image_tf, image, to_width, to_height)
