import tensorflow as tf

from research.ablation_studies.train.keypoint_tf import KeypointHelper


# images: uint8
# image values: uint8
# keypoints: float32

class KeypointHelperTest(tf.test.TestCase):
    def setUp(self):
        self.none_kp = tf.stack([-1.0, -1.0])
        # seed 1: negate tensor does nothing
        tf.random.set_seed(1)

        # tf.compat.v1.disable_eager_execution()
        return

    def test_row_major(self):
        helper = KeypointHelper(row_major=True)
        tf.assert_equal(helper.is_row_major(), True)
        return

    def test_col_major(self):
        helper = KeypointHelper(row_major=False)
        tf.assert_equal(helper.is_row_major(), False)
        return

    def test_get_x_row(self):
        helper = KeypointHelper(row_major=True)
        kp = tf.stack([100.0, 0.0])

        tf.assert_equal(helper.get_x(kp), 0.0)
        return

    def test_get_x_col(self):
        helper = KeypointHelper(row_major=False)
        kp = tf.stack([100.0, 0.0])

        tf.assert_equal(helper.get_x(kp), 100.0)
        return

    def test_get_y_row(self):
        helper = KeypointHelper(row_major=True)
        kp = tf.stack([100.0, 0.0])

        tf.assert_equal(helper.get_y(kp), 100.0)
        return

    def test_get_y_col(self):
        helper = KeypointHelper(row_major=False)
        kp = tf.stack([100.0, 0.0])

        tf.assert_equal(helper.get_y(kp), 0.0)
        return

    def test_set_x_row(self):
        helper = KeypointHelper(row_major=True)
        kp = tf.stack([100.0, 0.0])
        kp = helper.set_x(kp, 300.0)

        tf.assert_equal(helper.get_x(kp), 300.0)
        return

    def test_set_x_col(self):
        helper = KeypointHelper(row_major=False)
        kp = tf.stack([100.0, 0.0])
        kp = helper.set_x(kp, 300.0)

        tf.assert_equal(helper.get_x(kp), 300.0)
        return

    def test_set_y_row(self):
        helper = KeypointHelper(row_major=True)
        kp = tf.stack([100.0, 0.0])
        kp = helper.set_y(kp, 300.0)

        tf.assert_equal(helper.get_y(kp), 300.0)
        return

    def test_set_y_col(self):
        helper = KeypointHelper(row_major=False)
        kp = tf.stack([100.0, 0.0])
        kp = helper.set_y(kp, 300.0)

        tf.assert_equal(helper.get_y(kp), 300.0)
        return
