import os

import numpy as np
import pytest
import tensorflow as tf
from tensorflow.python.keras.models import load_model


def do_roi_inference(model, image):
    output = model([image], training=False)
    return


@pytest.mark.benchmark(
    warmup=True,
    warmup_iterations=100,
    min_time=2.0,
)
def test_do_roi_inference(benchmark):
    model_weights = "/home/florin/diss_models/final_weights/kp/roi/effnet-b0/effnetb0_roi_u4_ff_ra9_best.178.h5"
    image_file = "/home/florin/data/patches_224/test/0_3_261.jpg"
    tf.config.optimizer.set_jit(True)
    my_model = load_model(model_weights)
    my_model.trainable = False
    image = tf.io.read_file(image_file)
    image = tf.image.decode_jpeg(image)
    image = tf.image.convert_image_dtype(image, tf.float32)
    # image = random.random((224, 224, 3))
    result = benchmark(do_roi_inference, my_model, image)


@pytest.mark.benchmark(
    warmup=True,
    warmup_iterations=100,
    min_time=2.0,
)
def test_do_roi_inference_trt(benchmark):
    model_weights_dir = "/home/florin/diss_models/trt_sm/roi_trt/"
    image_file = "/home/florin/data/patches_224/test/0_3_261.jpg"

    my_model = load_model(model_weights_dir)
    my_model.trainable = False
    image = tf.io.read_file(image_file)
    image = tf.image.decode_jpeg(image)
    image = tf.image.convert_image_dtype(image, tf.float32)
    # image = random.random((224, 224, 3))
    result = benchmark(do_roi_inference, my_model, image)


@pytest.mark.benchmark(
    warmup=True,
    warmup_iterations=100,
    min_time=2.0,
)
def test_do_detail_inference(benchmark):
    model_weights = "/home/florin/diss_models/final_weights/kp/detail/effnet-b0/effnetb0_detail_u4_ff_ra9_best.187.h5"
    image_file = "/home/florin/data/patches_224/test/0_3_261.jpg"
    tf.config.optimizer.set_jit(True)
    my_model = load_model(model_weights)
    my_model.trainable = False
    image = tf.io.read_file(image_file)
    image = tf.image.decode_jpeg(image)
    image = tf.image.convert_image_dtype(image, tf.float32)
    result = benchmark(do_roi_inference, my_model, image)


@pytest.mark.benchmark(
    warmup=True,
    warmup_iterations=100,
    min_time=2.0,
)
def test_do_detail_inference_trt(benchmark):
    model_weights_dir = "/home/florin/diss_models/trt_sm/detail_trt/"
    image_file = "/home/florin/data/patches_224/test/0_3_261.jpg"

    my_model = load_model(model_weights_dir)
    my_model.trainable = False
    image = tf.io.read_file(image_file)
    image = tf.image.decode_jpeg(image)
    image = tf.image.convert_image_dtype(image, tf.float32)
    # image = random.random((224, 224, 3))
    result = benchmark(do_roi_inference, my_model, image)


@pytest.mark.benchmark(
    warmup=True,
    warmup_iterations=100,
    min_time=2.0,
)
def test_do_text_ex_inference(benchmark):
    model_weights = "/home/florin/diss_models/effnetb0_text_ex_lorem/correct_pair/1_chkpt/correct_pair_best.156.h5"
    image_file = "/home/florin/data/patches_224/test/0_3_261.jpg"
    tf.config.optimizer.set_jit(True)
    my_model = load_model(model_weights)
    my_model.trainable = False
    image = tf.io.read_file(image_file)
    image = tf.image.decode_jpeg(image)
    image = tf.image.convert_image_dtype(image, tf.float32)
    result = benchmark(do_roi_inference, my_model, image)


@pytest.mark.benchmark(
    warmup=True,
    warmup_iterations=100,
    min_time=2.0,
)
def test_do_text_ex_inference_trt(benchmark):
    model_weights_dir = "/home/florin/diss_models/trt_sm/text_trt/"
    image_file = "/home/florin/data/patches_224/test/0_3_261.jpg"

    my_model = load_model(model_weights_dir)
    my_model.trainable = False
    image = tf.io.read_file(image_file)
    image = tf.image.decode_jpeg(image)
    image = tf.image.convert_image_dtype(image, tf.float32)
    # image = random.random((224, 224, 3))
    result = benchmark(do_roi_inference, my_model, image)


@pytest.mark.benchmark(
    warmup=True,
    warmup_iterations=100,
    min_time=2.0,
)
def test_do_sequence_inference(benchmark):
    model_weights = "/home/florin/diss_models/effnetb0_classify_rnn_diss_conv/rnn_ab2_ts30_shift30_convlstm64_7dtbnf_normlr3_aug_adam/1_chkpt/rnn_ab2_ts30_shift30_convlstm64_7dtbnf_normlr3_aug_adam_best.002.h5"
    tf.config.optimizer.set_jit(True)
    my_model = load_model(model_weights)
    my_model.trainable = False

    image_dir = "/home/florin/data/patches_224/test/"
    image_files = os.listdir(image_dir)
    image_files = [file for file in image_files if '.jpg' in file]

    my_sequence = image_files[:30]
    my_input = np.zeros(shape=(1, 30, 224, 224, 3))
    for index, image_file in enumerate(my_sequence):
        image = tf.io.read_file(os.path.join(image_dir, image_file))
        image = tf.image.decode_jpeg(image)
        image = tf.image.convert_image_dtype(image, tf.float32)
        my_input[0, index, :, :, :] = image

    result = benchmark(do_roi_inference, my_model, my_input)


@pytest.mark.benchmark(
    warmup=True,
    warmup_iterations=100,
    min_time=2.0,
)
def test_do_sequence_inference_trt(benchmark):
    model_weights_dir = "/home/florin/diss_models/trt_sm/sequence_trt/"

    my_model = load_model(model_weights_dir)
    my_model.trainable = False

    image_dir = "/home/florin/data/patches_224/test/"
    image_files = os.listdir(image_dir)
    image_files = [file for file in image_files if '.jpg' in file]

    my_sequence = image_files[:30]
    my_input = np.zeros(shape=(1, 30, 224, 224, 3))
    for index, image_file in enumerate(my_sequence):
        image = tf.io.read_file(os.path.join(image_dir, image_file))
        image = tf.image.decode_jpeg(image)
        image = tf.image.convert_image_dtype(image, tf.float32)
        my_input[0, index, :, :, :] = image

    result = benchmark(do_roi_inference, my_model, my_input)
