import os

import numpy as np
import tensorflow as tf
from PIL import Image
from natsort import natsorted

from research.ablation_studies.dataset.dataset_utils import bin_set_pair_to_tfexample, write_tfrecord_with_examples
from research.ablation_studies.text_ex.text_ex_helpers import write_loremipsum_on_image


def write_dataset_tfrecord(counter, examples, out_dir, use_lorem=False):
    postfix = "lorem_" if use_lorem else ""
    tfrecord_filename = "text_ex_set_{}{}.tfrecord".format(postfix, counter)
    tfrecord_path = os.path.join(out_dir, tfrecord_filename)
    print("Writing text_ex_set tfrecord {}".format(tfrecord_filename))
    write_tfrecord_with_examples(tfrecord_path, examples)


def create_binary_text_tfrecords(use_lorem=False):
    image_dir = "/home/florin/data/bin_patches/set/"
    bin_dir = "/home/florin/data/bin_patches/bin/"
    out_dir = "/home/florin/data/bin_patches/"

    img_files = os.listdir(image_dir)
    img_files = [os.path.join(image_dir, file) for file in img_files if '.jpg' in file]
    img_files = natsorted(img_files)
    bin_files = os.listdir(bin_dir)
    bin_files = [os.path.join(bin_dir, file) for file in bin_files if '.jpg' in file]
    bin_files = natsorted(bin_files)

    assert (len(img_files) == len(bin_files))

    counter = 0
    file_counter = 0
    interval = 500
    examples = []

    for img_file, bin_file in zip(img_files, bin_files):
        if use_lorem:
            orig_img = Image.open(img_file)
            orig_img_with_text = write_loremipsum_on_image(orig_img, randomize=True)
            orig_img_with_text = np.array(orig_img_with_text)
            store_img = tf.image.encode_jpeg(orig_img_with_text, chroma_downsampling=False)
        else:
            store_img = tf.io.read_file(img_file)

        tf_bin = tf.io.read_file(bin_file)

        if counter < len(img_files) // 5:
            subject_id = 0
        else:
            subject_id = 1

        example = bin_set_pair_to_tfexample(counter,
                                            subject_id,
                                            store_img,
                                            tf_bin,
                                            224,
                                            224)
        examples.append(example)

        if (counter % interval == 0) and counter > 0:
            write_dataset_tfrecord(file_counter, examples, out_dir, use_lorem=use_lorem)
            examples.clear()
            file_counter += 1

        counter += 1

    if len(examples) > 0:
        write_dataset_tfrecord(file_counter, examples, out_dir, use_lorem=use_lorem)
        examples.clear()

    return


if __name__ == '__main__':
    create_binary_text_tfrecords(use_lorem=True)
