import os

from research.ablation_studies.dataset.dataset_utils import gather_tfrecords, bin_set_pair_feature_description
from research.ablation_studies.dataset.pipeline_creator import PipelineCreator
from research.ablation_studies.model.create_keypoint_models import get_preprocess_func, \
    create_model_effnet_b0, compile_created_model_keypoint
from research.ablation_studies.text_ex.bin_pair_input_map import BinPairInputMap
from research.ablation_studies.train.model_keypoint_trainer import ModelKeypointTrainer

if __name__ == '__main__':
    arch = "effnetb0"
    config_id = "{}_text_ex_lorem".format(arch)
    train_prefix_id = "correct_pair"
    constant_lr = None
    initial_lr = 1e-3
    # fine_tune_weights = "/home/florin/diss_models/effnetb0_classify_final/agnostic_no_xdense_weighted_ls0.1_m9_frozen/1_chkpt/agnostic_no_xdense_weighted_ls0.1_m9_frozen_best.003.h5"
    effnet_trainable = True

    magnitude = 6
    batch_size = 32

    input_shape = (224, 224, 3)

    tfrecord_dir = "/home/florin/data/bin_patches/"
    tfrecords = gather_tfrecords(tfrecord_dir=tfrecord_dir)

    # get datasets train/test
    input_mapper = BinPairInputMap(
        input_shape,
        (211, 211, 1),
        magnitude,
        input_prep_func=get_preprocess_func(arch))

    #
    pipeline = PipelineCreator(
        input_mapper,
        batch_size,
        tfrecords,
        image_features=bin_set_pair_feature_description,
        test_subjects=[0])
    ds_train, ds_test = pipeline.get_datasets()

    # create model
    effnet_model = create_model_effnet_b0(
        input_shape=input_shape,
        output_shape=(211, 211, 1),
        upsampling_layers=4,
        freeze=False)

    my_model = compile_created_model_keypoint(effnet_model)

    model_save_path = "/home/florin/diss_models/{}/{}/1/".format(config_id, train_prefix_id)
    os.makedirs(model_save_path, exist_ok=True)
    print("Storing model in {}".format(model_save_path))

    checkpoint_save_path = "/home/florin/diss_models/{}/{}/1_chkpt/".format(config_id, train_prefix_id)
    os.makedirs(checkpoint_save_path, exist_ok=True)
    print("Storing checkpoints in {}".format(checkpoint_save_path))

    trainer = ModelKeypointTrainer(
        ds_train,
        ds_test,
        batch_size,
        my_model,
        train_prefix_id,
        checkpoint_save_path,
        test_count=2140,  # 1070
        train_count=8562,  # 4281
        save_regular_checkpoints=False)

    print("Starting training!")
    trainer.train(200)

    print("Saving model ...")

    my_model.save(model_save_path, save_format='tf')
