class TrainConfig:
    configs = {
        "nasnetmobile": {
            "stages": {
                "detail": {
                    "batch_size": 32,
                    "input_shape": (224, 224, 3),
                    "ra_magnitude": 9,
                    "epochs": 200,
                    "downscale_ratio": 1,
                    "runs": [
                        {
                            "id": "detail_u2_ft_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (204, 204, 1),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "detail_u4_ft_ra9",
                            "upsampling_layers": 4,
                            "output_shape": (211, 211, 1),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "detail_u2_ff_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (204, 204, 1),
                            "freeze": False,  # freeze 20% of layers
                        },
                        {
                            "id": "detail_u4_ff_ra9",
                            "upsampling_layers": 4,
                            "output_shape": (211, 211, 1),
                            "freeze": False,  # freeze 20% of layers
                        }
                    ]
                },
                "roi": {
                    "batch_size": 32,
                    "input_shape": (224, 224, 3),
                    "ra_magnitude": 9,
                    "epochs": 200,
                    "downscale_ratio": 4,
                    "runs": [
                        {
                            "id": "roi_u2_ft_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (204, 204, 2),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "roi_u4_ft_ra9",
                            "upsampling_layers": 4,
                            "output_shape": (211, 211, 2),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "roi_u2_ff_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (204, 204, 2),
                            "freeze": False,  # freeze 20% of layers
                        },
                        {
                            "id": "roi_u4_ff_ra9",
                            "upsampling_layers": 4,
                            "output_shape": (211, 211, 2),
                            "freeze": False,  # freeze 20% of layers
                        }
                    ]
                }
            }
        },
        "effnetb0": {
            "stages": {
                "detail": {
                    "batch_size": 32,
                    "input_shape": (224, 224, 3),
                    "ra_magnitude": 9,
                    "epochs": 200,
                    "downscale_ratio": 1,
                    "runs": [
                        {
                            "id": "detail_u2_ft_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (204, 204, 1),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "detail_u4_ft_ra9",
                            "upsampling_layers": 4,
                            "output_shape": (211, 211, 1),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "detail_u2_ff_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (204, 204, 1),
                            "freeze": False,  # freeze 20% of layers
                        },
                        {
                            "id": "detail_u4_ff_ra9",
                            "upsampling_layers": 4,
                            "output_shape": (211, 211, 1),
                            "freeze": False,  # freeze 20% of layers
                        }
                    ]
                },
                "roi": {
                    "batch_size": 32,
                    "input_shape": (224, 224, 3),
                    "ra_magnitude": 9,
                    "epochs": 200,
                    "downscale_ratio": 4,
                    "runs": [
                        {
                            "id": "roi_u2_ft_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (204, 204, 2),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "roi_u4_ft_ra9",
                            "upsampling_layers": 4,
                            "output_shape": (211, 211, 2),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "roi_u2_ff_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (204, 204, 2),
                            "freeze": False,  # freeze 20% of layers
                        },
                        {
                            "id": "roi_u4_ff_ra9",
                            "upsampling_layers": 4,
                            "output_shape": (211, 211, 2),
                            "freeze": False,  # freeze 20% of layers
                        }
                    ]
                }
            }
        },
        "effnetb5": {
            "stages": {
                "detail": {
                    "batch_size": 4,
                    "input_shape": (456, 456, 3),
                    "ra_magnitude": 9,
                    "epochs": 200,
                    "downscale_ratio": 1,
                    "runs": [
                        {
                            "id": "detail_u2_ft_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (396, 396, 1),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "detail_u3_ft_ra9",
                            "upsampling_layers": 3,
                            "output_shape": (444, 444, 1),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "detail_u2_ff_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (396, 396, 1),
                            "freeze": False,  # freeze 20% of layers
                        },
                        {
                            "id": "detail_u3_ff_ra9",
                            "upsampling_layers": 3,
                            "output_shape": (444, 444, 1),
                            "freeze": False,  # freeze 20% of layers
                        }
                    ]
                },
                "roi": {
                    "batch_size": 4,
                    "input_shape": (456, 456, 3),
                    "ra_magnitude": 9,
                    "epochs": 200,
                    "downscale_ratio": 2,
                    "runs": [
                        {
                            "id": "roi_u2_ft_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (396, 396, 2),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "roi_u3_ft_ra9",
                            "upsampling_layers": 3,
                            "output_shape": (444, 444, 2),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "roi_u2_ff_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (396, 396, 2),
                            "freeze": False,  # freeze 20% of layers
                        },
                        {
                            "id": "roi_u3_ff_ra9",
                            "upsampling_layers": 3,
                            "output_shape": (444, 444, 2),
                            "freeze": False,  # freeze 20% of layers
                        }
                    ]
                }
            }
        },
        "resnetv2": {
            "stages": {
                "detail": {
                    "batch_size": 32,
                    "input_shape": (224, 224, 3),
                    "ra_magnitude": 9,
                    "epochs": 200,
                    "downscale_ratio": 1,
                    "runs": [
                        {
                            "id": "detail_u2_ft_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (204, 204, 1),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "detail_u4_ft_ra9",
                            "upsampling_layers": 4,
                            "output_shape": (211, 211, 1),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "detail_u2_ff_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (204, 204, 1),
                            "freeze": False,  # freeze 20% of layers
                        },
                        {
                            "id": "detail_u4_ff_ra9",
                            "upsampling_layers": 4,
                            "output_shape": (211, 211, 1),
                            "freeze": False,  # freeze 20% of layers
                        }
                    ]
                },
                "roi": {
                    "batch_size": 32,
                    "input_shape": (224, 224, 3),
                    "ra_magnitude": 9,
                    "epochs": 200,
                    "downscale_ratio": 4,
                    "runs": [
                        {
                            "id": "roi_u2_ft_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (204, 204, 2),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "roi_u4_ft_ra9",
                            "upsampling_layers": 4,
                            "output_shape": (211, 211, 2),
                            "freeze": True,  # freeze 20% of layers
                        },
                        {
                            "id": "roi_u2_ff_ra9",
                            "upsampling_layers": 2,
                            "output_shape": (204, 204, 2),
                            "freeze": False,  # freeze 20% of layers
                        },
                        {
                            "id": "roi_u4_ff_ra9",
                            "upsampling_layers": 4,
                            "output_shape": (211, 211, 2),
                            "freeze": False,  # freeze 20% of layers
                        }
                    ]
                }
            }
        },
        "resnetv1": {
            "stages": {
                "roi": {
                    "batch_size": 32,
                    "input_shape": (224, 224, 3),
                    "ra_magnitude": 9,
                    "epochs": 200,
                    "downscale_ratio": 4,
                    "runs": [
                        # {
                        #     "id": "roi_u4_ff_ra9",
                        #     "upsampling_layers": 4,
                        #     "output_shape": (211, 211, 2),
                        #     "freeze": False,  # freeze 20% of layers
                        # }
                    ]
                }
            }
        }
    }

    @staticmethod
    def get_id_run(run_id):
        for arch in TrainConfig.configs:
            for stage in TrainConfig.configs[arch]["stages"]:
                for run in TrainConfig.configs[arch]["stages"][stage]["runs"]:
                    if run_id == arch + "_" + run["id"]:
                        return arch, stage, run
        return None, None, None

    @staticmethod
    def get_run_input_shape(run_id):
        arch, stage, run = TrainConfig.get_id_run(run_id)
        if arch is not None:
            return TrainConfig.configs[arch]["stages"][stage]["input_shape"]
        else:
            return None

    @staticmethod
    def get_run_output_shape(run_id):
        arch, stage, run = TrainConfig.get_id_run(run_id)
        if arch is not None:
            return run["output_shape"]
        else:
            return None

    @staticmethod
    def get_run_upsampling_layers(run_id):
        arch, stage, run = TrainConfig.get_id_run(run_id)
        if arch is not None:
            return run["upsampling_layers"]
        else:
            return None

    @staticmethod
    def get_run_freeze(run_id):
        arch, stage, run = TrainConfig.get_id_run(run_id)
        if arch is not None:
            return run["freeze"]
        else:
            return None
