import matplotlib.pyplot as plt
import tikzplotlib


def setup_plot_ax(ax, xlim, xlabel, ylabel, title="F-LSTM at 3 Timesteps"):
    # ax.set_yscale("log")
    ax.set(xlim=(-0.01, xlim))

    ax.set_title(title)
    ax.spines["top"].set_visible(False)
    # ax.spines["bottom"].set_visible(False)
    ax.spines["right"].set_visible(False)
    ax.spines["left"].set_visible(False)

    plt.xticks(fontsize=12, color="black", alpha=0.7)
    plt.yticks(fontsize=12, color="black", alpha=0.7)

    ax.set_xlabel(xlabel, labelpad=8, alpha=0.6)
    ax.set_ylabel(ylabel, labelpad=8, alpha=0.6)

    ax.yaxis.grid(b=True, which='major', color='lightgray', linestyle='solid')
    ax.yaxis.grid(b=True, which='minor', color='lightgray', linestyle='dotted')


def plot_classification_layer_size():
    flstm_3_sizes = [32, 64, 128, 256, 512]
    flstm_3_loss = [0.5707, 0.5551, 0.5594, 0.5601, 0.5688]
    gru_3_loss = [0.5761, 0.5522, 0.5657, 0.5608, 0.5725]

    fig, axes = plt.subplots(nrows=1, ncols=1, squeeze=False)
    ax = axes[0][0]
    setup_plot_ax(ax, xlim=544, xlabel="Layer Size", ylabel="Loss", title="Loss vs. RNN Layer Sizes")

    ax.set_xticks(flstm_3_sizes)

    ax.plot([0, 544], [0.691, 0.691], lw=2, alpha=.8, color='slategray', label='Naive')
    ax.plot([0, 544], [0.6135, 0.6135], lw=2, alpha=.8, color='crimson', label='AB-2')

    ax.plot(flstm_3_sizes, flstm_3_loss, 'o-', lw=2, alpha=.8, color='cornflowerblue', label='F-LSTM')
    ax.plot(flstm_3_sizes, gru_3_loss, 'o-', lw=2, alpha=.8, color='orange', label='F-GRU')
    ax.legend()


def plot_classification_timesteps():
    flstm_64_timesteps = [3, 10, 30, 60]
    flstm_64_loss = [0.5551, 0.5613, 0.5739, 0.5686]
    fgru_64_loss = [0.5522, 0.5646, 0.5649, 0.5709]

    fig, axes = plt.subplots(nrows=1, ncols=1, squeeze=False)
    ax = axes[0][0]
    setup_plot_ax(ax, xlim=63, xlabel="Sequence Length", ylabel="Loss", title="Loss vs. RNN Sequence Length")

    ax.set_xticks(flstm_64_timesteps)

    ax.plot([0, 63], [0.691, 0.691], lw=2, alpha=.8, color='slategray', label='Naive')
    ax.plot([0, 63], [0.6135, 0.6135], lw=2, alpha=.8, color='crimson', label='AB-2')

    ax.plot(flstm_64_timesteps, flstm_64_loss, 'o-', lw=2, alpha=.8, color='cornflowerblue', label='F-LSTM')
    ax.plot(flstm_64_timesteps, fgru_64_loss, 'o-', lw=2, alpha=.8, color='orange', label='F-GRU')
    ax.legend()
    return


if __name__ == '__main__':
    plot_classification_timesteps()
    tikzplotlib.save("plot_classification_analysis_f-lstm-timesteps.tex")
    plt.show()
