import matplotlib

matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
# import tikzplotlib
from matplotlib.ticker import FuncFormatter


def setup_lr_plot_ax(ax, epochs=200, title="Learning Rate Schedule"):
    ax.set_yscale("log")
    ax.set(xlim=(-0.01, epochs))

    ax.set_title(title)
    ax.spines["top"].set_visible(False)
    ax.spines["bottom"].set_visible(False)
    ax.spines["right"].set_visible(False)
    ax.spines["left"].set_visible(False)

    ax.set_xlabel('Epochs', labelpad=8, alpha=0.6)
    ax.set_ylabel('Learning Rate', labelpad=8, alpha=0.6)

    formatter = FuncFormatter(lambda y, _: '{:2.0e}'.format(y))
    ax.yaxis.set_major_formatter(formatter)
    ax.yaxis.grid(b=True, which='major', color='lightgray', linestyle='solid')
    ax.yaxis.grid(b=True, which='minor', color='lightgray', linestyle='dotted')


def plot_classification_lr_schedule(initial_lr):
    def step_lr(epoch):
        lr = initial_lr

        if epoch < 5:
            lr = (1 / (5 - epoch)) * lr

        if epoch >= 30 and epoch < 60:
            lr = 0.75 * lr
        if epoch >= 60 and epoch < 90:
            lr = 0.5 * lr
        if epoch >= 90 and epoch < 120:
            lr = lr * 1e-1
        if epoch >= 120 and epoch < 150:
            lr = lr * 1e-2
        if epoch >= 150:
            lr = 0.5 * lr * 1e-2
        return lr

    x_vals = []
    y_vals = []
    for i in range(200):
        x_vals.append(i)
        y_vals.append(step_lr(i))

    fig, axes = plt.subplots(nrows=1, ncols=1, squeeze=False)
    ax = axes[0][0]
    setup_lr_plot_ax(ax, epochs=200, title="Learning Rate Schedule (Classification)")
    ax.step(x_vals, y_vals, lw=2, alpha=.8, color='cornflowerblue')


if __name__ == '__main__':
    plot_classification_lr_schedule(1e-3)
    # tikzplotlib.save("lr_schedule_plot_classification_1e-3.tex")
    plt.show()
