from applications.logged_object import LoggedObject
from applications.network_base.port_listener import SinglePortListener
from util.network.bidirectional_imagezmq import WorkDistributorHub, ZMQHubFactory
from util.timer import Timer


class InferenceBroker(LoggedObject, SinglePortListener):
    def __init__(self, open_ports=None, inference_servers_parallel=None, inference_servers_seq=None, name="broker",
                 compressed_parallel_distribution=False, do_logging=True):
        LoggedObject.__init__(self, do_logging=do_logging)
        SinglePortListener.__init__(self, open_ports)
        self.logger.debug("Listening on Port {}".format(open_ports[0]))

        self.inference_servers_parallel = inference_servers_parallel
        self.inference_servers_seq = inference_servers_seq

        self.inference_sockets_seq = {}
        self.distributor_hub = None
        self.compressed_parallel_distribution = compressed_parallel_distribution
        self._prepare_inference_server_connections()
        return

    def _prepare_inference_server_connections(self):
        for item in self.inference_servers_seq:
            self.inference_sockets_seq[item] = ZMQHubFactory.get_REQ(item)
            self.logger.debug("Created socket for sequential inference server {}".format(item))
        self.distributor_hub = WorkDistributorHub(self.inference_servers_parallel,
                                                  compressed=self.compressed_parallel_distribution,
                                                  do_logging=self.do_logging)
        self.logger.debug("Created Distributor Hub for inference servers {}".format(self.inference_servers_parallel))
        return

    def _get_inference_sockets_seq(self):
        return list(self.inference_sockets_seq.values())

    def _run_inner_loop(self):
        return

    def run(self):
        frame_timer = Timer(verbose=False)
        frame_count = 0
        loop_ok = True
        while loop_ok:
            self.logger.info("Frame {}".format(frame_count))
            frame_timer.start("")

            loop_ok = self._run_inner_loop()

            frame_time = frame_timer.stop()
            self.logger.info("Frame time: {}".format(frame_time))
            frame_count += 1
        return
