import time

import cv2
from PIL import Image

from applications.clients.inference_client import InferenceClient
from applications.network_utils import send_image
from imgproc.imgproc_utils import resize_image_array_pil
from prototype.camera_streamer import VideoFrameStreamer
from util.timer import log_time


class SingleServerInferenceClient(InferenceClient):
    def __init__(self, stream_provider,
                 server_address,
                 store_inferences=False,
                 compressed_send=False,
                 no_decoding=False,
                 do_logging=False,
                 display=False):
        super().__init__(server_address, compressed_send, no_decoding=no_decoding, do_logging=do_logging)
        self.stream_provider = stream_provider
        self.display = display
        self.frame_counter = 0
        self.store_inferences = store_inferences
        self.test_image = self.stream_provider.get()

    @log_time
    def _acquire_image(self):
        image = self.stream_provider.get()
        return image

    @log_time
    def _acquire_image2(self):
        image = self.stream_provider.get()
        return image

    def _display_image(self, window_name, image):
        cv2.imshow(window_name, image)

    def _split_inference(self, inference_list, inferences):
        inference_dict = {}
        width_counter = 0
        for elem in inference_list:
            h, w = elem[1]
            current_inference = inferences[0:h, width_counter:width_counter + w]
            inference_dict[elem[0]] = current_inference
            width_counter += w
        return inference_dict

    @log_time
    def _run_inner_loop(self):
        start_loop_time = time.perf_counter()
        image = self._acquire_image()
        # image = self.test_image
        # self.logger.debug(get_ndarray_diagnostics(image))
        after = time.perf_counter() - start_loop_time
        print("After Acquire time {}".format(after))

        resized_image = resize_image_array_pil(image, (224, 224), algo=Image.CUBIC)
        after = time.perf_counter() - start_loop_time
        print("After Resize time {}".format(after))

        send_image(self.client_socket,
                   resized_image,
                   message="client_image",
                   compressed_send=self.compressed_send)
        after = time.perf_counter() - start_loop_time
        print("After Send time {}".format(after))

        msg, inferences = self.client_socket.recv_data()
        self.logger.info("received inference from {}".format(msg))
        after = time.perf_counter() - start_loop_time
        print("After Recv time {}".format(after))

        #
        # if self.store_inferences:
        #     #imsave("../tmp_out/{}_resized.jpg".format(self.frame_counter), np.array(resized_image))
        #     #imsave("../tmp_out/{}.jpg".format(self.frame_counter), np.array(image))
        #     # imsave("../tmp_out/static_roinet710/{}_static_roinet710.jpg".format(self.frame_counter), inferences, quality=100)
        #     before = time.perf_counter()
        #     imsave("../tmp_out/trt_roinet/{}_static_hm_224.jpg".format(self.frame_counter), inferences,
        #            quality=100)
        #     after = time.perf_counter() - before
        #     print("Save time {}".format(after))
        #     #imsave("../tmp_out/{}_kp_tail_inference.jpg".format(self.frame_counter), inferences[:, :, 1])
        #
        # if self.display:
        #     # self._draw_marker_on_image(converted_image, fine_tip, 0)
        #     self._display_image("camera", np.array(cropped_image))
        #     self._display_image("inferences", inferences)
        #
        #     cv2.waitKey(1)
        self.frame_counter += 1

        end_loop_time = time.perf_counter() - start_loop_time
        print("End loop time {}".format(end_loop_time))
        return True


if __name__ == "__main__":
    no_decoding = False
    # camera_stream = BRIOCameraStreamer(no_decoding=no_decoding)
    # camera_stream = DummyCameraStreamer("L:\\tmp\\demo\\keypoints\\")
    camera_stream = VideoFrameStreamer("L:\\video_analysis\\static\\static_test.mp4")
    # camera_stream = VideoFrameStreamer("L:\\video_analysis\\gesture_check\\check.mp4")

    client = SingleServerInferenceClient(camera_stream,
                                         "tcp://132.199.132.34:60003",
                                         store_inferences=False,
                                         compressed_send=True,
                                         display=False,
                                         no_decoding=no_decoding,
                                         do_logging=True)
    client.run()
