from ast import literal_eval
from math import sqrt

import numpy as np
import pandas as pd

from dataset_utils import is_none_keypoint

pd.set_option('display.max_colwidth', -1)
pd.set_option('display.max_columns', None)


def ptip_distance(tip1, tip2):
    if is_none_keypoint(tip1):
        if is_none_keypoint(tip2):
            return 0
        else:
            return -1
    if is_none_keypoint(tip2):
        if is_none_keypoint(tip1):
            return 0
        else:
            return -1

    dx = tip2[0] - tip1[0]
    dy = tip2[1] - tip1[1]
    distance = sqrt(dx * dx + dy * dy)
    return distance


def eval_subject(id):
    current_annos = pd.read_csv('stroke_study_annos_{}.csv'.format(id), index_col=0,
                                dtype={'frame': np.int32},
                                converters={
                                    'ptip': literal_eval,
                                    'ptail': literal_eval})

    accuracy = pd.read_csv('accuracy_{}.csv'.format(id), index_col=0,
                           dtype={'frame': np.int32},
                           converters={
                               'ptip': literal_eval})

    accuracy['frame'] *= 15
    # get all rows where we have frames in accuracy for
    accuracy_frames = pd.Index(accuracy['frame'])
    eval_annos = current_annos.loc[current_annos.frame.isin(accuracy_frames)]
    eval_annos = eval_annos.reset_index()
    eval_annos = eval_annos.drop(columns=['full_path', 'ptail'])
    eval_annos = eval_annos.rename(columns={"frame": "frame_gt", "ptip": "ptip_gt"})

    eval_df = eval_annos.join(accuracy)
    eval_df['distance'] = eval_df.apply(lambda x: ptip_distance(x.ptip, x.ptip_gt), axis=1)
    return eval_df


def get_average(eval_df):
    no_mismatches = eval_df.loc[eval_df['distance'] != -1]
    only_mismatches = eval_df.loc[eval_df['distance'] == -1]
    return no_mismatches['distance'].mean(), only_mismatches['distance'].size


means = []
outliers_sum = 0
all_sum = 0
sum_5px = 0
sum_without_correct_nones = 0
sum_correct_nones = 0

sum_correctly_in_image = 0
sum_correctly_outside = 0

for i in range(0, 8):
    print("subject {}".format(i))
    eval_df = eval_subject(i)
    eval_df['distance'].to_csv("distance_{}.csv".format(i))
    mean, outliers = get_average(eval_df)
    print(mean)
    print(outliers)
    means.append(mean)

    outliers_sum += outliers
    all_sum += eval_df['distance'].size

    distances_only_zeros = eval_df.loc[eval_df['distance'] == 0]
    sum_correctly_outside += distances_only_zeros['distance'].size

    distances_only_positive_nonzeros = eval_df.loc[eval_df['distance'] > 0]
    sum_correctly_in_image += distances_only_positive_nonzeros['distance'].size

    distances_5px = distances_only_positive_nonzeros.loc[distances_only_positive_nonzeros['distance'] <= 5]
    sum_5px += distances_5px['distance'].size

print("All outliers: {}".format(outliers_sum))
print("All sum: {}".format(all_sum))
print("Outlier percentage: {}".format(outliers_sum / all_sum))

print("-----------")
print("Sum of correctly inside image: {}".format(sum_correctly_in_image))
print("Sum of correctly outside image: {}".format(sum_correctly_outside))
print("Percentage correctly classified: {}".format((sum_correctly_outside + sum_correctly_in_image) / all_sum))

print("Sum of 5px points: {}".format(sum_5px))
print("Percentage of 5px points to correctly inside image: {}".format(sum_5px / sum_correctly_in_image))
