import os
from ast import literal_eval
from itertools import chain
from math import sqrt

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns

from applications.final_prototype.val2_subjects import angle_rotation_nr_crop_history_prefix
from applications.final_prototype.val2_subjects import angle_rotation_nr_prefix
from applications.final_prototype.val2_subjects import angle_rotation_prefix
from applications.final_prototype.val2_subjects import native_rotation_prefix
from applications.final_prototype.val2_subjects import val2_subjects
from dataset_utils import is_none_keypoint

pd.set_option('display.max_colwidth', -1)
pd.set_option('display.max_columns', None)


def ptip_distance_px(tip_calc, tip_gt, subject_dict):
    if is_none_keypoint(tip_calc):
        if is_none_keypoint(tip_gt):
            return 0
        else:
            # this represents false negatives
            return -1
    if is_none_keypoint(tip_gt):
        if is_none_keypoint(tip_calc):
            return 0
        else:
            # this represents false positives
            return -2

    new_gt_x, new_gt_y = rotate_coordinate_cw(tip_gt, subject_dict["angle_cw"], subject_dict["res"])

    dx = new_gt_x - tip_calc[0]
    dy = new_gt_y - tip_calc[1]

    distance = sqrt(dx * dx + dy * dy)

    return distance


def ptip_distance_norm(tip_calc, tip_gt, subject_dict):
    if is_none_keypoint(tip_calc):
        if is_none_keypoint(tip_gt):
            return 0
        else:
            # this represents false negatives
            return -1
    if is_none_keypoint(tip_gt):
        if is_none_keypoint(tip_calc):
            return 0
        else:
            # this represents false positives
            return -2

    angle = subject_dict["angle_cw"]

    new_gt_x, new_gt_y = rotate_coordinate_cw(tip_gt, angle, subject_dict["res"])

    dx = new_gt_x - tip_calc[0]
    dy = new_gt_y - tip_calc[1]

    if angle == 90 or angle == 270:
        if NO_RESIZE:
            if CROP:
                x_scale = subject_dict["res"][1]
                y_scale = subject_dict["res"][1]
            else:
                x_scale = subject_dict["res"][0]
                y_scale = subject_dict["res"][1]
        else:
            if CROP:
                print("ALERT! Wrong configuration!")
            x_scale = subject_dict["res"][1]
            y_scale = subject_dict["res"][0]
    else:
        x_scale = subject_dict["res"][0]
        y_scale = subject_dict["res"][1]

    dx /= x_scale
    dy /= y_scale

    distance = sqrt(dx * dx + dy * dy)

    return distance


def eval_subject(id, subject_dict, use_angle=False):
    if use_angle:
        prefix = angle_rotation_prefix
        acc_path = subject_dict["path_acc_angle"]
        if NO_RESIZE:
            prefix = angle_rotation_nr_prefix
        if CROP:
            prefix = angle_rotation_nr_crop_history_prefix
            acc_path = subject_dict["path_acc_angle_crop_history"]
            # prefix = "val2_accuracy_final_"
            # acc_path = subject_dict["path"]
    else:
        prefix = native_rotation_prefix
        acc_path = subject_dict["path_acc"]

    current_annos = pd.read_csv(os.path.join(subject_dict["path"], 'val2_study_annos_{}.csv'.format(id)), index_col=0,
                                dtype={'frame': np.int32},
                                converters={
                                    'ptip': literal_eval,
                                    'ptail': literal_eval})

    accuracy = pd.read_csv(os.path.join(acc_path, '{}{}.csv'.format(prefix, id)), index_col=0,
                           dtype={'frame': np.int32},
                           converters={
                               'ptip': literal_eval})

    accuracy['frame'] *= subject_dict["step"]
    # get all rows where we have frames in accuracy for
    accuracy_frames = pd.Index(accuracy['frame'])
    eval_annos = current_annos.loc[current_annos.frame.isin(accuracy_frames)]
    eval_annos = eval_annos.reset_index()
    eval_annos = eval_annos.drop(columns=['full_path', 'ptail'])
    eval_annos = eval_annos.rename(columns={"frame": "frame_gt", "ptip": "ptip_gt"})

    eval_df = eval_annos.join(accuracy)
    eval_df['distance_px'] = eval_df.apply(lambda x: ptip_distance_px(x.ptip, x.ptip_gt, subject_dict), axis=1)
    eval_df['distance_norm'] = eval_df.apply(lambda x: ptip_distance_norm(x.ptip, x.ptip_gt, subject_dict), axis=1)
    return eval_df


def rotate_coordinate_cw(coord, angle, resolution):
    if not USE_ANGLE:
        return coord

    if NO_RESIZE:
        if CROP:
            border_x = 0
            border_y = (resolution[0] - resolution[1]) / 2
        else:
            border_x = (resolution[0] - resolution[1]) / 2
            border_y = (resolution[0] - resolution[1]) / 2
    else:
        border_x = 0
        border_y = 0

    if angle == 0 or angle == 360:
        return coord
    if angle == 90:
        new_y = coord[0] - border_y
        # if new_y < 0:
        #    return make_none_keypoint()
        return (border_x + resolution[1] - coord[1], new_y)
    if angle == 180:
        return (resolution[0] - coord[0], resolution[1] - coord[1])
    if angle == 270:
        new_y = resolution[0] - coord[0] - border_y
        # if new_y < 0:
        #    return make_none_keypoint()
        return (border_x + coord[1], new_y)

    print("illegal angle")
    return


def rotate_coordinate_ccw(coord, angle, resolution):
    return rotate_coordinate_cw(coord, 360 - angle, resolution)


def get_average(eval_df):
    no_mismatches = eval_df.loc[eval_df['distance_norm'] >= 0]
    only_mismatches = eval_df.loc[eval_df['distance_norm'] < 0]
    return no_mismatches['distance_norm'].median(), only_mismatches['distance_norm'].size


###############################################################################

USE_ANGLE = True
NO_RESIZE = True
CROP = True

# TODO: make sure that points that are outside the frame because of no_resize are classified as off

###############################################################################

means = []
outliers_sum = 0
all_sum = 0
sum_5px = 0
sum_1e3 = 0
sum_2e3 = 0
sum_3e3 = 0
sum_4e3 = 0
sum_5e3 = 0
sum_without_correct_nones = 0
sum_correct_nones = 0

sum_correctly_in_image = 0
sum_correctly_outside = 0
sum_falsely_in_image = 0
sum_falsely_out_of_image = 0

skip = [1, 3, 10]

distance_norm_collection = []
distance_px_collection = []
# distance_px_df = pd.DataFrame()

for i in range(0, 11):
    if i in skip:
        continue
    print("subject {}".format(i))
    eval_df = eval_subject(i, val2_subjects[i], use_angle=USE_ANGLE)
    eval_df = eval_df.sample(n=200, replace=False, random_state=1)
    mean, outliers = get_average(eval_df)
    print(mean)
    print(outliers)

    distance_norm_collection.append(eval_df.loc[eval_df['distance_norm'] > 0, 'distance_norm'].to_numpy())
    distance_px_collection.append(eval_df.loc[eval_df['distance_px'] > 0, 'distance_px'].to_numpy())

    outliers_sum += outliers
    all_sum += eval_df['distance_norm'].size

    distances_only_zeros = eval_df.loc[eval_df['distance_norm'] == 0]
    sum_correctly_outside += distances_only_zeros['distance_norm'].size

    distances_only_positive_nonzeros = eval_df.loc[eval_df['distance_norm'] > 0]
    sum_correctly_in_image += distances_only_positive_nonzeros['distance_norm'].size

    false_negatives = eval_df.loc[eval_df['distance_norm'] == -1]
    sum_falsely_out_of_image += false_negatives['distance_norm'].size

    false_positives = eval_df.loc[eval_df['distance_norm'] == -2]
    sum_falsely_in_image += false_positives['distance_norm'].size

    distances_5px = distances_only_positive_nonzeros.loc[distances_only_positive_nonzeros['distance_norm'] <= 0.003]
    sum_5px += distances_5px['distance_norm'].size

print("############## VALIDATION STUDY 2 ##############")

print("USE ANGLE IS {}".format(USE_ANGLE))
print("NO_RESIZE IS {}".format(NO_RESIZE))
print("CROP IS {}".format(CROP))

print("All outliers: {}".format(outliers_sum))
print("All sum: {}".format(all_sum))
print("Outlier percentage: {}".format(outliers_sum / all_sum))

print("-----------")
print("Sum of correctly inside image: {}".format(sum_correctly_in_image))
print("Sum of correctly outside image: {}".format(sum_correctly_outside))
print("Percentage correctly classified: {}".format((sum_correctly_outside + sum_correctly_in_image) / all_sum))

print("Sum of 5px points: {}".format(sum_5px))
print("Percentage of 5px points to correctly inside image: {}".format(sum_5px / sum_correctly_in_image))

print("-----------")

precision = sum_correctly_in_image / (sum_correctly_in_image + sum_falsely_in_image)
recall = sum_correctly_in_image / (sum_correctly_in_image + sum_falsely_out_of_image)
f1 = 2 * ((precision * recall) / (precision + recall))

print("Precision: {}".format(precision))
print("Recall: {}".format(recall))
print("F1: {}".format(f1))

###############################################################################
# plot
###############################################################################

# s = sns.swarmplot(data=distance_px_collection)
median_vals = np.array([np.median(elem) for elem in distance_norm_collection])
norm = plt.Normalize(median_vals.min(), median_vals.max())

colors = plt.cm.coolwarm(norm(median_vals))

tmp = list(chain.from_iterable(distance_norm_collection))

b = sns.boxplot(data=tmp,
                palette=colors)
b.set(yscale='log')
plt.show()
