import matplotlib

from applications.final_prototype.analysis_utils import get_index_of_min_distance_to_perfect_for_fpr_tpr

matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
from sklearn import metrics
import numpy as np
import pandas as pd
import seaborn as sns


def do_roc_plot():
    fpr_coords_median_val1 = np.load("val1_fpr_coords_median.npy")
    tpr_coords_median_val1 = np.load("val1_tpr_coords_median.npy")
    fpr_coords_median_val2 = np.load("val2_fpr_coords_median.npy")
    tpr_coords_median_val2 = np.load("val2_tpr_coords_median.npy")
    fpr_coords_median_testset = np.load("testset_fpr_coords_median.npy")
    tpr_coords_median_testset = np.load("testset_tpr_coords_median.npy")

    fpr_coords_mean_val1 = np.load("val1_fpr_coords_mean.npy")
    tpr_coords_mean_val1 = np.load("val1_tpr_coords_mean.npy")
    fpr_coords_mean_val2 = np.load("val2_fpr_coords_mean.npy")
    tpr_coords_mean_val2 = np.load("val2_tpr_coords_mean.npy")
    fpr_coords_mean_testset = np.load("testset_fpr_coords_mean.npy")
    tpr_coords_mean_testset = np.load("testset_tpr_coords_mean.npy")

    auc_val1 = metrics.auc(fpr_coords_mean_val1, tpr_coords_mean_val1)
    auc_val2 = metrics.auc(fpr_coords_mean_val2, tpr_coords_mean_val2)
    auc_test = metrics.auc(fpr_coords_mean_testset, tpr_coords_mean_testset)

    mean_opt_index_test = get_index_of_min_distance_to_perfect_for_fpr_tpr(fpr_coords_mean_testset,
                                                                           tpr_coords_mean_testset)
    mean_opt_point_test = (fpr_coords_mean_testset[mean_opt_index_test], tpr_coords_mean_testset[mean_opt_index_test])

    mean_opt_index_val1 = get_index_of_min_distance_to_perfect_for_fpr_tpr(fpr_coords_mean_val1, tpr_coords_mean_val1)
    mean_opt_point_val1 = (fpr_coords_mean_val1[mean_opt_index_val1], tpr_coords_mean_val1[mean_opt_index_val1])

    mean_opt_index_val2 = get_index_of_min_distance_to_perfect_for_fpr_tpr(fpr_coords_mean_val2, tpr_coords_mean_val2)
    mean_opt_point_val2 = (fpr_coords_mean_val2[mean_opt_index_val2], tpr_coords_mean_val2[mean_opt_index_val2])

    fig, ax = plt.subplots()

    ax.spines["top"].set_visible(False)
    ax.spines["bottom"].set_visible(False)
    ax.spines["right"].set_visible(False)
    ax.spines["left"].set_visible(False)

    ax.get_xaxis().tick_bottom()
    ax.get_yaxis().tick_left()

    plt.xticks(fontsize=12, color="black", alpha=0.7)
    plt.yticks(fontsize=12, color="black", alpha=0.7)

    for y in [0.0, 0.2, 0.4, 0.6, 0.8, 1.0]:
        ax.plot([0, 1], [y, y], linestyle=(0, (5, 5)), lw=0.5, color="black", alpha=0.3)

    ax.plot([0, 1], [0, 1], '--', lw=0.5, color="black", alpha=0.6)

    plt.tick_params(axis="both", which="both", bottom="off", top="off",
                    labelbottom="on", left="off", right="off", labelleft="on")

    ax.set_xlabel('False Positive Rate', labelpad=8, alpha=0.6)
    ax.set_ylabel('True Positive Rate', labelpad=8, alpha=0.6)
    # plt.text(0.5, 1.1, "ROC: Pen Detection", fontsize=10, ha="center")

    #### roc curves

    ax.step(fpr_coords_mean_testset, tpr_coords_mean_testset, label="Testset (AUC {:1.2f})".format(auc_test), lw=2.5,
            color="#1f78b4")
    ax.step(fpr_coords_mean_val1, tpr_coords_mean_val1, label='Validation Study 1 (AUC {:1.2f})'.format(auc_val1),
            lw=2.5, color="#a6cee3")
    ax.step(fpr_coords_mean_val2, tpr_coords_mean_val2, label='Validation Study 2 (AUC {:1.2f})'.format(auc_val2),
            lw=2.5, color="#b2df8a")

    #### opt points

    ax.scatter(*mean_opt_point_test, color="#fca503", marker='+', lw=2.0, s=[245], zorder=1000)
    ax.scatter(*mean_opt_point_val1, color="#fca503", marker='+', lw=2.0, s=[245], zorder=1000)
    ax.scatter(*mean_opt_point_val2, color="#fca503", marker='+', lw=2.0, s=[245], zorder=1000)

    ax.plot([mean_opt_point_test[0], mean_opt_point_test[0]], [0, mean_opt_point_test[1]], '--', lw=0.5,
            color="#1f78b4", alpha=0.8)
    ax.plot([0, mean_opt_point_test[0]], [mean_opt_point_test[1], mean_opt_point_test[1]], '--', lw=0.5,
            color="#1f78b4", alpha=0.8)

    ax.plot([mean_opt_point_val1[0], mean_opt_point_val1[0]], [0, mean_opt_point_val1[1]], '--', lw=0.5,
            color="#a6cee3", alpha=0.8)
    ax.plot([0, mean_opt_point_val1[0]], [mean_opt_point_val1[1], mean_opt_point_val1[1]], '--', lw=0.5,
            color="#a6cee3", alpha=0.8)

    ax.plot([mean_opt_point_val2[0], mean_opt_point_val2[0]], [0, mean_opt_point_val2[1]], '--', lw=0.5,
            color="#b2df8a", alpha=0.8)
    ax.plot([0, mean_opt_point_val2[0]], [mean_opt_point_val2[1], mean_opt_point_val2[1]], '--', lw=0.5,
            color="#b2df8a", alpha=0.8)

    ax.set_aspect('equal', 'box')
    ax.set(xlim=(-0.01, 1.1), ylim=(-0.01, 1.1))
    ax.legend()


def do_boxplot_sets():
    test_distances = np.load("testset_threshold_1_distances.npy")
    val1_distances = np.load("val1_threshold_8_distances.npy")
    val2_distances = np.load("val2_threshold_81_distances.npy")

    fig, ax = plt.subplots()

    ax.spines["top"].set_visible(False)
    ax.spines["bottom"].set_visible(False)
    ax.spines["right"].set_visible(False)
    ax.spines["left"].set_visible(False)

    ax.get_xaxis().tick_bottom()
    ax.get_yaxis().tick_left()

    plt.xticks(fontsize=12, color="black", alpha=0.7)
    plt.yticks(fontsize=12, color="black", alpha=0.7)

    plt.tick_params(axis="both", which="both", bottom="off", top="off",
                    labelbottom="on", left="on", right="off", labelleft="on")

    # ax.set_xlabel('Studies', labelpad=8, alpha=0.6)
    ax.set_ylabel('Normalized Distance Error', labelpad=8, alpha=0.6)
    # plt.text(0.5, 1.1, "ROC: Pen Detection", fontsize=10, ha="center")

    my_palette = {0: "#1f78b4", 1: "#a6cee3", 2: "#b2df8a"}

    # ax.yaxis.set_minor_locator(mticker.LogLocator(base=10, subs='all'))
    # ax.yaxis.set_minor_formatter(mticker.NullFormatter())
    ax.set_yscale('log')

    ax.set_axisbelow(True)
    ax.grid(True, which="major", ls="-", c='gray', alpha=0.4)
    ax.grid(True, which="minor", ls="-", c='gray', alpha=0.1)

    b = sns.boxplot(data=[test_distances, val1_distances, val2_distances], palette=my_palette, zorder=1000)

    median_vals = np.array([np.median(elem) for elem in [test_distances, val1_distances, val2_distances]])
    box_labels = ["Test Set", "Validation 1", "Validation 2"]
    # category labels
    plt.xticks(plt.xticks()[0], ["Test Set", "Validation 1", "Validation 2"])

    labels = ["{}\n\n{:1.1e}".format(name, median) for name, median in zip(box_labels, median_vals)]
    plt.xticks(plt.xticks()[0], labels, fontsize=10)
    plt.text(2.63, 0.000004, " Median", fontsize=10, ha="center", alpha=0.7)


def do_boxplot_val2():
    val2_distances = np.load("val2_threshold_81_distances_separate_subjects.npy", allow_pickle=True)

    fig, ax = plt.subplots()

    ax.spines["top"].set_visible(False)
    ax.spines["bottom"].set_visible(False)
    ax.spines["right"].set_visible(False)
    ax.spines["left"].set_visible(False)

    ax.get_xaxis().tick_bottom()
    ax.get_yaxis().tick_left()

    plt.xticks(fontsize=12, color="black", alpha=0.7)
    plt.yticks(fontsize=12, color="black", alpha=0.7)

    plt.tick_params(axis="both", which="both", bottom="off", top="off",
                    labelbottom="on", left="on", right="off", labelleft="on")

    # ax.set_xlabel('Studies', labelpad=8, alpha=0.6)
    ax.set_ylabel('Normalized Distance Error', labelpad=8, alpha=0.6)
    # plt.text(0.5, 1.1, "ROC: Pen Detection", fontsize=10, ha="center")

    my_palette = {0: "#1f78b4", 1: "#a6cee3", 2: "#b2df8a"}

    # ax.yaxis.set_minor_locator(mticker.LogLocator(base=10, subs='all'))
    # ax.yaxis.set_minor_formatter(mticker.NullFormatter())
    ax.set_yscale('log')

    ax.set_axisbelow(True)
    ax.grid(True, which="major", ls="-", c='gray', alpha=0.4)
    ax.grid(True, which="minor", ls="-", c='gray', alpha=0.1)

    median_vals = np.array([np.median(elem) for elem in val2_distances])
    norm = plt.Normalize(median_vals.min(), median_vals.max())

    colors = plt.cm.coolwarm(norm(median_vals))

    b = sns.boxplot(data=val2_distances, palette=colors, zorder=1000)

    # category labels
    labels = ["{}\n\n{:1.1e}".format(index, median) for index, median in enumerate(median_vals)]
    plt.xticks(plt.xticks()[0], labels, fontsize=10)
    plt.text(8, 0.0000089, " Median", fontsize=10, ha="center", alpha=0.7)


def describe_distances():
    test_distances = np.load("testset_threshold_1_distances.npy")
    val1_distances = np.load("val1_threshold_8_distances.npy")
    val2_distances = np.load("val2_threshold_81_distances.npy")

    test_series = pd.Series(test_distances)
    val1_series = pd.Series(val1_distances)
    val2_series = pd.Series(val2_distances)

    print(test_series.describe())
    # print(test_series.quantile(0.9))
    print(val1_series.describe())
    # print(val1_series.quantile(0.85))
    # for i in range(30):
    #    print("{}: {}".format(0.7 + i/100, val1_series.quantile(0.7 + i/100)))
    print(val2_series.describe())

    print(test_series.median())
    print(val1_series.median())
    print(val2_series.median())
    # for i in range(40):
    #    print("{}: {}".format(0.6 + i/100, val2_series.quantile(0.6 + i/100)))


if __name__ == '__main__':
    # do_roc_plot()
    do_boxplot_sets()
    # do_boxplot_val2()
    plt.savefig('boxplot_sets.png', dpi=300, format='png', bbox_inches='tight')
    plt.show()
    # describe_distances()
