import os

from applications.final_prototype.path_process_server import PathProcessServer
from prototype.camera_streamer import DummyCameraStreamer


class RecordedPathProcessServer(PathProcessServer):
    def __init__(self,
                 stream_bin,
                 stream_kp,
                 stream_patch,
                 patch_size=(224, 224),
                 heatmap_size=(211, 211),
                 surface_size=(1080, 1920),
                 do_logging=False,
                 interval=30,
                 static_aggregate=None):
        super().__init__('tcp://127.0.0.1:9999',
                         'tcp://127.0.0.1:9999',
                         patch_size,
                         heatmap_size,
                         surface_size,
                         do_logging,
                         interval,
                         record=False,
                         static_aggregate=static_aggregate)
        self.stream_bin = stream_bin
        self.stream_kp = stream_kp
        self.stream_patch = stream_patch
        self.frame_id_inf = 0
        self.frame_id_patch = 0
        return

    def _receive_patch_and_location(self):
        patch = self.stream_patch.get()
        name = self.stream_patch.get_current_image_name()
        split_name = name.split('_')
        loc_x = int(split_name[-2])
        loc_y = int(split_name[-1].split('.')[0])
        location = (loc_x, loc_y)
        self.frame_id_patch = int(split_name[0])
        return self.frame_id_patch, patch, location

    def _receive_inferences(self):
        bin_hm = self.stream_bin.get()
        name = self.stream_bin.get_current_image_name()
        split_name = name.split('_')
        kp_hm = self.stream_kp.get()
        self.frame_id_inf = int(split_name[0])
        return self.frame_id_inf, kp_hm, bin_hm


if __name__ == '__main__':
    subject_id = 8
    which_study = "val2"

    val1_base_path = "L:/stroke_study/analysis/"
    val2_base_path = "L:/validation_study_2/videos/"

    if which_study == "val1":
        subject_path = os.path.join(val1_base_path, "{}/output/".format(subject_id))
    else:
        subject_path = os.path.join(val2_base_path, "P{}/final/".format(subject_id + 1))

    kp_path = os.path.join(subject_path, "kp")
    bin_path = os.path.join(subject_path, "bin")
    patch_path = os.path.join(subject_path, "patch")

    bin_stream = DummyCameraStreamer(bin_path)
    kp_stream = DummyCameraStreamer(kp_path)
    patch_stream = DummyCameraStreamer(patch_path)
    server = RecordedPathProcessServer(
        stream_bin=bin_stream,
        stream_kp=kp_stream,
        stream_patch=patch_stream,
        do_logging=True,
        interval=30,
        static_aggregate=None)

    server.run()
