from itertools import chain

import matplotlib
from scipy import stats

from applications.final_prototype.analysis_utils import AnnotationsHeatmapsDistancesSetAnalyzer, \
    get_index_of_min_distance_to_perfect_for_fpr_tpr

matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
from sklearn import metrics
import numpy as np

import os


def make_subject_base_path(study_base_path, id):
    return os.path.join(study_base_path, "{}".format(id))


###############################################################################

val1_study_base_path = "L:/stroke_study/analysis"

val1_subjects = {
    0: {  # P0
        "name": "P0",
        "path": make_subject_base_path(val1_study_base_path, 0),
        "path_acc": os.path.join(make_subject_base_path(val1_study_base_path, 0), "output"),
        "step": 15,
        "res": (1920, 1080),
        "scale_factor": (1, 1),
        "angle_cw": 0,
        "count": 410,
        "actual_count": 410
    },
    1: {  # P1
        "name": "P1",
        "path": make_subject_base_path(val1_study_base_path, 1),
        "path_acc": os.path.join(make_subject_base_path(val1_study_base_path, 1), "output"),
        "step": 15,
        "res": (1920, 1080),
        "scale_factor": (1, 1),
        "angle_cw": 0,
        "count": 192,
        "actual_count": 185
    },
    2: {  # P2
        "name": "P2",
        "path": make_subject_base_path(val1_study_base_path, 2),
        "path_acc": os.path.join(make_subject_base_path(val1_study_base_path, 2), "output"),
        "step": 15,
        "res": (1920, 1080),
        "scale_factor": (1, 1),
        "angle_cw": 0,
        "count": 235,
        "actual_count": 231
    },
    3: {  # P3
        "name": "P3",
        "path": make_subject_base_path(val1_study_base_path, 3),
        "path_acc": os.path.join(make_subject_base_path(val1_study_base_path, 3), "output"),
        "step": 15,
        "res": (1920, 1080),
        "scale_factor": (1, 1),
        "angle_cw": 0,
        "count": 208,
        "actual_count": 203
    },
    4: {  # P4
        "name": "P4",
        "path": make_subject_base_path(val1_study_base_path, 4),
        "path_acc": os.path.join(make_subject_base_path(val1_study_base_path, 4), "output"),
        "step": 15,
        "res": (1920, 1080),
        "scale_factor": (1, 1),
        "angle_cw": 0,
        "count": 182,
        "actual_count": 181
    },
    5: {  # P5
        "name": "P5",
        "path": make_subject_base_path(val1_study_base_path, 5),
        "path_acc": os.path.join(make_subject_base_path(val1_study_base_path, 5), "output"),
        "step": 15,
        "res": (1920, 1080),
        "scale_factor": (1, 1),
        "angle_cw": 0,
        "count": 172,
        "actual_count": 167
    },
    6: {  # P6
        "name": "P6",
        "path": make_subject_base_path(val1_study_base_path, 6),
        "path_acc": os.path.join(make_subject_base_path(val1_study_base_path, 6), "output"),
        "step": 15,
        "res": (1920, 1080),
        "scale_factor": (1, 1),
        "angle_cw": 0,
        "count": 223,
        "actual_count": 219
    },
    7: {  # P7
        "name": "P7",
        "path": make_subject_base_path(val1_study_base_path, 7),
        "path_acc": os.path.join(make_subject_base_path(val1_study_base_path, 7), "output"),
        "step": 15,
        "res": (1920, 1080),
        "scale_factor": (1, 1),
        "angle_cw": 0,
        "count": 285,
        "actual_count": 280
    },
}


##################################################

def calc_weight(subject_id):
    return val1_subjects[5]["actual_count"] / val1_subjects[subject_id]["actual_count"]


if __name__ == '__main__':
    skip = []

    all_scores_mean = []
    all_scores_median = []
    all_scores_std = []
    all_scores_var = []
    all_labels = []
    all_weights = []

    percentile_vals = []
    all_distances = []

    percentile_thresholds = np.linspace(0, 255, 255)
    error_point = 1e-3

    for i in range(0, len(val1_subjects.keys())):
        if i not in skip:
            anno_path = os.path.join(val1_subjects[i]["path"], "stroke_study_annos_{}.csv".format(i))
            detections_path = os.path.join(val1_subjects[i]["path"], "val1_accuracy_{}.csv".format(i))
            heatmap_path = val1_subjects[i]["path_acc"]
            analyzer = AnnotationsHeatmapsDistancesSetAnalyzer(anno_path,
                                                               heatmap_path,
                                                               detections_path,
                                                               filename_identifier="val1_study",
                                                               frame_step=val1_subjects[i]["step"])

            scores_mean, scores_median, scores_std, scores_var, labels = analyzer.get_scores_labels_for_set()
            weights = np.ones((len(scores_mean))) * calc_weight(i)

            #########
            # percentiles
            #########

            # normalized error values
            percentiles, all_subject_distances = analyzer.get_distances_percentile_for_thresholds(
                error_point,
                percentile_thresholds,
                1920, 1080,
                lambda x: x)

            percentile_vals.append(percentiles)
            all_distances.append(all_subject_distances)

            all_scores_mean.extend(scores_mean)
            all_scores_median.extend(scores_median)
            all_scores_std.extend(scores_std)
            all_scores_var.extend(scores_var)
            all_labels.extend(labels)
            all_weights.extend(weights)

    ###########################################################################

    fpr_coords_mean, tpr_coords_mean, thresholds_mean = metrics.roc_curve(all_labels, all_scores_mean,
                                                                          sample_weight=all_weights)
    fpr_coords_median, tpr_coords_median, thresholds_median = metrics.roc_curve(all_labels, all_scores_median,
                                                                                sample_weight=all_weights)
    fpr_coords_std, tpr_coords_std, thresholds_std = metrics.roc_curve(all_labels, all_scores_std,
                                                                       sample_weight=all_weights)
    fpr_coords_var, tpr_coords_var, thresholds_var = metrics.roc_curve(all_labels, all_scores_var,
                                                                       sample_weight=all_weights)

    mean_auc = metrics.roc_auc_score(all_labels, all_scores_mean, sample_weight=all_weights)
    median_auc = metrics.roc_auc_score(all_labels, all_scores_median, sample_weight=all_weights)
    std_auc = metrics.roc_auc_score(all_labels, all_scores_std, sample_weight=all_weights)
    var_auc = metrics.roc_auc_score(all_labels, all_scores_var, sample_weight=all_weights)

    mean_ap = metrics.average_precision_score(all_labels, all_scores_mean, sample_weight=all_weights)
    median_ap = metrics.average_precision_score(all_labels, all_scores_median, sample_weight=all_weights)
    std_ap = metrics.average_precision_score(all_labels, all_scores_std, sample_weight=all_weights)
    var_ap = metrics.average_precision_score(all_labels, all_scores_var, sample_weight=all_weights)

    print("AUC // Mean {} // Median {} // Std {} // Var {} ".format(mean_auc, median_auc, std_auc, var_auc))
    print("AP // Mean {} // Median {} // Std {} // Var {} ".format(mean_ap, median_ap, std_ap, var_ap))

    print('\033[92m' + "Best AUC: {}".format(np.max([mean_auc, median_auc, std_auc, var_auc])))
    print('\033[92m' + "Best AP: {}".format(np.max([mean_ap, median_ap, std_ap, var_ap])))

    mean_opt_index = get_index_of_min_distance_to_perfect_for_fpr_tpr(fpr_coords_mean, tpr_coords_mean)
    mean_opt_point = (fpr_coords_mean[mean_opt_index], tpr_coords_mean[mean_opt_index])

    median_opt_index = get_index_of_min_distance_to_perfect_for_fpr_tpr(fpr_coords_median, tpr_coords_median)
    median_opt_point = (fpr_coords_median[median_opt_index], tpr_coords_median[median_opt_index])

    ###########################################################################

    fig, axes = plt.subplots(ncols=2)

    axes[0].step(fpr_coords_mean, tpr_coords_mean, label='Mean')
    axes[0].step(fpr_coords_median, tpr_coords_median, label='Median')
    axes[0].step(fpr_coords_std, tpr_coords_std, label='StdDev')
    axes[0].step(fpr_coords_var, tpr_coords_var, label='Variance')

    np.save("val1_fpr_coords_median.npy", fpr_coords_median)
    np.save("val1_tpr_coords_median.npy", tpr_coords_median)
    # np.save("val1_labels.npy", all_labels)
    np.save("val1_scores_median.npy", all_scores_median)

    axes[0].annotate("Median Optimum Threshold: {}".format(thresholds_median[median_opt_index]),
                     median_opt_point,
                     textcoords="offset points",
                     xytext=(0, 50),
                     arrowprops=dict(facecolor='black', shrink=0.05))

    axes[0].set_aspect('equal', 'box')
    axes[0].set(xlim=(-0.1, 1.1), ylim=(-0.1, 1.1))
    axes[0].legend()

    ##############
    # percentile plot
    ##############

    final_percentiles = []
    # we get the percentile for all distances at a certain threshold
    all_distances_all_thresholds_all_subjects = []
    for index, elem in enumerate(percentile_thresholds):
        threshold_distances_all_subjects = list(chain.from_iterable([sub[index] for sub in all_distances]))
        percentile = stats.percentileofscore(threshold_distances_all_subjects, error_point, kind="weak")
        final_percentiles.append(percentile)
        all_distances_all_thresholds_all_subjects.append(threshold_distances_all_subjects)

    opt_real_threshold = int(255 - thresholds_mean[mean_opt_index])
    np.save("val1_threshold_{}_distances.npy".format(opt_real_threshold),
            all_distances_all_thresholds_all_subjects[opt_real_threshold])
    print("Optimal Mean Threshold {}".format(opt_real_threshold))
    # b = sns.boxplot(data=all_distances_all_thresholds_all_subjects[int(255-thresholds_mean[mean_opt_index])], ax=axes[1])
    # b.set(yscale='log')

    #

    # axes[1].axvline(x=255-thresholds_mean[mean_opt_index])

    # plt.show()
