from itertools import chain

import matplotlib
import seaborn as sns
from scipy import stats

from applications.final_prototype.analysis_utils import AnnotationsHeatmapsDistancesSetAnalyzer, \
    get_index_of_min_distance_to_perfect_for_fpr_tpr

matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
from sklearn import metrics
import numpy as np
import os

from applications.final_prototype.val2_subjects import val2_subjects


##################################################


def calc_weight(subject_id):
    return val2_subjects[9]["actual_count"] / val2_subjects[subject_id]["actual_count"]


def get_scale_factors_for_distance_calc(angle, no_resize, crop, res):
    if angle == 90 or angle == 270:
        if no_resize:
            if crop:
                x_scale = res[1]
                y_scale = res[1]
            else:
                x_scale = res[0]
                y_scale = res[1]
        else:
            if crop:
                print("ALERT! Wrong configuration!")
            x_scale = res[1]
            y_scale = res[0]
    else:
        x_scale = res[0]
        y_scale = res[1]

    return x_scale, y_scale


def rotate_coordinate_cw(coord, angle, resolution, use_angle, no_resize, crop):
    if not use_angle:
        return coord

    if no_resize:
        if crop:
            border_x = 0
            border_y = (resolution[0] - resolution[1]) / 2
        else:
            border_x = (resolution[0] - resolution[1]) / 2
            border_y = (resolution[0] - resolution[1]) / 2
    else:
        border_x = 0
        border_y = 0

    if angle == 0 or angle == 360:
        return coord
    if angle == 90:
        new_y = coord[0] - border_y
        # if new_y < 0:
        #    return make_none_keypoint()
        return (border_x + resolution[1] - coord[1], new_y)
    if angle == 180:
        return (resolution[0] - coord[0], resolution[1] - coord[1])
    if angle == 270:
        new_y = resolution[0] - coord[0] - border_y
        # if new_y < 0:
        #    return make_none_keypoint()
        return (border_x + coord[1], new_y)

    print("illegal angle")
    return


def get_distances_for_certain_threshold_for_all_subjects(all_distances_all_thresholds_all_subjects, threshold):
    subject_distances = []
    for item in all_distances_all_thresholds_all_subjects:
        subject_distances.append(item[threshold])
    return subject_distances


if __name__ == '__main__':
    skip = [1, 3, 10]
    # skip = [1, 2, 3, 4, 5, 6, 7, 8, 10]

    all_scores_mean = []
    all_scores_median = []
    all_scores_std = []
    all_scores_var = []
    all_labels = []
    all_weights = []

    percentile_vals = []
    all_distances_all_thresholds_all_subjects = []
    percentile_thresholds = np.linspace(0, 255, 255)
    error_point = 1e-3

    for i in range(0, len(val2_subjects.keys())):
        if i not in skip:
            anno_path = os.path.join(val2_subjects[i]["path"], "val2_study_annos_{}.csv".format(i))
            detections_path = os.path.join(val2_subjects[i]["path"], "val2_accuracy_final_{}.csv".format(i))
            heatmap_path = os.path.join(val2_subjects[i]["path"], "final")
            analyzer = AnnotationsHeatmapsDistancesSetAnalyzer(anno_path,
                                                               heatmap_path,
                                                               detections_path,
                                                               filename_identifier="val2_study",
                                                               frame_step=val2_subjects[i]["step"])

            scores_mean, scores_median, scores_std, scores_var, labels = analyzer.get_scores_labels_for_set()
            weights = np.ones((len(scores_mean))) * calc_weight(i)

            #########
            # percentiles
            #########

            x_scale, y_scale = get_scale_factors_for_distance_calc(val2_subjects[i]["angle_cw"], True, True,
                                                                   val2_subjects[i]["res"])
            transform_func = lambda x: rotate_coordinate_cw(x, val2_subjects[i]["angle_cw"], val2_subjects[i]["res"],
                                                            True, True, True)

            # normalized error values
            percentiles, all_subject_distances = analyzer.get_distances_percentile_for_thresholds(
                error_point,
                percentile_thresholds,
                x_scale, y_scale,
                transform_func)

            percentile_vals.append(percentiles)
            all_distances_all_thresholds_all_subjects.append(all_subject_distances)

            all_scores_mean.extend(scores_mean)
            all_scores_median.extend(scores_median)
            all_scores_std.extend(scores_std)
            all_scores_var.extend(scores_var)
            all_labels.extend(labels)
            all_weights.extend(weights)

    ###########################################################################

    fpr_coords_mean, tpr_coords_mean, thresholds_mean = metrics.roc_curve(all_labels, all_scores_mean,
                                                                          sample_weight=all_weights)
    fpr_coords_median, tpr_coords_median, thresholds_median = metrics.roc_curve(all_labels, all_scores_median,
                                                                                sample_weight=all_weights)
    fpr_coords_std, tpr_coords_std, thresholds_std = metrics.roc_curve(all_labels, all_scores_std,
                                                                       sample_weight=all_weights)
    fpr_coords_var, tpr_coords_var, thresholds_var = metrics.roc_curve(all_labels, all_scores_var,
                                                                       sample_weight=all_weights)

    mean_auc = metrics.roc_auc_score(all_labels, all_scores_mean, sample_weight=all_weights)
    median_auc = metrics.roc_auc_score(all_labels, all_scores_median, sample_weight=all_weights)
    std_auc = metrics.roc_auc_score(all_labels, all_scores_std, sample_weight=all_weights)
    var_auc = metrics.roc_auc_score(all_labels, all_scores_var, sample_weight=all_weights)

    mean_ap = metrics.average_precision_score(all_labels, all_scores_mean, sample_weight=all_weights)
    median_ap = metrics.average_precision_score(all_labels, all_scores_median, sample_weight=all_weights)
    std_ap = metrics.average_precision_score(all_labels, all_scores_std, sample_weight=all_weights)
    var_ap = metrics.average_precision_score(all_labels, all_scores_var, sample_weight=all_weights)

    print("AUC // Mean {} // Median {} // Std {} // Var {} ".format(mean_auc, median_auc, std_auc, var_auc))
    print("AP // Mean {} // Median {} // Std {} // Var {} ".format(mean_ap, median_ap, std_ap, var_ap))

    print('\033[92m' + "Best AUC: {}".format(np.max([mean_auc, median_auc, std_auc, var_auc])))
    print('\033[92m' + "Best AP: {}".format(np.max([mean_ap, median_ap, std_ap, var_ap])))

    mean_opt_index = get_index_of_min_distance_to_perfect_for_fpr_tpr(fpr_coords_mean, tpr_coords_mean)
    mean_opt_point = (fpr_coords_mean[mean_opt_index], tpr_coords_mean[mean_opt_index])

    median_opt_index = get_index_of_min_distance_to_perfect_for_fpr_tpr(fpr_coords_median, tpr_coords_median)
    median_opt_point = (fpr_coords_median[median_opt_index], tpr_coords_median[median_opt_index])

    ###########################################################################

    fig, axes = plt.subplots(ncols=2)

    axes[0].step(fpr_coords_mean, tpr_coords_mean, label='Mean')
    # axes[0].step(fpr_coords_median, tpr_coords_median, label='Median')
    # axes[0].step(fpr_coords_std, tpr_coords_std, label='StdDev')
    # axes[0].step(fpr_coords_var, tpr_coords_var, label='Variance')

    np.save("val2_fpr_coords_mean.npy", fpr_coords_mean)
    np.save("val2_tpr_coords_mean.npy", tpr_coords_mean)
    # np.save("val2_labels.npy", all_labels)
    np.save("val2_scores_mean.npy", all_scores_mean)

    axes[0].annotate("Mean Optimum Threshold: {}".format(thresholds_mean[mean_opt_index]),
                     mean_opt_point,
                     textcoords="offset points",
                     xytext=(0, 50),
                     arrowprops=dict(facecolor='black', shrink=0.05))
    axes[0].set_aspect('equal', 'box')
    axes[0].set(xlim=(0, 1.1), ylim=(0, 1.1))
    axes[0].legend()

    ##############
    # percentile plot
    ##############

    final_percentiles = []
    all_final_distances = []
    # we get the percentile for all distances at a certain threshold
    for index, elem in enumerate(percentile_thresholds):
        threshold_distances_all_subjects = list(
            chain.from_iterable([sub[index] for sub in all_distances_all_thresholds_all_subjects]))
        percentile = stats.percentileofscore(threshold_distances_all_subjects, error_point, kind="weak")
        final_percentiles.append(percentile)
        all_final_distances.append(threshold_distances_all_subjects)

    opt_real_threshold = int(255 - thresholds_mean[mean_opt_index])
    np.save("val2_threshold_{}_distances.npy".format(opt_real_threshold),
            all_final_distances[opt_real_threshold])
    print("Optimal Mean Threshold {}".format(opt_real_threshold))

    # axes[1].plot(percentile_thresholds, final_percentiles)
    # axes[1].axvline(x=255-thresholds_mean[mean_opt_index])

    b = sns.boxplot(data=all_final_distances[opt_real_threshold], ax=axes[1])
    b.set(yscale='log')
    # plt.show()

    opt_threshold_all_subjects = get_distances_for_certain_threshold_for_all_subjects(
        all_distances_all_thresholds_all_subjects, opt_real_threshold)
    np.save("val2_threshold_{}_distances_separate_subjects.npy".format(opt_real_threshold),
            np.array(opt_threshold_all_subjects))
