import cv2
import numpy as np

from imgproc.path_extraction.online_ink_requester import OnlineInkRequester


class TabletRecorder():
    count = 0

    def __init__(self, window_width, window_height):
        self.drawing = False
        self.lines = []
        self.current_line = []
        self.window_width = window_width
        self.window_height = window_height
        self.line_image = np.zeros((window_height, window_width, 3), dtype='uint8')
        cv2.imshow("Tablet Recorder", self.line_image)
        cv2.setMouseCallback("Tablet Recorder", self.image_window_mouse_callback)

    def image_window_mouse_callback(self, event, click_x, click_y, flags, param):
        if event == cv2.EVENT_LBUTTONDOWN:
            self.drawing = True
        if event == cv2.EVENT_LBUTTONUP:
            self.drawing = False
            if len(self.current_line) > 0:
                self.lines.append(self.current_line.copy())
                self.current_line.clear()
        if event == cv2.EVENT_MOUSEMOVE:
            if self.drawing:
                self.current_line.append([click_x, click_y])
                cv2.polylines(
                    self.line_image,
                    np.array([self.current_line]).astype('int32'),
                    False,
                    (0, 0, 255),
                    thickness=2)
                # self.line_image[click_y, click_x] = (0, 0, 255)
        return

    def start(self):
        while True:
            cv2.imshow("Tablet Recorder", self.line_image)
            keypressed = cv2.waitKey(1)
            if keypressed == 27:
                break
            elif keypressed == ord('s'):
                print("Storing lines {}".format(self.lines))
                np.save(
                    "recorded_strokes_{}.npy".format(TabletRecorder.count),
                    np.array(self.lines),
                    allow_pickle=True)
                self.lines.clear()
                self.line_image = np.zeros((self.window_height, self.window_width, 3), dtype='uint8')
                TabletRecorder.count += 1
            elif keypressed == ord('r'):
                online_recognizer = OnlineInkRequester().online_ink(self.lines)


if __name__ == '__main__':
    viewer = TabletRecorder(800, 800)
    viewer.start()
