import sys

import numpy as np
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *


class TabletSampleWindow(QWidget):
    count = 0

    def __init__(self, app, parent=None):
        super(TabletSampleWindow, self).__init__(parent)
        self.setAttribute(Qt.WA_TabletTracking)
        self.app = app
        self.setMouseTracking(True)
        self.pen_is_down = False
        self.pen_x = 0
        self.pen_y = 0
        self.pen_pressure = 0
        self.text = ""
        # Resizing the sample window to full desktop size:
        frame_rect = app.desktop().frameGeometry()
        width, height = frame_rect.width(), frame_rect.height()
        self.width = width
        self.height = height
        self.resize(width, height)
        self.move(-9, 0)
        self.setWindowTitle("Tablet Recorder")

        self.lines = []
        self.current_line = []
        self.drawing = False
        self.init_press = False
        self.update()

    def tabletEvent(self, tabletEvent):
        self.pen_x = tabletEvent.globalX()
        self.pen_y = tabletEvent.globalY() - 38

        if not self.init_press:
            self.init_press = True
            self.update()
            return

        self.pen_pressure = int(tabletEvent.pressure() * 100)
        if tabletEvent.type() == QTabletEvent.TabletPress:
            self.pen_is_down = True
            self.text = "TabletPress event"
            self.drawing = True
        elif tabletEvent.type() == QTabletEvent.TabletMove:
            self.text = "TabletMove event"
            if self.drawing:
                self.text += " Drawing "
                self.current_line.append([self.pen_x, self.pen_y])
        elif tabletEvent.type() == QTabletEvent.TabletRelease:
            self.pen_is_down = False
            self.text = "TabletRelease event"
            self.drawing = False
            # if len(self.current_line) > 0:
            self.lines.append(self.current_line.copy())
            self.current_line.clear()

        self.text += " at x={0}, y={1}, pressure={2}%,".format(self.pen_x, self.pen_y, self.pen_pressure)
        if self.pen_is_down:
            self.text += " Pen is down."
        else:
            self.text += " Pen is up."

        tabletEvent.accept()

        self.update()

    def keyPressEvent(self, QKeyEvent):
        print("Storing lines {}".format(self.lines))
        np.save(
            "recorded_strokes_{}.npy".format(TabletSampleWindow.count),
            np.array(self.lines),
            allow_pickle=True)
        self.lines.clear()
        self.current_line.clear()
        self.update()

    def paintEvent(self, event):
        text = self.text
        i = text.find("\n\n")
        if i >= 0:
            text = text.left(i)
        painter = QPainter(self)
        painter.setRenderHint(QPainter.HighQualityAntialiasing)
        pen = QPen(Qt.black, 4, Qt.SolidLine)
        painter.setPen(pen)
        painter.begin(self)
        painter.drawText(self.rect(), Qt.AlignTop | Qt.AlignLeft, text)

        for line in self.lines:
            path = self.paint_line(line)
            painter.drawPath(path)
        painter.drawPath(self.paint_line(self.current_line))

        painter.end()

    def paint_line(self, line):
        if len(line) == 0:
            return QPainterPath()
        path = QPainterPath()
        path.moveTo(*line[0])
        for point in line:
            path.lineTo(*point)
        return path


app = QApplication(sys.argv)
mainform = TabletSampleWindow(app)
mainform.show()
app.exec_()
