import os

import tensorflow as tf
from keras.backend.tensorflow_backend import set_session

from applications.inference_servers.keypoint_inference_server import KeypointInferenceServer
from eval.online_inferrer import OnlineInferrer
from model.model_config import ModelConfig

config_yaml = "./saved_models/binarization_subset_1_train1/subpixel_binarization.yaml"
model_weights = "./saved_models/binarization_subset_1_train1/alpaca_binarization_subset_deconv3_subpix2_e140.h5"

# only use specific GPU(s)
os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
os.environ["CUDA_VISIBLE_DEVICES"] = "2"  # choose from GPUs 0-3
# only use as much VRAM as needed
config = tf.ConfigProto()
config.gpu_options.allow_growth = True
set_session(tf.Session(config=config))

model_config = ModelConfig(config_yaml)

input_params = {
    "image_filenames": "",
    "output_dir": "",
    "model_weights": model_weights,
    "separate_heatmaps": False,
    "annotations": None
}
inferrer = OnlineInferrer(input_params, model_config)
# inferrer = TensorRTOnlineInferrer(input_params, model_config)
server = KeypointInferenceServer(inferrer,
                                 'tcp://*:19858',
                                 name="binarization",
                                 compressed_send=False,
                                 do_logging=True)
server.run()
