import numpy as np
from typing import Tuple
from zmq import ZMQError

from imgproc.imgproc_utils import decompress_image_pil, decompress_image_cv2, compress_image


def send_image(zmq_socket, image, message="image", compressed_send=True, comp_quality=95, comp_method='pil'):
    if compressed_send:
        image = compress_image(image, quality=comp_quality, method=comp_method)

    zmq_socket.send_data(image, message, compressed=compressed_send)
    return


def receive_image(zmq_socket, comp_method='pil') -> Tuple[str, np.ndarray]:
    try:
        client_name, image = zmq_socket.recv_data()
    except ZMQError:
        print(ZMQError)
        raise IOError
    if not isinstance(image, np.ndarray):
        if comp_method == 'pil':
            image = decompress_image_pil(image)
        elif comp_method == 'cv2':
            image = decompress_image_cv2(image)
        else:
            raise ValueError("Unknown decompression method.")
    return client_name, image
