import logging

from applications.network_utils import receive_image
from applications.patch_prototype.patch_broker import PatchBroker
from dataset.dataset_utils import get_ndarray_diagnostics
from util.timer import log_time


class DebugPatchBroker(PatchBroker):

    def __init__(self, roinet_server, path_process_server, roinet_input_size=(400, 710), patch_size=(224, 224),
                 debug_sends=False, **kwargs):
        super().__init__(roinet_server, path_process_server, roinet_input_size=roinet_input_size, patch_size=patch_size,
                         debug_sends=debug_sends, **kwargs)
        self.logger.setLevel(logging.DEBUG)

    @log_time
    def _receive_and_decompress_image(self):
        msg, image = receive_image(self._get_open_port_socket(), comp_method='pil')
        self.logger.debug("Received image from {}".format(msg))
        self.logger.debug(get_ndarray_diagnostics(image))
        return msg, image

    @log_time
    def _run_inner_loop(self):
        loop_ok = True

        msg, image = self._receive_and_decompress_image()

        self._get_open_port_socket().send(b"Ok")  # send_data(strokes, "NextFrame")
        return loop_ok


if __name__ == "__main__":
    parallel_inference_servers = ['tcp://127.0.0.1:60000',
                                  'tcp://127.0.0.1:60001']

    patch_broker_server = DebugPatchBroker(
        roinet_server='tcp://127.0.0.1:60002',
        path_process_server='tcp://127.0.0.1:60003',
        roinet_input_size=(400, 710),
        patch_size=(224, 224),
        open_ports=['tcp://*:19858'],
        inference_servers_parallel=parallel_inference_servers,
        name="patch_broker",
        do_logging=True,
        debug_sends=True)

    patch_broker_server.run()
