import time

from applications.network_utils import send_image, receive_image
from applications.patch_prototype.patch_client import PatchInferenceClient
from prototype.camera_streamer import BRIOCameraStreamer
from util.timer import log_time


# matplotlib.use('TkAgg')

class DebugPatchInferenceClient(PatchInferenceClient):
    '''
    Client part of prototype with patch networks. Takes image from webcam,
    sends it to inference broker in full resolution.

    Works as follows:
    - 1 Sends image
    - 2 acquires new one
    - 3 waits to receive ok from broker
    - 4 upon ok, goto 1

    Not implemented yet:
    Sometimes, in the payload the pen path arrives, as well as any
    detected gestures. The client can yield this information
    to other applications.
    '''

    def __init__(self, stream_provider,
                 server_address,
                 store_inferences=False,
                 compressed_send=True,
                 no_decoding=False,
                 do_logging=False,
                 display=False,
                 debug_data_receive=False):
        super().__init__(stream_provider,
                         server_address,
                         store_inferences=store_inferences,
                         compressed_send=compressed_send,
                         no_decoding=no_decoding,
                         do_logging=do_logging,
                         display=display,
                         debug_data_receive=debug_data_receive)
        self.start_time = time.time()

    def _acquire_image(self):
        image = self.stream_provider.get()

        # if image is None:
        #    self.logger.error("Couldn't acquire image. Exiting.")
        #    raise IOError
        # self.image_counter += 1

        # self.logger.debug("Acquired image {}".format(self.image_counter))
        # self.logger.debug(get_ndarray_diagnostics(image))

        return image

    # def _display_image(self, window_name, image):
    #    cv2.imshow(window_name, image)

    def _receive_debug_data(self):
        msg, roinet_inference = receive_image(self.client_socket)
        self.client_socket.send(b"Yes1")
        msg, keypoint_heatmap = receive_image(self.client_socket)
        self.client_socket.send(b"Yes2")
        msg, bin_heatmap = receive_image(self.client_socket)
        self.client_socket.send(b"Yes3")
        msg, patch = receive_image(self.client_socket)
        self.client_socket.send(b"Yes4")
        return roinet_inference, keypoint_heatmap, bin_heatmap, patch

    @log_time
    def _send_image(self, image_to_send):
        send_image(self.client_socket,
                   image_to_send,
                   message="client_image",
                   comp_method='pil',
                   compressed_send=self.compressed_send)

        self.logger.debug("Sent image {}".format(self.image_counter))
        return

    def _run_inner_loop(self):
        # print(self.client_socket.recv())
        before = time.perf_counter()
        self.image = self._acquire_image()
        print("ACQUIRE: {}".format(time.perf_counter() - before))
        patch = self.image[:224, :224, :]
        self._send_image(patch)
        print("SEND: {}".format(time.perf_counter() - before))
        msg, first_heatmap = receive_image(self.client_socket)
        print("RECEIVE: {}".format(time.perf_counter() - before))
        self.loop_counter += 1
        if self.loop_counter % 10 == 0:
            print(self.loop_counter / (time.time() - self.start_time))

        return True


if __name__ == "__main__":
    no_decoding = False
    camera_stream = BRIOCameraStreamer(no_decoding=no_decoding)

    client = DebugPatchInferenceClient(camera_stream,
                                       "tcp://132.199.140.68:19858",
                                       store_inferences=False,
                                       compressed_send=True,
                                       display=False,
                                       no_decoding=no_decoding,
                                       do_logging=True,
                                       debug_data_receive=False)
    client.run()
