import argparse
import os

from applications.config.server_config import ServerConfig
from applications.inference_servers.keypoint_inference_server import KeypointInferenceServer
from eval.SavedModelInferrer import SavedModelInferrer
from model.model_config import ModelConfig
from util.gpu_utils import limit_gpu_memory_usage


def parse_args():
    parser = argparse.ArgumentParser(description="Keypoint Inference Server")
    parser.add_argument('--config_yaml', dest='config_yaml', required=True, help='')
    return parser.parse_args()


if __name__ == '__main__':
    args = parse_args()
    server_config = ServerConfig(args.config_yaml)
    listen_address = 'tcp://*:' + str(server_config.get("port"))

    # K.set_floatx('float16')

    # configure the tf session
    os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
    os.environ["CUDA_VISIBLE_DEVICES"] = str(server_config.get("gpu"))  # choose from GPUs 0-3
    # only use as much VRAM as needed
    limit_gpu_memory_usage()

    # get the inference parameters ready
    model_config_yaml = server_config.get("model_config")
    model_weights = server_config.get("model_weights")
    model_config = ModelConfig(model_config_yaml)
    input_params = {
        "image_filenames": "",
        "output_dir": "",
        "model_weights": model_weights,
        "separate_heatmaps": False,
        "annotations": None
    }
    inferrer = SavedModelInferrer(input_params, model_config)

    # start the server
    server = KeypointInferenceServer(inferrer,
                                     listen_address,
                                     name=server_config.get("name"),
                                     compressed_send=False,
                                     do_logging=True)
    server.run()
