# https://stackoverflow.com/questions/51843149/loading-batches-of-images-in-keras-from-pandas-dataframe
# https://medium.com/datadriveninvestor/keras-training-on-large-datasets-3e9d9dbc09d4

from imgaug import augmenters as iaa

from dataset.augmented_heatmap_sequence import AugmentedKeypointDataGenerator

""" 
This Keras Sequence implementation delivers image data and corresponding heatmaps
for training. It requires a list of image filenames and the corresponding keypoints as lists:
images: [       'image1.jpg', 'image2.jpg', 'image3.jpg']
keypoints: [    [[100, 100], [-1, -1]], 
                [[200, 100], [120, 90]], 
                [[-1, -1], [20, 10]] ]

The image rgb data gets scaled to 0..1.                
"""


class AugmentedPatchKeypointDataGenerator(AugmentedKeypointDataGenerator):

    def _create_augmentation_sequences(self):
        sequences = []

        sequences.append(iaa.Sequential([iaa.Affine(
            rotate=(-40, 40),
            scale=(0.9, 1.3),
            mode=["edge"]),
            iaa.Fliplr(0.5)
        ]))

        sequences.append(iaa.Sequential([iaa.Fliplr(0.5),
                                         iaa.CropAndPad(percent=(-0.3, 0.3),
                                                        pad_mode=["edge"])
                                         ]))

        sequences.append(iaa.Sequential([iaa.Multiply((0.7, 1.3), per_channel=0.5),
                                         iaa.Fliplr(0.5),
                                         iaa.AverageBlur(k=((2, 5), (2, 5))),
                                         iaa.AdditiveGaussianNoise(scale=(0, 0.02 * 255))
                                         ]))

        sequences.append(iaa.Sequential([iaa.Multiply((0.7, 1.3), per_channel=0.5),
                                         iaa.Flipud(0.5),
                                         iaa.AverageBlur(k=((2, 5), (2, 5))),
                                         iaa.AdditiveGaussianNoise(scale=(0, 0.02 * 255))
                                         ]))

        sequences.append(iaa.Sequential([iaa.Affine(
            rotate=(-30, 30),
            shear=(-15, 15),
            scale=(0.9, 1.15),
            mode=["edge"]),
            iaa.Fliplr(0.5)
        ]))

        sequences.append(iaa.Sequential([iaa.Affine(
            rotate=(-30, 30),
            scale=(0.9, 1.1)),
            iaa.CropAndPad(percent=(-0.3, 0.3),
                           pad_mode=["edge"]),
            iaa.Flipud(1.0)
        ]))

        sequences.append(iaa.Sequential([iaa.Affine(
            rotate=(-40, 40),
            scale=(0.85, 1.3)),
            iaa.ContrastNormalization((0.5, 1.5), per_channel=0.5),
            iaa.CropAndPad(percent=(-0.3, 0.3),
                           pad_mode=["edge"]),
            iaa.Fliplr(0.5)
        ]))

        return sequences
