import json
import os

import numpy as np
from natsort import natsorted


def get_files_and_keypoints(annotation_json, dataset_dir=None):
    image_filenames = []
    annotations = []
    if os.path.isfile(annotation_json):
        json_data = read_json(annotation_json)
        if dataset_dir is None:
            datadir = json_data['dataset_dir']
        else:
            datadir = dataset_dir
        image_filenames = []
        annotations = []
        if os.path.isdir(datadir):
            for elem in json_data:
                if elem != "dataset_dir":
                    image_filenames.append(os.path.join(datadir, elem))
                    annotations.append(json_data[elem])

        else:
            print("Directory given in json file does not exist.")
    else:
        print("File given does not exist.")
    return image_filenames, annotations


def save_json(data, path):
    json_content = json.dumps(data)
    with open(path, 'w') as json_file:
        json_file.write(json_content)
    json_file.close()


def read_json(path):
    with open(path, 'r') as json_file:
        json_data = json.load(json_file)
    json_file.close()
    return json_data


def get_image_files_in_directory(dir, full_path=False):
    files = []
    if os.path.isdir(dir):
        if full_path:
            files = [os.path.join(dir, filename) for filename in os.listdir(dir) if 'jpg' in filename]
        else:
            files = [filename for filename in os.listdir(dir) if 'jpg' in filename]
    else:
        print("Not a directory: {}".format(dir))
    return files


def get_natsorted_image_list_in_directory(dir, full_path=False):
    filenames = get_image_files_in_directory(dir, full_path)
    sorted_names = natsorted(filenames)
    return sorted_names


# keypoints only get rescaled if they are not denoted as non existent (-1, -1)
def rescale_keypoint(keypoint, from_width, from_height, to_width, to_height):
    if not is_none_keypoint(keypoint):
        return [keypoint[0] / from_width * to_width, keypoint[1] / from_height * to_height]
    else:
        return make_none_keypoint()


def is_none_keypoint(keypoint):
    return np.array_equal(keypoint, make_none_keypoint())


def make_none_keypoint():
    return [-1, -1]


def get_ndarray_diagnostics(data):
    assert (isinstance(data, np.ndarray))
    diagnostics_string = "Shape {} | MIN {} | MAX {} | dtype {}".format(data.shape, np.min(data), np.max(data),
                                                                        data.dtype)
    return diagnostics_string


# agnostic to getting handed names/messages or filenames with extension
def get_frame_id_patch_location_from_image_name(image_name):
    our_name = image_name
    if image_name.endswith('.jpg'):
        our_name = image_name[:-4]

    split_name = our_name.split('_')
    frame_id = int(split_name[0])
    location = (int(float(split_name[1])), int(float(split_name[2])))
    return frame_id, location


def get_frame_id_patch_location_from_patch_filename(image_name):
    our_name = image_name
    if image_name.endswith('.jpg'):
        our_name = image_name[:-4]

    split_name = our_name.split('_')
    frame_id = int(split_name[0])
    location = (int(float(split_name[2])), int(float(split_name[3])))
    return frame_id, location
