# https://fairyonice.github.io/Achieving-top-5-in-Kaggles-facial-keypoints-detection-using-FCN.html

import numpy as np

# this class generates a heatmap for the given parameters
# upon construction. It assumes that keypoint annotations
# equal to [-1, -1] are not visible.
from dataset.dataset_utils import is_none_keypoint


class Heatmap:
    def __init__(self, annotations, width, height, standard_dev):
        self._make_heatmap(annotations, width, height, standard_dev)
        return

    def get(self):
        return self.heatmap

    # TODO check if switch of parameters height and width is intentional
    # Expected value: center coordinate
    # Variance: standard_dev squared
    # Constant factor multiplied with exponential function: constant
    def _create_gaussian_around(self, center_x, center_y, standard_dev, width, height):
        x = np.arange(0, height, 1, float)
        y = np.arange(0, width, 1, float)[:, np.newaxis]
        exp_parameter = -(((x - center_x) ** 2 + (y - center_y) ** 2) / 2 * standard_dev ** 2)
        # constant = 1./(2 * np.pi * standard_dev**2)
        return np.exp(exp_parameter)

    def _make_heatmap(self, annotations, width, height, standard_dev):
        self.heatmap = np.zeros((height, width, len(annotations)), dtype=np.float32)
        label_count = len(annotations)
        for i in range(label_count):
            if not is_none_keypoint(annotations[i]):
                self.heatmap[:, :, i] = self._create_gaussian_around(annotations[i][0],
                                                                     annotations[i][1],
                                                                     standard_dev,
                                                                     height,
                                                                     width)
            else:
                self.heatmap[:, :, i] = np.zeros((height, width))
        return
