from natsort import natsorted

from dataset.dataset_utils import get_files_and_keypoints


class FromFileVectorizer:
    def __init__(self, annotation_json):
        self.annotation_json = annotation_json
        self.image_filenames, self.annotations = get_files_and_keypoints(annotation_json)
        self._natsort_annotations()

        self.pen_samples = self._collect_pen_samples()

    def _collect_pen_samples(self):
        pen_samples = []
        for index, elem in enumerate(self.image_filenames):
            pen_samples.append(
                {
                    "coords": self.annotations[index],
                    "image": elem
                }
            )
        return pen_samples

    def _natsort_annotations(self):
        anno_dict = {image_file: self.annotations[index] for index, image_file in enumerate(self.image_filenames)}
        sorted_names = natsorted(self.image_filenames)
        new_annos = [anno_dict[elem] for elem in sorted_names]
        self.image_filenames = sorted_names
        self.annotations = new_annos
        return

    def get_keypoint_path(self, keypoint_index):
        result = []
        for elem in self.pen_samples:
            result.append(elem["coords"][keypoint_index])
        return result

    def get_pen_samples(self):
        return self.pen_samples
