import numpy as np

from dataset.dataset_utils import rescale_keypoint
from imgproc.heatmap_keypoint_extractor import HeatmapKeypointExtractor, MaxFilterPeakExtraction
from imgproc.image_processor import ImageProcessor
from imgproc.patch_extractor import PatchExtractor


class PatchByHeatmapExtractor(ImageProcessor):
    '''
    Extracts a patch with given shape and coordinates from an image.
    and fills parts outside the image boundaries with zeroes.

    - Image is assumed to be RGB
    - Coords need to be row-major.
    '''

    def __init__(self, image: np.ndarray, heatmap: np.ndarray, patch_shape: tuple):
        super().__init__(None)
        self.image = image
        self.heatmap = heatmap
        self.patch_shape = patch_shape

    # in: row-major tip
    # out: patch and new column-major keypoint coordinate
    def run(self):
        peak = HeatmapKeypointExtractor(self.heatmap, MaxFilterPeakExtraction).run()
        h, w = self.image.shape[:2]
        hh, hw = self.heatmap.shape[:2]
        roi_location = rescale_keypoint(peak, hh, hw, h, w)
        patch, new_coord = PatchExtractor(self.image, self.patch_shape, np.array(roi_location), do_pad=True).run()
        return patch, new_coord, roi_location
