import cv2
import numpy as np

from imgproc.image_processor import ImageProcessor
from imgproc.patch_extractor import PatchExtractor


class PatchByPathMasker(ImageProcessor):
    def __init__(self, patch: np.ndarray, path: np.ndarray, location, path_surface_dimensions=(1080, 1920),
                 stroke_width=5, empty_fill_value=255):
        super().__init__(processing_strategy=None)
        self.patch = patch
        self.path = path
        self.location = location
        self.stroke_width = stroke_width
        self.path_surface_dimensions = path_surface_dimensions
        self.empty_fill_value = empty_fill_value
        return

    def run(self) -> np.ndarray:
        self.path_surface_dimensions = (1304, 2144)
        self.location = (self.location[0] + 112, self.location[1] + 112)
        self.path += 112
        path_image = np.zeros(self.path_surface_dimensions, dtype='uint8')
        cv2.polylines(path_image, [self.path.astype('int32')], False, color=(255, 255, 255),
                      thickness=self.stroke_width)
        path_image_patch, new_coord = PatchExtractor(path_image, self.patch.shape, self.location, channels=1,
                                                     do_pad=False).run()
        patch_mask = np.where(path_image_patch == 0)
        self.patch[patch_mask] = self.empty_fill_value / 255
        return self.patch
