# import matplotlib
# matplotlib.use('TkAgg')
#
# from skimage import io, transform
#
# import numpy as np
# import matplotlib.pyplot as plt
# import os
#
#
# # Load the two landscape photos
# img0 = io.imread('patches/patch17.jpg')
# img1 = io.imread('patches/patch18.jpg')
#
#
# def choose_corresponding_points():
#     """Utility function for finding corresponding features in images.
#     Alternately click on image 0 and 1, indicating the same feature.
#     """
#     f, (ax0, ax1) = plt.subplots(1, 2)
#     ax0.imshow(img0)
#     ax1.imshow(img1)
#
#     coords = plt.ginput(8, timeout=0)
#
#     np.savez('_reg_coords.npz', source=coords[::2], target=coords[1::2])
#
#     plt.close()
#
#
# # Re use previous coordinates, if found
# if not os.path.exists('_reg_coords.npz'):
#     choose_corresponding_points()
#
# coords = np.load('_reg_coords.npz')
#
# # Estimate the transformation between the two sets of coordinates,
# # assuming it is an affine transform
# tf = transform.estimate_transform('similarity', coords['source'], coords['target'])
#
# # Use a translation transformation to center both images for display purposes
# offset = transform.SimilarityTransform(translation=(-200, -170))
#
# img0_warped = transform.warp(img0, inverse_map=offset,
#                              output_shape=(600, 600))
#
# img1_warped = transform.warp(img1, inverse_map=offset + tf,
#                              output_shape=(600, 600))
#
#
# # Find where both images overlap; in that region average their values
# mask = (img0_warped != 0) & (img1_warped != 0)
# registered = img0_warped + img1_warped
# registered[mask] /= 2
#
# # Display the results
# f, (ax0, ax1, ax2) = plt.subplots(1, 3, subplot_kw={'xticks': [], 'yticks': []})
# ax0.imshow(img0)
# ax1.imshow(img1)
# ax2.imshow(registered)
#
# # ## ## Display image boundaries
# # y, x = img1.shape[:2]
# # box = np.array([[0, 0], [0, y], [x, y], [x, 0], [0, 0]])
# # tf_box = (offset + tf).inverse(box)
# # plt.plot(tf_box[:, 0], tf_box[:, 1], 'r-')
#
# plt.show()
import matplotlib

matplotlib.use('TkAgg')
from skimage.color import rgb2gray
from skimage import io, transform
from skimage.feature import ORB, match_descriptors, plot_matches
from skimage.measure import ransac
import matplotlib.pyplot as plt
import numpy as np

from imgproc.patch_extractor import PenPatchExtractor

extractor = PenPatchExtractor()

f, (ax0, ax1, ax2) = plt.subplots(1, 3, subplot_kw={'xticks': [], 'yticks': []}, figsize=(14, 7))

patch1 = io.imread("patches/patch17.jpg")
patch2 = io.imread("patches/patch18.jpg")

image0 = rgb2gray(patch1)
image1 = rgb2gray(patch2)

# image0 = transform.rescale(image0, 0.25)
# image1 = transform.rescale(image1, 0.25)

ax0.imshow(image0)
ax1.imshow(image1)
plt.show()
orb = ORB(n_keypoints=1000)

orb.detect_and_extract(image0)
keypoints1 = orb.keypoints[:, :]
descriptors1 = orb.descriptors[:, :]

orb.detect_and_extract(image1)
keypoints2 = orb.keypoints[:, :]
descriptors2 = orb.descriptors[:, :]

matches12 = match_descriptors(descriptors1,
                              descriptors2,
                              cross_check=True)

src = keypoints2[matches12[:, 1]][:, ::-1]
dst = keypoints1[matches12[:, 0]][:, ::-1]

model_robust, inliers = ransac((src, dst), transform.ProjectiveTransform, min_samples=4, residual_threshold=2)

# print(model_robust.transform)
inlier_idxs = np.nonzero(inliers)[0]
# my_matches = matches12
my_matches = np.column_stack((inlier_idxs, inlier_idxs))

plot_matches(ax2, image0, image1, keypoints1, keypoints2, my_matches)

r, c = image1.shape[:2]

corners = np.array([[0, 0],
                    [0, r],
                    [c, 0],
                    [c, r]])

# Warp the image corners to their new positions.
warped_corners = model_robust(corners)

# Find the extents of both the reference image and
# the warped target image.
all_corners = np.vstack((warped_corners, corners))

corner_min = np.min(all_corners, axis=0)
corner_max = np.max(all_corners, axis=0)

output_shape = (corner_max - corner_min)
output_shape = np.ceil(output_shape[::-1])

from skimage.color import gray2rgb
from skimage.transform import warp
from skimage.transform import SimilarityTransform

offset = SimilarityTransform(translation=-corner_min)

image0_ = warp(image0, offset.inverse,
               output_shape=output_shape, cval=-1)

image1_ = warp(image1, (model_robust + offset).inverse,
               output_shape=output_shape, cval=-1)


def add_alpha(image, background=-1):
    """Add an alpha layer to the image.

    The alpha layer is set to 1 for foreground
    and 0 for background.
    """
    rgb = gray2rgb(image)
    alpha = (image != background)
    return np.dstack((rgb, alpha))


image0_alpha = add_alpha(image0_)
image1_alpha = add_alpha(image1_)

merged = (image0_alpha + image1_alpha)
alpha = merged[..., 3]

# The summed alpha layers give us an indication of
# how many images were combined to make up each
# pixel.  Divide by the number of images to get
# an average.
merged /= np.maximum(alpha, 1)[..., np.newaxis]
# ax2.imshow(merged)
plt.show()
