import numpy as np

from imgproc.image_processor import ImageProcessor


class OnOffClassifier(ImageProcessor):
    '''
    points: column-major
    reference_batch: shape (count, height, width), if shape is only height, width then a
    single image is used
    inclusion_threshold: which intensity value classifies a point as on;
    TODO: DIFFERENTIATE BETWEEN BLACK BG AND WHITE BG REFERENCES -> < or > with threshold
    mean_threshold: which image mean makes a batch item invalid
    reference_offset: how far in the future the algorithm looks to classify points
    '''

    def __init__(self, points: np.ndarray, reference_batch: np.ndarray, inclusion_threshold=100, mean_threshold=100,
                 reference_offset=20):
        super().__init__(None)
        self.points = points
        self.reference_batch = reference_batch
        self.inclusion_threshold = inclusion_threshold
        self.mean_threshold = mean_threshold
        self.reference_offset = reference_offset

    def _create_on_off_indices(self, bin_offset):
        my_points = self.points
        on_off = []
        counter = 0
        # if we only get a single image, we use it for all points
        if len(self.reference_batch.shape) == 2:
            mean = np.mean(self.reference_batch)
            for point in my_points:
                if self.reference_batch[
                    int(point[1]), int(point[0])] < self.inclusion_threshold and mean < self.mean_threshold:
                    on_off.append(counter)
                counter += 1
        # if we get a whole batch, we use every image of that batch
        elif len(self.reference_batch.shape) == 3:
            for elem in zip(self.reference_batch[bin_offset:, :, :], my_points):
                bin = elem[0]
                point = elem[1]
                mean = np.mean(bin)
                if bin[int(point[1]), int(point[0])] > self.inclusion_threshold and mean < self.mean_threshold:
                    on_off.append(counter)
                counter += 1
        return on_off

    def run(self):
        indices = self._create_on_off_indices(self.reference_offset)
        on_points = np.take(self.points, indices, axis=0)
        return on_points, np.array(indices)

###############################################################################
