import numpy as np

from imgproc.image_processor import ImageProcessor
from imgproc.path_extraction.path_extraction_utils import move_points_by_gradient
from imgproc.path_extraction.point_gradient_calculator import PointGradientCalculator


class PathBeautifier(ImageProcessor):
    '''
    points: column-major
    '''

    def __init__(self, path_points: np.ndarray, on_indices: list, gradient_image: np.ndarray, on_threshold=0.5,
                 gap_limit=3, iterations=2):
        super().__init__(None)
        self.gap_limit = gap_limit
        self.on_indices = on_indices
        self.path_points = path_points
        self.gradient_image = gradient_image
        self.on_threshold = on_threshold
        self.iterations = iterations

    def _beautify_path(self, points_to_move):

        gradients = PointGradientCalculator(points_to_move, self.gradient_image).run()
        moved_points = move_points_by_gradient(points_to_move, gradients)
        return moved_points

    def run(self):
        if self.on_indices is None or len(self.on_indices) == 0:
            points_to_move = self.path_points
        else:
            points_to_move = np.take(self.path_points, self.on_indices, axis=0)
        for i in range(self.iterations):
            points_to_move = self._beautify_path(points_to_move)

        return points_to_move
