import numpy as np

from imgproc.image_processor import ImageProcessor


class PathFromKeypointExtractor(ImageProcessor):
    '''
    Creates a path from given heatmaps.

    - The heatmap batch has the format count, rows, cols
    - Coords need to be row-major.
    '''

    def __init__(self, heatmap_batch: np.ndarray):
        super().__init__(None)
        self.heatmap_batch = heatmap_batch
        # self.peak_extraction_strategy = peak_extraction_strategy

    def _get_points(self):
        max_idx = self.heatmap_batch.reshape(self.heatmap_batch.shape[0], -1).argmax(1)
        maxpos_vect = np.column_stack(np.unravel_index(max_idx, self.heatmap_batch[0, :, :].shape))
        return maxpos_vect

    def run(self):
        return self._get_points()

###############################################################################
# path extraction strategies
###############################################################################
