import numpy as np

from imgproc.image_processor import ImageProcessor


class PathStrokeSplitter(ImageProcessor):
    '''
    points: column-major
    '''

    def __init__(self, path_points, on_indices, split_gap_size=8):
        super().__init__(None)
        self.path_points = path_points
        self.on_indices = on_indices
        self.split_gap_size = split_gap_size

    def split_path_on_gaps(self, on_indices, points, split_gap_size):
        gap_list = np.diff(on_indices, axis=0)
        gap_start_indices = np.where((gap_list > split_gap_size))
        strokes = []
        last_start_index = on_indices[0]

        if gap_start_indices[0].size == 0:
            return []

        for gap_start_index in np.nditer(gap_start_indices):
            start_points_index = on_indices[gap_start_index]
            stroke_points = points[last_start_index:start_points_index]
            strokes.append(stroke_points)
            last_start_index = start_points_index + gap_list[gap_start_index] + 1
        return strokes

    def _split_on_indices_on_gaps(self, on_indices, split_gap_size):
        # if len(on_indices) < split_gap_size:
        #    return [[on_indices]]

        gap_list_in_on_indices = np.diff(on_indices, axis=0)
        gap_indices = np.where(gap_list_in_on_indices > split_gap_size)

        # if there are on_indices but no gaps,
        # we return a single stroke
        if gap_indices[0].size == 0:
            return [[on_indices]]

        stroke_indices = []
        new_start_on_index = 0

        for gap_index in np.nditer(gap_indices):
            stroke_indices.append([on_indices[new_start_on_index:gap_index + 1]])
            new_start_on_index = gap_index + 1

        stroke_indices.append([on_indices[new_start_on_index:]])
        return stroke_indices

    def run(self) -> list:
        stroke_indices = self._split_on_indices_on_gaps(self.on_indices, self.split_gap_size)

        strokes = []
        for elem in stroke_indices:
            if len(elem) > 0:
                elem_indices = np.take(self.path_points, elem, axis=0)
                if elem_indices.size > 0:
                    strokes.extend(elem_indices)
        return strokes
