from imgproc.image_processor import ImageProcessor

from imgproc.path_extraction.path_distances_extractor import PathDistancesExtractor


class PathSubdivider(ImageProcessor):
    '''
    points: column-major
    '''

    def __init__(self, strokes: list):
        super().__init__(None)
        self.strokes = strokes

    # https://stackoverflow.com/questions/43594646/how-to-calculate-the-coordinates-of-the-line-between-two-points-in-python
    def _subdivide_line(self, first_point, second_point, count):
        x_spacing = (second_point[0] - first_point[0]) / (count + 1)
        y_spacing = (second_point[1] - first_point[1]) / (count + 1)
        list_of_points = [[first_point[0] + i * x_spacing, first_point[1] + i * y_spacing] for i in range(1, count + 1)]
        return list_of_points

    def _subdivide_stroke(self, stroke):
        # we only use the extractor to get unique points
        distances, points = PathDistancesExtractor(stroke).run()
        new_points = []
        for index, item in enumerate(points[:-1]):
            new_points.append(item)
            line_points = self._subdivide_line(item, points[index + 1], 3)
            new_points.extend(line_points)
            new_points.append(points[index + 1])
        return new_points

    def run(self):
        new_strokes = []
        for stroke in self.strokes:
            new_points = self._subdivide_stroke(stroke)
            new_strokes.append(new_points)
        return new_strokes
