import numpy as np

from imgproc.image_processor import ImageProcessor


class PointGradientCalculator(ImageProcessor):
    '''
    points: column-major
    '''

    def __init__(self, points: np.ndarray, grayscale_image: np.ndarray):
        super().__init__(None)
        self.points = points
        self.grayscale_image = grayscale_image

    def run(self):
        gradients = []
        h, w = self.grayscale_image.shape[:2]
        for item in self.points:
            left = item[0]
            if item[0] - 1 >= 0 and item[0] - 1 < w:
                left = item[0] - 1

            right = item[0]
            if item[0] + 1 >= 0 and item[0] + 1 < w:
                right = item[0] + 1
            x_diff = (self.grayscale_image[int(item[1]), int(left)] - self.grayscale_image[
                int(item[1]), int(right)]) / 2
            top = item[1]
            if item[1] - 1 >= 0 and item[1] - 1 < h:
                top = item[1] - 1
            bottom = item[1]
            if item[1] + 1 >= 0 and item[1] + 1 < h:
                bottom = item[1] + 1
            y_diff = (self.grayscale_image[int(top), int(item[0])] - self.grayscale_image[
                int(bottom), int(item[0])]) / 2

            gradients.append((x_diff, -y_diff))
        return np.array(gradients)
