import numpy as np

from imgproc.path_extraction.path_distances_extractor import PathDistancesExtractor


class PointPath():
    def __init__(self, points: np.ndarray, coord_order='row'):
        self.coord_order = coord_order
        self.points = points

    ###############################################################################

    def get(self, filtered: bool, threshold=20, coord_order='row', marks=None, filter_duplicate_points=True):
        result_marks = marks
        if filtered:
            result, indices_removed = self._filter_outliers_from_points(self.points, threshold,
                                                                        filter_duplicate_points=filter_duplicate_points)
            if marks is not None:
                result_marks = np.delete(marks, indices_removed, axis=0)
        else:
            result = self.points

        if self.coord_order != coord_order:
            result = np.array([(coord[1], coord[0]) for coord in result])

        return result, result_marks

    ###############################################################################

    def _filter_outliers_from_points(self, points, threshold, check_before_and_after=False,
                                     filter_duplicate_points=True):
        distances, _ = PathDistancesExtractor(points, cumulative=False, normalized=False, unique=False).run()
        recorded_indices = []

        # check before and after: check if the distance before a point
        # and after a point exceeds the threshold, which marks clear outliers
        if not check_before_and_after:
            for index, distance in enumerate(distances):
                if distance > threshold:
                    recorded_indices.append(index + 1)
                elif distance == 0:
                    if filter_duplicate_points:
                        recorded_indices.append(index + 1)

        else:
            for index, distance in enumerate(distances[:-1]):
                if index % 2 == 0:
                    if distance > threshold and distances[index + 1] > threshold or distance == 0:
                        recorded_indices.append(index + 1)

        result_points = np.delete(points, recorded_indices, axis=0)
        return result_points, recorded_indices
