import numpy as np

from imgproc.path_extraction.vector_path import VectorPath
from prototype.text_binarizer import TextBinarizer


# OLD

class RaycastVectorPath(VectorPath):
    def __init__(self, intrinsic_matrix):
        super().__init__()
        self.intrinsic_matrix = intrinsic_matrix
        self.binarizer = TextBinarizer(heatmap_threshold=40)
        return

    def _raycast(self, pixel_coord, rmat, tvec):
        rmat_inv = np.transpose(rmat)
        cam_inv = np.linalg.inv(self.intrinsic_matrix)

        pixel_coord_h = np.array([pixel_coord[0], pixel_coord[1], 1])

        leftside_mat = rmat_inv @ cam_inv @ pixel_coord_h
        rightside_mat = rmat_inv @ tvec

        zconst = 0
        scale = (zconst + rightside_mat[2]) / leftside_mat[2]
        px = leftside_mat[0] * scale - rightside_mat[0]
        py = leftside_mat[1] * scale - rightside_mat[1]
        my_point = [px, py, 0]

        # point = rmat_inv @ (scale * cam_inv @ pixel_coord_h - tvec)
        return my_point

    def add_sample(self, frame, keypoints, binarize_heatmap, camera_pose_rmat, camera_pose_tvec):
        pen_tip = keypoints[0]
        pen_tail = keypoints[1]

        self.all_keypoints.append(keypoints)

        # binarize frame
        binarized_frame = self.binarizer.binarize(frame, binarize_heatmap)

        pen_tip_index = [int(np.round(el)) for el in pen_tip]
        # match keypoints and binarized frame
        is_on = binarized_frame[pen_tip_index[0], pen_tip_index[1]]
        if is_on:
            self.keypoints_on.append(pen_tip_index)
        else:
            self.keypoints_off.append(pen_tip_index)

        # raycast at keypoint position to get coordinate
        # in XZ plane
        pen_tip = np.reshape(pen_tip, 2)
        scene_point = self._raycast(pen_tip, camera_pose_rmat, camera_pose_tvec)
        self.whole_path.append(scene_point)

        if is_on:
            self.on_path.append(scene_point)

        return

    def get_whole_path(self):
        return self.whole_path

    def get_on_path(self):
        return self.on_path
