from abc import ABC, abstractmethod


class ProcessingStrategy(ABC):
    @abstractmethod
    def process_img(self, image, **kwargs):
        return None

    # We implement a subclasshook, so that
    # every class that implements the right interface
    # is a subclass and isinstance checks succeed
    # see: https://stackoverflow.com/questions/3570796/why-use-abstract-base-classes-in-python
    @classmethod
    def __subclasshook__(cls, C):
        if cls is ProcessingStrategy:
            if any("process_img" in B.__dict__ for B in C.__mro__):
                return True
        return NotImplemented
