from tensorflow.keras.layers import Conv2DTranspose, BatchNormalization, Activation, Conv2D
from tensorflow.keras.models import Model
from tensorflow.keras.utils import multi_gpu_model

from model.deconv_resnet import KeypointResnet


class DetailKeypointResnet(KeypointResnet):

    def __init__(self, num_keypoints, multi_gpu, verbose, freeze_resnet,
                 include_top=True, input_shape=None):
        super().__init__(num_keypoints, multi_gpu, verbose, freeze_resnet,
                         include_top=include_top, input_shape=input_shape)

    # overrides the model creation in the parent class
    def _create_model(self):
        c5_layer = self.resnet_model.get_layer('activation_49')

        # TODO: check if residual addition is needed - apparently residual addition
        # is only performed in the resnet part in msra implementation

        # 3 deconv layers mit BN und ReLu, kernel size 4, stride 2 und 256 channels
        # BN layers haben laut Simple Baseline Repo momentum = 0.9, default in keras ist 0.99
        deconv1 = Conv2DTranspose(256, (4, 4), strides=(2, 2))(c5_layer.output)
        deconv1_bn = BatchNormalization(momentum=0.9)(deconv1)
        deconv1_bn = Activation('relu')(deconv1_bn)
        deconv2 = Conv2DTranspose(256, (4, 4), strides=(2, 2))(deconv1_bn)
        deconv2_bn = BatchNormalization(momentum=0.9)(deconv2)
        deconv2_bn = Activation('relu')(deconv2_bn)
        deconv3 = Conv2DTranspose(256, (4, 4), strides=(2, 2))(deconv2_bn)
        deconv3_bn = BatchNormalization(momentum=0.9)(deconv3)
        deconv3_bn = Activation('relu')(deconv3_bn)

        deconv4 = Conv2DTranspose(128, (4, 4), strides=(2, 2))(deconv3_bn)
        deconv4_bn = BatchNormalization(momentum=0.9)(deconv4)
        deconv4_bn = Activation('relu')(deconv4_bn)
        # deconv5 = Deconvolution2D(64, (2, 2), strides=(2, 2))(deconv4_bn)
        # deconv5_bn = BatchNormalization(momentum=0.9)(deconv5)
        # deconv5_bn = Activation('relu')(deconv5_bn)

        # last conv layer 1x1 with output for keypoint predictions ?
        final_kernel_size = (1, 1)
        last_conv = Conv2D(self.num_keypoints, final_kernel_size, strides=(1, 1))(deconv4_bn)

        my_model = Model(self.resnet_model.input, last_conv)

        if self.multi_gpu > 0:
            my_model = multi_gpu_model(my_model, gpus=self.multi_gpu)

        my_model.compile(loss='mean_squared_error', optimizer='adam', metrics=['mse'])
        return my_model
