import tensorflow as tf
from tensorflow.keras.applications import resnet50
from tensorflow.keras.layers import Conv2DTranspose, BatchNormalization, Activation, Conv2D
from tensorflow.keras.models import Model
from tensorflow.keras.utils import multi_gpu_model


def keypoint_loss(y_pred, y_true):
    # find position of max value in y_true
    #

    # find position of max value in y_pred

    # calculate quadratic distance

    # average over all batch elements

    # return averagenvidia
    return


class PatchNetwork:
    def __init__(self, num_keypoints, multi_gpu, verbose=False, freeze_resnet=False,
                 include_top=True, input_shape=None, use_custom_loss=False, use_distributed_strategy=False):
        self.num_keypoints = num_keypoints
        self.multi_gpu = multi_gpu
        self.use_custom_loss = use_custom_loss
        self.use_distributed_strategy = use_distributed_strategy

        if self.use_distributed_strategy:
            mirrored_strategy = tf.distribute.MirroredStrategy()

            with mirrored_strategy.scope():
                self.keypoint_model = self._create_model(include_top, freeze_resnet, input_shape)
        else:
            self.keypoint_model = self._create_model(include_top, freeze_resnet, input_shape)
        if verbose:
            self.keypoint_model.summary()
        return

    def _create_model(self, include_top, freeze_resnet, input_shape):
        if include_top:
            self.resnet_model = resnet50.ResNet50(weights='imagenet')
        else:
            self.resnet_model = resnet50.ResNet50(include_top=False,
                                                  input_shape=input_shape,
                                                  weights='imagenet')

        if freeze_resnet:
            for layer in self.resnet_model.layers:
                layer.trainable = False

        c5_layer = self.resnet_model.get_layer('conv5_block3_out')

        # 3 deconv layers mit BN und ReLu, kernel size 4, stride 2 und 256 channels
        # BN layers haben laut Simple Baseline Repo momentum = 0.9, default in keras ist 0.99
        deconv1 = Conv2DTranspose(256, (4, 4), strides=(2, 2))(c5_layer.output)
        deconv1_bn = BatchNormalization(momentum=0.9)(deconv1)
        deconv1_bn = Activation('relu')(deconv1_bn)
        deconv2 = Conv2DTranspose(256, (4, 4), strides=(2, 2))(deconv1_bn)
        deconv2_bn = BatchNormalization(momentum=0.9)(deconv2)
        deconv2_bn = Activation('relu')(deconv2_bn)

        deconv3 = Conv2DTranspose(128, (4, 4), strides=(2, 2))(deconv2_bn)
        deconv3_bn = BatchNormalization(momentum=0.9)(deconv3)
        deconv3_bn = Activation('relu')(deconv3_bn)

        deconv4 = Conv2DTranspose(64, (4, 4), strides=(3, 3))(deconv3_bn)
        deconv4_bn = BatchNormalization(momentum=0.9)(deconv4)
        deconv4_bn = Activation('relu')(deconv4_bn)

        # last conv layer 1x1 with output for keypoint predictions ?
        final_kernel_size = (1, 1)
        last_conv = Conv2D(self.num_keypoints, final_kernel_size, strides=(1, 1))(deconv4_bn)

        my_model = Model(self.resnet_model.input, last_conv)

        if self.multi_gpu > 0:
            my_model = multi_gpu_model(my_model, gpus=self.multi_gpu)

        if not self.use_custom_loss:
            my_model.compile(loss='mean_squared_error', optimizer='adam', metrics=['mse'])
        else:
            my_model.compile(loss=keypoint_loss, optimizer='adam', metrics=['mse'])
        return my_model

    def predict(self, image_batch):
        predictions = self.keypoint_model.predict(image_batch)
        return predictions

    def get_model(self):
        return self.keypoint_model


if __name__ == '__main__':
    network = PatchNetwork(1, 0, verbose=True, include_top=False, input_shape=(224, 398))
