import numpy as np

from dataset.peak_detection import get_peaks_for_heatmap_collection


class KeypointHeatmapExtractor:
    def __init__(self, image_width, image_height, heatmap_width, heatmap_height):
        self.image_width = image_width
        self.image_height = image_height
        self.heatmap_width = heatmap_width
        self.heatmap_height = heatmap_height
        pass

    def detect_heatmap_peaks(self, heatmap):
        # use argpartition to find n smallest values in negated image
        # we use flattened array indices by giving axis=None for ease of use and recreate them afterwards
        # argpart = (-(heatmap.astype('int'))).argpartition(peak_count, axis=None)
        # max_indices = argpart[:peak_count]
        max_index = np.where(heatmap == np.amax(heatmap))
        # max_indices_x, max_indices_y = np.unravel_index(max_indices, heatmap.shape)

        return np.array(max_index).astype('float64')  # max_indices_x, max_indices_y

    def get_peaks_for_heatmap_collection(self, heatmaps):
        peaks = []
        for i in range(heatmaps.shape[2]):
            # peaks.append(list(zip(detect_heatmap_peaks(heatmaps[:, :, i], 1))))
            peaks.append(self.detect_heatmap_peaks(heatmaps[:, :, i]))
        return peaks

    def extract_single_heatmap(self, single_heatmap):
        return

    def extract(self, heatmaps):
        pen_tip_heatmap = heatmaps[:, :, 0]
        # pen_tail_heatmap = heatmaps[:, :, 1]

        heatmap_h, heatmap_w = pen_tip_heatmap.shape[:2]
        scale_width = self.image_width / heatmap_w
        scale_height = self.image_height / heatmap_h

        peaks = get_peaks_for_heatmap_collection(heatmaps)

        for peak in peaks:
            peak[0] *= scale_height
            peak[1] *= scale_width

        return peaks
