import pickle
import socket

import cv2
import numpy as np

from dataset.dataset_utils import read_json


class PoseProvider:
    def __init__(self):
        self.pose_buffer = [np.identity(4)]
        self.host = "localhost"
        self.port = 31337
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.bind((self.host, self.port))
        self.done = False

    def _listen(self):
        self.socket.listen(1)
        conn, addr = self.socket.accept()
        print("Connection from {}".format(addr))
        while not self.done:
            data = conn.recv(4096)
            pose_data = pickle.loads(data)
            self.pose_buffer.append(pose_data)
            if not data:
                break
            conn.send(data)
        conn.close()

    def get(self):
        return self.pose_buffer[-1]


class DummyPoseProvider:
    def __init__(self, pose_json):
        self.poses = []
        self.pose_counter = 0
        json_data = read_json(pose_json)
        for item in json_data:
            current_element = json_data[item]
            self.poses.append([current_element["rvec"][0][0], current_element["tvec"][0][0]])
        return

    def get(self):
        if self.pose_counter < len(self.poses):
            result = self.poses[self.pose_counter]
            self.pose_counter += 1
        else:
            self.pose_counter = 0
            result = self.poses[self.pose_counter]

        rvec = np.array(result[0])
        rmat_cm = np.zeros((3, 3))
        cv2.Rodrigues(rvec, rmat_cm)
        return rmat_cm, np.array(result[1])
