import matplotlib
import numpy as np

from imgproc.path_extraction.vector_path import VectorPath
from prototype.camera_streamer import DummyCameraStreamer
from prototype.inference_provider import DummyBinarizationInferenceStrategy
from prototype.inference_provider import DummyKeypointInferenceStrategy
from prototype.inference_provider import InferenceProvider
from prototype.keypoint_heatmap_extractor import KeypointHeatmapExtractor
from prototype.pose_provider import DummyPoseProvider

matplotlib.use('TkAgg')
import matplotlib.pyplot as plt


def setup_result_sender():
    return


def create_3d_plot(our_points):
    fig3d = plt.figure()
    ax3d = fig3d.add_subplot(111, projection='3d')
    ax3d.set_xlabel('X')
    ax3d.set_ylabel('Y')
    ax3d.set_zlabel('Z')
    our_ps = np.array(our_points)
    ax3d.plot(our_ps[:, 0], our_ps[:, 1], our_ps[:, 2], marker='^', c='b')
    plt.show()
    return


def run(model_config_binary, model_config_keypoints):
    # start listening for pose information
    camera_pose_provider = DummyPoseProvider("./prototype/dummy_set/poses_aruco.json")

    # setup camera
    camera_stream = DummyCameraStreamer("./prototype/dummy_set/")
    image_width, image_height = camera_stream.get_width_height()

    # instantiate inference provider
    keypoint_inference_provider = InferenceProvider(
        DummyKeypointInferenceStrategy("./prototype/dummy_set/keypoints_aruco.json"))
    binary_inference_provider = InferenceProvider(
        DummyBinarizationInferenceStrategy("./prototype/dummy_set/bin/"))

    intrinsic_matrix = np.load(
        "./test/camera_matrix_1080p.npy")

    vector_path = VectorPath(intrinsic_matrix)

    heatmap_extractor = KeypointHeatmapExtractor(image_width, image_height)

    # result_sender = setup_result_sender()

    counter = 0
    while not counter == 61:
        frame = camera_stream.get()
        camera_rmat, camera_tvec = camera_pose_provider.get()
        keypoint_heatmaps = keypoint_inference_provider.get(frame)
        binarize_heatmap = binary_inference_provider.get(frame)
        # TODO check row majorness consistency for keypoints
        keypoints = heatmap_extractor.extract(keypoint_heatmaps)
        vector_path.add_sample(frame, keypoints, binarize_heatmap, camera_rmat, camera_tvec)
        counter += 1
        # result_sender.send(vector_path)

    on_path = vector_path.get_on_path()
    whole_path = vector_path.get_whole_path()
    print("ON path length")
    print(len(on_path))
    print("Whole path length")
    print(len(whole_path))
    create_3d_plot(on_path)
    create_3d_plot(whole_path)


if __name__ == "__main__":
    run("", "")
