import argparse

from dataset.dataset_utils import get_natsorted_image_list_in_directory
from model.model_config import ModelConfig
from train.patch_binarization_trainer import PatchBinarizationTrainer
from util.gpu_utils import limit_gpu_memory_usage


def parse_args():
    parser = argparse.ArgumentParser(description="stalwart-keypoint-alpaca PATCH BINARIZATION training")

    parser.add_argument('--image_dir', dest='image_dir', required=True, help='')
    parser.add_argument('--test_image_dir', dest='test_image_dir', required=False, help='')
    parser.add_argument('--gt_dir', dest='gt_dir', required=True, help='')
    parser.add_argument('--test_gt_dir', dest='test_gt_dir', required=False, help='')

    parser.add_argument('--config_yaml', dest='config_yaml', required=True, help='')
    parser.add_argument('--pretrained_weights', dest='pretrained_weights', required=False, help='')

    return parser.parse_args()


def main():
    # only use specific GPU(s)
    # os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
    # os.environ["CUDA_VISIBLE_DEVICES"] = "2"  # choose from GPUs 0-3
    # only use as much VRAM as needed
    limit_gpu_memory_usage()

    args = parse_args()

    model_config = ModelConfig(args.config_yaml)
    image_filenames = get_natsorted_image_list_in_directory(args.image_dir, full_path=True)
    gt_filenames = get_natsorted_image_list_in_directory(args.gt_dir, full_path=True)

    input_params = {
        "image_filenames": image_filenames,
        "annotations": gt_filenames,
        "pretrained_weights": args.pretrained_weights
    }

    if args.test_image_dir is not None:
        test_image_filenames = get_natsorted_image_list_in_directory(args.test_image_dir, full_path=True)
        input_params["test_image_filenames"] = test_image_filenames
        if args.test_gt_dir is not None:
            test_gt_filenames = get_natsorted_image_list_in_directory(args.test_gt_dir, full_path=True)
            input_params["test_annotations"] = test_gt_filenames
        else:
            print("Warning: Undefined behavior.")

    my_trainer = PatchBinarizationTrainer(input_params, model_config)
    my_trainer.train()
    pass


if __name__ == "__main__":
    main()
