import argparse
import os

from dataset.dataset_utils import get_files_and_keypoints
from model.model_config import ModelConfig
from train.tiplet_trainer import TipletTrainer
from util.gpu_utils import limit_gpu_memory_usage


def parse_args():
    parser = argparse.ArgumentParser(description="stalwart-keypoint-alpaca TIPLET training")

    parser.add_argument('--annotation_json', dest='annotation_json', required=True, help='')
    parser.add_argument('--test_annotation_json', dest='test_annotation_json', required=False, default=None, help='')
    parser.add_argument('--config_yaml', dest='config_yaml', required=True, help='')
    parser.add_argument('--pretrained_weights', dest='pretrained_weights', required=False, help='')

    return parser.parse_args()


def main():
    # only use specific GPU(s)
    os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
    os.environ["CUDA_VISIBLE_DEVICES"] = "3"  # choose from GPUs 0-3
    # only use as much VRAM as needed
    limit_gpu_memory_usage()

    args = parse_args()

    model_config = ModelConfig(args.config_yaml)
    image_filenames, annotations = get_files_and_keypoints(args.annotation_json)

    input_params = {
        "image_filenames": image_filenames,
        "annotations": annotations,
        "pretrained_weights": args.pretrained_weights
    }

    if args.test_annotation_json is not None:
        test_image_filenames, test_annotations = get_files_and_keypoints(args.test_annotation_json)
        input_params["test_image_filenames"] = test_image_filenames
        input_params["test_annotations"] = test_annotations

    my_trainer = TipletTrainer(input_params, model_config)
    my_trainer.train()
    pass


if __name__ == "__main__":
    main()
