import os
from ast import literal_eval

import numpy as np
import pandas as pd

from dataset_utils import save_json, rescale_keypoint
from util.select_and_extract_keypoint_annotations import split_image_name


def resize_and_save_final_consistent_annos(width, height, target_dir):
    all_annos = pd.read_csv('final_consistent_annos.csv', index_col=0,
                            dtype={'subject': np.int32,
                                   'task': np.int32,
                                   'frame': np.int32},
                            converters={
                                'ptip': literal_eval,
                                'ptail': literal_eval})

    anno_dict = {}
    anno_test_dict = {}
    list_test = [0, 6, 10, 14, 20, 23, 38, 39]
    for index, row in all_annos.iterrows():
        path = row['full_path']
        img_name = row.name

        # pil_img = Image.open(path)
        # pil_img = resize_image_pil(pil_img, (710, 400))
        # pil_img.save(os.path.join(target_dir, img_name), format="JPEG", quality=100, subsampling=0)

        # print("saved image {}".format(img_name))
        ptip_keypoint = row['ptip']
        ptail_keypoint = row['ptail']
        ptip_keypoint = rescale_keypoint(ptip_keypoint, 3840, 2160, width, height)
        ptail_keypoint = rescale_keypoint(ptail_keypoint, 3840, 2160, width, height)

        if row['subject'] in list_test:
            anno_test_dict[img_name] = [ptip_keypoint, ptail_keypoint]
        else:
            anno_dict[img_name] = [ptip_keypoint, ptail_keypoint]

    save_json(anno_dict, os.path.join(target_dir, 'final_annos_train.json'))
    save_json(anno_test_dict, os.path.join(target_dir, 'final_annos_test.json'))


def save_patches_224_annos():
    all_annos = pd.read_csv('patches_224_keypoints.csv', index_col=0,
                            dtype={'img_name': np.str},
                            converters={
                                'ptip': literal_eval})

    target_dir = "L:/large_dataset_study/patches_224"
    anno_dict = {}
    anno_test_dict = {}
    list_test = [0, 6, 10, 14, 20, 23, 38, 39]
    for index, row in all_annos.iterrows():
        path = row['img_name']
        subject, task, frame = split_image_name(path)
        img_name = os.path.split(path)[1]

        ptip_keypoint = row['ptip']

        if subject in list_test:
            anno_test_dict[img_name] = [ptip_keypoint]
        else:
            anno_dict[img_name] = [ptip_keypoint]

    save_json(anno_dict, os.path.join(target_dir, 'patches_224_train.json'))
    save_json(anno_test_dict, os.path.join(target_dir, 'patches_224_test.json'))


# target_dir = "L:/large_dataset_study/final_resized_set"
if __name__ == '__main__':
    pd.set_option('display.max_colwidth', -1)
    pd.set_option('display.max_columns', None)

    save_patches_224_annos()
