import random
from ast import literal_eval

import numpy as np
import pandas as pd
from imageio import imsave

from imgproc.patch_extractor import PatchExtractor
from prototype.camera_streamer import DummyCameraStreamer

image_stream = DummyCameraStreamer("L:/large_dataset_study/binarization_subset_1")
bin_stream = DummyCameraStreamer("L:/large_dataset_study/binarization_subset_1/binarized")

all_annos = pd.read_csv('final_consistent_annos.csv', index_col=0,
                        dtype={'subject': np.int32,
                               'task': np.int32,
                               'frame': np.int32},
                        converters={
                            'ptip': literal_eval,
                            'ptail': literal_eval})

img = image_stream.get()
bin = bin_stream.get()

img_count = 0
patch_count = 0
patch_limit = 10
while img is not None:
    h, w = img.shape[:2]
    patch_half = 224 / 2
    img_name = image_stream.get_current_image_name()
    tip_keypoint = all_annos.loc[img_name, 'ptip']
    keypoint = np.array((tip_keypoint[1], tip_keypoint[0]))
    for patch_count in range(patch_limit):
        x_range = patch_half - 20
        y_range = patch_half - 20
        offset_x = random.uniform(-x_range, x_range)
        offset_y = random.uniform(-y_range, y_range)
        offset = (int(offset_y), int(offset_x))
        img_patch, _ = PatchExtractor(
            img,
            (224, 224),
            keypoint,
            offset=offset,
            do_pad=True).run()
        bin_patch, _ = PatchExtractor(
            img,
            (224, 224),
            keypoint,
            offset=offset,
            do_pad=True).run()
        imsave("../tmp_out/bin_patches/{}_{}.jpg".format(img_name, patch_count), img_patch, quality=100)
        imsave("../tmp_out/bin_patches/{}_{}_bin.jpg".format(img_name, patch_count), bin_patch, quality=100)
    for patch_count in range(patch_limit):
        x_loc = random.uniform(patch_half, w - patch_half)
        y_loc = random.uniform(patch_half, h - patch_half)
        coord = np.array((y_loc, x_loc))
        img_patch, _ = PatchExtractor(
            img,
            (224, 224),
            coord,
            do_pad=True).run()
        bin_patch, _ = PatchExtractor(
            img,
            (224, 224),
            coord,
            do_pad=True).run()
        imsave("../tmp_out/bin_patches/{}_{}_r.jpg".format(img_name, patch_count), img_patch, quality=100)
        imsave("../tmp_out/bin_patches/{}_{}_bin_r.jpg".format(img_name, patch_count), bin_patch, quality=100)
    img = image_stream.get()
    bin = bin_stream.get()
    img_count += 1
