import os
from ast import literal_eval

import numpy as np
import pandas as pd

from util.keypoint_analysis import get_keypoints_dataframe, get_subject, get_task, get_frame
from util.select_and_extract_keypoint_annotations import get_all_image_paths_units, copy_images


def get_img_file_from_full_path(full_path):
    return os.path.split(full_path)[1]


def copy_image_files_from_dataframe_to(target_dir, dataframe):
    image_files = dataframe['full_path'].to_list()
    copy_images(target_dir, image_files)
    return


def get_all_image_full_paths_dataframe():
    full_path_file_list = get_all_image_paths_units("L:/large_dataset_study/extracted_frames/")
    files_df = pd.DataFrame(full_path_file_list)
    files_df.columns = ['full_path']
    files_df['img_file'] = files_df.full_path.map(get_img_file_from_full_path)
    return files_df


def sample_unannotated_images():
    from util import keypoint_analysis
    keypoints_df = get_keypoints_dataframe(keypoint_analysis.all_keypoint_files, statistics=False)

    files_df = get_all_image_full_paths_dataframe()

    keypoints_idx = keypoints_df.index
    files_idx = pd.Index(files_df['img_file'])

    files_df = files_df.loc[~files_idx.isin(keypoints_idx)]
    files_df['subject'] = files_df.img_file.map(get_subject)
    files_df['task'] = files_df.img_file.map(get_task)
    files_df['frame'] = files_df.img_file.map(get_frame)

    # remove subjects with already large amount of annotations
    enough_anno_subs = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, 16]
    files_df = files_df[~files_df['subject'].isin(enough_anno_subs)]
    return files_df


def sample_subjects(unannotated_image_dataframe, n=350):
    files_grouped = unannotated_image_dataframe.groupby(['subject'])
    sampled_images = files_grouped.apply(lambda x: x.sample(n=n))
    return sampled_images


def sample_and_copy_unannotated_images():
    files_df = sample_unannotated_images()
    sampled_images = sample_subjects(files_df, 350)
    copy_image_files_from_dataframe_to("L:/large_dataset_study/last_todo_files/", sampled_images)


def read_new_annotations_and_remove_marked(json_files, marked_images_txt):
    marked_img_series = pd.read_csv(marked_images_txt, header=None)
    print(marked_img_series.head())
    keypoints_df = get_keypoints_dataframe(json_files, statistics=False)
    print(keypoints_df.shape)
    keypoints_idx = keypoints_df.index
    marked_idx = pd.Index(marked_img_series[0])
    marked_removed_df = keypoints_df.loc[~keypoints_idx.isin(marked_idx)]
    return marked_removed_df


def save_last_todos_with_marked_removed():
    new_json_files = [
        "L:/large_dataset_study/last_todo_start_annos/keypoints_10_59_22-28_11_19.json",
        "L:/large_dataset_study/last_todo_start_annos/keypoints_12_53_48-29_11_19.json",
        "L:/large_dataset_study/last_todo_start_annos/keypoints_13_10_23-27_11_19.json",
        "L:/large_dataset_study/last_todo_start_annos/keypoints_14_40_36-28_11_19.json",
        "L:/large_dataset_study/last_todo_start_annos/keypoints_17_23_25-27_11_19.json",
        "L:/large_dataset_study/last_todo_start_annos/keypoints_18_19_42-28_11_19.json"
    ]

    marked_removed_df = read_new_annotations_and_remove_marked(new_json_files,
                                                               'L:/large_dataset_study/last_todo_start_annos/all_marked_files.txt')
    print(marked_removed_df.head())
    print(marked_removed_df.shape)

    # marked_removed_df.reset_index(inplace=True)
    # print(marked_removed_df.head())
    # print(marked_removed_df.shape)
    marked_removed_df.to_csv('last_todos_with_marked_removed.csv')


if __name__ is "__main__":
    pd.set_option('display.max_colwidth', -1)
    pd.set_option('display.max_columns', None)

    all_old = pd.read_csv('updated_all_old.csv', index_col=0,
                          dtype={'subject': np.int32,
                                 'task': np.int32,
                                 'frame': np.int32},
                          converters={
                              'ptip': literal_eval,
                              'ptail': literal_eval})

    new_todos = pd.read_csv('last_todos_with_marked_removed.csv', index_col=0,
                            dtype={'subject': np.int32,
                                   'task': np.int32,
                                   'frame': np.int32},
                            converters={
                                'ptip': literal_eval,
                                'ptail': literal_eval})

    all_old = all_old.append(new_todos)
    print(all_old.head())

    # grouped_all = all_old.groupby(['subject'])
    # print(grouped_all.size())

    enough_anno_subs = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14, 15, 16, 17]
    downsampled_annos = all_old[all_old['subject'].isin(enough_anno_subs)]
    downsampled_annos = sample_subjects(downsampled_annos, n=500)
    print(downsampled_annos.shape)
    print(downsampled_annos.head())

    not_enough_anno_subs = [11, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39]
    smaller_annos = all_old[all_old['subject'].isin(not_enough_anno_subs)]
    print(smaller_annos.shape)
    print(smaller_annos.head())

    final_annos = downsampled_annos.append(smaller_annos)
    final_annos.to_csv('final_annos.csv')
