import matplotlib

from dataset.augmented_binarization_sequence import AugmentedBinarizationGenerator
from dataset.dataset_utils import get_natsorted_image_list_in_directory
from model.model_config import ModelConfig
from visual.sequence_viewer import SequenceViewer

matplotlib.use('TkAgg')


class BinarizationSequenceViewer(SequenceViewer):
    def __init__(self, display_data_provider, batch_size):
        super().__init__(display_data_provider, batch_size, 2, 3)
        self.index_increment = 2
        pass

    def _view_axes(self):
        self._clear_axes()

        if self.index_counter + 2 == 32:
            print("bla")

        self.axes[0][0].imshow(self.batch_x[self.index_counter], interpolation='nearest')
        self.axes[0][1].imshow(self.batch_x[self.index_counter + 1], interpolation='nearest')
        self.axes[0][2].imshow(self.batch_x[self.index_counter + 2], interpolation='nearest')

        self.axes[1][0].imshow(self.batch_y[self.index_counter][:, :, 0], interpolation='nearest')
        self.axes[1][1].imshow(self.batch_y[self.index_counter + 1][:, :, 0], interpolation='nearest')
        self.axes[1][2].imshow(self.batch_y[self.index_counter + 2][:, :, 0], interpolation='nearest')
        pass


if __name__ == '__main__':
    heatmap_path = "L:\\dataset_study\\prestudy_dataset\\easy_read\\gts\\new_converted\\"
    image_path = "L:\\dataset_study\\prestudy_dataset\\easy_read\\images_to_gt\\"

    images = get_natsorted_image_list_in_directory(image_path, full_path=True)
    gts = get_natsorted_image_list_in_directory(heatmap_path, full_path=True)

    model_config = ModelConfig('../train/params/subpixel_binarization.yaml')

    data_provider = AugmentedBinarizationGenerator(
        images,
        gts,
        model_config)

    viewer = BinarizationSequenceViewer(data_provider, 32)
    viewer.view()
