import matplotlib

from imgproc.heatmap_image_binarizer import HeatmapImageBinarizer

matplotlib.use('TkAgg')

import matplotlib.pyplot as plt
import numpy as np
import cv2
import random

from visual.image_heatmap_file_data_provider import ImageHeatmapFileDataProvider
from visual.sequence_viewer import SequenceViewer


class HeatmapViewer(SequenceViewer):
    def __init__(self, display_data_provider, batch_size):
        super().__init__(display_data_provider, batch_size, 2, 3, figsize=(28, 14))
        self.index_increment = 4
        self.batch_counter = 3
        self._load_current_data()
        pass

    def _get_contours(self, img):
        color_img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
        gray_img = cv2.cvtColor(color_img, cv2.COLOR_BGR2GRAY)
        binary_img = cv2.Canny(gray_img, 10, 160)
        contours, hierarchy = cv2.findContours(binary_img, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)

        print(len(contours))
        filtered_contours = []
        contour_colors = []

        for index, contour in enumerate(contours):
            filtered_contours.append(contour)

            # mask = np.zeros(gray_img.shape, np.uint8)
            # cv2.drawContours(mask, [contour], 0, 255, -1)
            # mean_val = cv2.mean(color_img, mask=mask)
            # ax2.append(mean_val)

            col_r = random.randint(0, 255)
            col_g = random.randint(0, 255)
            col_b = random.randint(0, 255)
            contour_colors.append([col_r / 255.0, col_g / 255.0, col_b / 255.0, 1.0])
            cv2.drawContours(color_img, [contour], -1, (col_r, col_g, col_b), 1)
        return color_img

    def _view_axes(self):
        self._clear_axes()
        binnum = 2
        bin_images = self._binarize_images(binnum)
        binbin = np.zeros_like(bin_images[0])
        # for i in range(binnum):
        #    binbin += bin_images[i]

        filter_bin_images = self._filter_bin_images(bin_images)

        for i in range(2):
            current_image = self.batch_x[self.index_counter + i]
            current_heatmap = self.batch_y[self.index_counter + i]
            # self.axes[i][0].imshow(current_heatmap)
            self.axes[i][0].imshow(current_image)
            self.axes[i][1].imshow(bin_images[i])
            self.axes[i][2].imshow(filter_bin_images[i])

            # self.axes[i][2].imshow(binbin)

        # match_img, warped_image1 = self._get_matches_img_of_binary_pair(bin_images[0], bin_images[1])

        match_source = self.batch_x[self.index_counter]
        match_target = self.batch_x[self.index_counter + 1]
        # match_img, warped_image1 = self._get_matches_img_of_binary_pair(match_source / 255., match_target / 255.)
        # diff_warp_target = compare_images(match_target, warped_image1, method='diff')
        # diff_source_target = compare_images(match_target, match_source, method='diff')

        # self.axes[0][1].imshow(warped_image1)
        # self.axes[1][1].imshow(diff_warp_target)
        # self.axes[1][2].imshow(diff_source_target)

        # self.axes[0][2].imshow(match_img)

        # self.axes[1][0].imshow(warped_image1, alpha=0.5)
        # self.axes[1][0].imshow(bin_images[0], alpha=0.5, cmap='winter')

        plt.show()

        pass

    def _filter_bin_images(self, images):
        result = []
        for elem in images:
            kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (2, 2))
            res = cv2.morphologyEx(elem, cv2.MORPH_OPEN, kernel)
            res = cv2.morphologyEx(res, cv2.MORPH_CLOSE, kernel, iterations=15)
            result.append(res)
        return result

    def _binarize_images(self, amount):
        result = []
        for i in range(amount):
            bin_image = HeatmapImageBinarizer(
                self.batch_y[self.index_counter + i],
                self.batch_x[self.index_counter + i],
                40).run()

            # closed = diameter_opening(bin_image)
            result.append(bin_image)
        return result

    def _get_matches_img_of_binary_pair(self, bin_image1, bin_image2):
        bin_image1u8 = (bin_image1 * 255.0).astype('uint8')
        bin_image2u8 = (bin_image2 * 255.0).astype('uint8')

        orb = cv2.ORB_create(nfeatures=500)

        keypoints_orb1, descriptors1 = orb.detectAndCompute(bin_image1u8, None)
        keypoints_orb2, descriptors2 = orb.detectAndCompute(bin_image2u8, None)

        matches = self._match_flann(descriptors1, descriptors2)
        matches_img = cv2.drawMatches(bin_image1u8, keypoints_orb1, bin_image2u8, keypoints_orb2, matches[:10], None,
                                      flags=2)

        points1 = np.float32([keypoints_orb1[match.queryIdx].pt for match in matches]).reshape(-1, 1, 2)
        points2 = np.float32([keypoints_orb2[match.trainIdx].pt for match in matches]).reshape(-1, 1, 2)
        homography, mask = cv2.findHomography(points1, points2, cv2.RANSAC)

        height, width = bin_image2.shape[:2]
        warped_img1 = cv2.warpPerspective(bin_image1, homography, (width, height))
        return matches_img, warped_img1

    def _match_bf(self, descriptors1, descriptors2):
        matcher = cv2.BFMatcher_create(cv2.NORM_HAMMING2, crossCheck=True)
        matches = matcher.match(descriptors1, descriptors2)
        matches = sorted(matches, key=lambda x: x.distance)
        return matches[:20]

    def _match_flann(self, descriptors1, descriptors2):
        FLANN_INDEX_LSH = 6
        index_params = dict(algorithm=FLANN_INDEX_LSH, table_number=6,  # 12
                            key_size=12,  # 20
                            multi_probe_level=1)  # 2
        matcher = cv2.FlannBasedMatcher(index_params, {})
        matches = matcher.knnMatch(descriptors1, descriptors2, 2)
        matches = self._filter_matches_lowe(matches)
        return matches

    def _filter_matches_lowe(self, matches):
        ratio_thresh = 0.7
        good_matches = []
        for item in matches:
            if len(item) < 2:
                continue
            m, n = item
            if m.distance < ratio_thresh * n.distance:
                good_matches.append(m)
        return good_matches

    def _calc_centroid_of_bin(self, bin_image):
        mask_ones = np.where(bin_image == 1)
        x_centroid = np.mean(mask_ones[0])
        y_centroid = np.mean(mask_ones[1])
        # self.axes[i][2].plot(centroid[1], centroid[0], 'rx')
        return (x_centroid, y_centroid)

    def on_key(self, event):
        super().on_key(event)
        # if event.key == 't':
        #    self.threshold += 5
        # elif event.key == 'r':
        #    self.threshold -= 5
        # elif event.key == 's':
        #    extent = self.axes[2][0].get_window_extent().transformed(self.fig.dpi_scale_trans.inverted())
        #    self.fig.savefig('out.png', bbox_inches=extent, pad_inches=0)
        self.view()


if __name__ == '__main__':
    image_path = "L:\\experimental_datasets\\video_bin_test\\steady_desk\\"
    heatmap_path = "L:\\experimental_datasets\\video_bin_test\\steady_desk\\inference\\"

    data_provider = ImageHeatmapFileDataProvider(image_path, heatmap_path, 32)
    viewer = HeatmapViewer(data_provider, 32)
    viewer.view()
