import argparse
import os

from skimage.draw import circle
from skimage.feature.peak import peak_local_max
from skimage.io import imread, imsave


def parse_args():
    parser = argparse.ArgumentParser(description="stalwart-keypoint-alpaca inference")

    parser.add_argument('--frame_dir', dest='frame_dir', required=False, help='')
    parser.add_argument('--pred_dir', dest='pred_dir', required=False, help='')

    return parser.parse_args()


def max_filter_heatmap(heatmap):
    image_max = peak_local_max(heatmap, min_distance=10, num_peaks=2)
    return image_max


def rescale_coords(coords, src_width, src_height, target_width, target_height):
    result_coords = []
    for coord in coords:
        new_coord = [int(coord[0] / src_height * target_height),
                     int(coord[1] / src_width * target_width)]
        result_coords.append(new_coord)
    return result_coords


def create_merged_frame(img, heatmap_peaks):
    for peak in heatmap_peaks:
        rr, cc = circle(peak[0], peak[1], 2)
        img[rr, cc] = (255, 0, 0)
    return img


def main():
    args = parse_args()

    if os.path.isdir(args.frame_dir) and \
            os.path.isdir(args.pred_dir):

        frame_list = os.listdir(args.frame_dir)
        frame_list = [os.path.join(args.frame_dir, filename) for filename in frame_list if 'jpg' in filename]

        pred_list = os.listdir(args.pred_dir)
        pred_list = [os.path.join(args.pred_dir, filename) for filename in pred_list if 'jpg' in filename]

        counter = 0
        for frame, pred in zip(frame_list, pred_list):
            image = imread(frame)
            heatmap = imread(pred)
            heatmap_max = max_filter_heatmap(heatmap)
            img_height, img_width = image.shape[:2]
            heatmap_height, heatmap_width = heatmap.shape[:2]
            rescaled_heatmap_max = rescale_coords(heatmap_max, heatmap_width, heatmap_height, img_width, img_height)
            output_frame = create_merged_frame(image, rescaled_heatmap_max)
            imsave("merged_{}.jpg".format(counter), output_frame)
            print("Processed {}".format(frame))
            counter += 1

    pass


if __name__ == "__main__":
    main()
