import matplotlib

matplotlib.use('TkAgg')
import matplotlib.pyplot as plt


class SequenceViewer(object):
    def __init__(self, display_data_provider, batch_size, rows=2, cols=2, figsize=(14, 14)):
        self.rows = rows
        self.cols = cols
        fig, axes = plt.subplots(ncols=cols, nrows=rows, figsize=figsize)
        self.fig = fig
        self.axes = axes
        self.index_counter = 0
        self.index_increment = 1
        self.batch_counter = 0
        self.display_data_provider = display_data_provider
        self.batch_size = batch_size

        self.batch_x = None
        self.batch_y = None

        fig.canvas.mpl_connect('key_press_event', self.on_key)
        pass

    def _load_current_data(self):
        if len(self.display_data_provider) > self.batch_counter >= 0:
            self.batch_x, self.batch_y = self.display_data_provider[self.batch_counter]
        return

    def _clear_axes(self):
        for ax in self.axes.flatten():
            ax.cla()
        return

    def _view_axes(self):
        # noop
        return

    def view(self):
        self._view_axes()
        plt.show()
        pass

    def next(self):

        if (self.index_counter + self.index_increment) >= self.batch_size:
            if self.batch_counter < len(self.display_data_provider):
                self.index_counter = 0
                self.batch_counter += 1
                self._load_current_data()
            else:
                self.index_counter -= self.index_increment
        else:
            self.index_counter += self.index_increment
        self.view()

    def prev(self):
        if self.index_counter > 0:
            self.index_counter -= self.index_increment
        elif self.index_counter == 0:
            if self.batch_counter > 0:
                self.batch_counter -= 1
                self._load_current_data()
                self.index_counter = self.batch_size - self.index_increment
        self.view()

    def on_key(self, event):
        if event.key == 'e':
            self.next()
        elif event.key == 'w':
            self.prev()
