import matplotlib
import numpy as np

from prototype.camera_streamer import DummyCameraStreamer
from visual.video_analysis.binary_analysis import BinaryAnalyzer

matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
from imgproc.imgproc_utils import resize_image_array_pil
from scipy.special import softmax


def split_hm(hm):
    tip_hm = hm[:, :396]
    tail_hm = hm[:, 396:]
    return tip_hm, tail_hm


def peak_img(image):
    i, j = np.unravel_index(image.argmax(), image.shape)
    return (i, j)


def rescale_img(img):
    return img / np.max(img) * 255.


bin_analyzer = BinaryAnalyzer(bin_dir="L:\\video_analysis\\static\\bin\\",
                              binz_dir="L:\\video_analysis\\static\\binz\\b2\\")

heatmap_stream = DummyCameraStreamer("L:\\video_analysis\\static\\keypoints\\")

hm = heatmap_stream.get()
tip_hm, tail_hm = split_hm(hm)
hm_sum = np.zeros_like(tip_hm).astype('float32')
peak_sum = np.zeros_like(tip_hm).astype('float32')
hm_count = 0

while hm is not None:
    hm_count += 1
    tip_hm, tail_hm = split_hm(hm)
    hm_sum += softmax(tip_hm.astype('float32'))
    peak = peak_img(tip_hm)
    peak_sum[peak] = 255

    hm = heatmap_stream.get()

hm_sum = hm_sum  # / 255.
binz_sum = bin_analyzer.get_binz_frame_sum()

hm_coords = np.where(hm_sum > 2)

testi = np.zeros_like(hm_sum)
testi[hm_coords] = 255

fig, axes = plt.subplots(ncols=1, nrows=3)

resized_hm_sum = resize_image_array_pil(hm_sum, (1920, 1080))
axes[0].imshow(resized_hm_sum)
axes[1].imshow(binz_sum * resized_hm_sum)
# axes[1].imshow(resize_image_array_pil(hm_sum, (1920, 1080)), alpha=0.5)
# axes[1].imshow(peak_sum - hm_sum)
# axes[1].imshow(hm_sum, alpha=0.5)

plt.show()
