import matplotlib
import numpy as np
from skimage.color import rgb2gray

from imgproc.heatmap_image_text_extractor import HeatmapImageTextExtractor
from imgproc.imgproc_utils import resize_image_array_pil
from imgproc.path_extraction.heatmap_path import HeatmapPath
from imgproc.path_extraction.on_off_classifier import OnOffClassifier
from imgproc.path_extraction.online_ink_requester import OnlineInkRequester
from imgproc.path_extraction.path_beautifier import PathBeautifier
from imgproc.path_extraction.path_stroke_splitter import PathStrokeSplitter
from prototype.camera_streamer import DummyCameraStreamer

matplotlib.use('TkAgg')
import matplotlib.pyplot as plt


def create_heatmap_batch_from_folder(dir, batch_count):
    hm_streamer = DummyCameraStreamer(dir)
    batch = np.zeros((batch_count, 224, 224))

    hm = hm_streamer.get()
    count = 0
    while hm is not None:
        hm_resized = resize_image_array_pil(hm, (224, 224))
        batch[count, :, :] = hm_resized
        hm = hm_streamer.get()
        count += 1
    return batch


def create_image_batch_from_folder(dir, batch_count):
    hm_streamer = DummyCameraStreamer(dir)
    batch = np.zeros((batch_count, 224, 224, 3), dtype='uint8')

    hm = hm_streamer.get()
    count = 0
    while hm is not None:
        hm_resized = resize_image_array_pil(hm, (224, 224))
        batch[count, :, :, :] = hm_resized
        hm = hm_streamer.get()
        count += 1
    return batch


def calc_gradient_on_point(point, image):
    x_diff = (image[point[1], point[0] - 1] - image[point[1], point[0] + 1]) / 2
    y_diff = (image[point[1] - 1, point[0]] - image[point[1] + 1, point[0]]) / 2
    return x_diff, -y_diff


def plot_interpolated_points(axes, interpolated_points):
    for method_name, curve in interpolated_points.items():
        axes[0].plot(*curve.T, '-', label=method_name)


def on_key(bla):
    pass


if __name__ == '__main__':
    patch_counter = 10
    fig, axes = plt.subplots(ncols=2, nrows=1, figsize=(14, 7))
    fig.canvas.mpl_connect('key_press_event', on_key)

    batch = create_heatmap_batch_from_folder("../tmp_out/static_224", 757)
    bin_batch = create_heatmap_batch_from_folder("../tmp_out/static_bin224", 1110)
    patch_frame_batch = create_image_batch_from_folder("../tmp_out/static_patchframes", 473)

    bin_finished_image = HeatmapImageTextExtractor(
        bin_batch[400, :, :],
        rgb2gray(patch_frame_batch[400, :, :, :]),
        black_threshold=40).run()

    path = HeatmapPath(batch)
    filtered_points = path.get_filtered_points(flip_y=False)
    # on_points, on_indices = OnOffClassifier(filtered_points, bin_batch).run()

    nicer_filtered_points = PathBeautifier(filtered_points, [], bin_finished_image, iterations=3).run()
    on_points, on_indices = OnOffClassifier(nicer_filtered_points, bin_batch).run()

    strokes = PathStrokeSplitter(nicer_filtered_points, on_indices).run()

    for ax in axes.flatten():
        ax.cla()

    axes[0].imshow(bin_finished_image)
    axes[0].axis('equal')
    axes[0].legend()
    axes[0].set_xlabel('x')
    axes[0].set_ylabel('y')

    axes[0].plot(*filtered_points.T, 'xk', label='original points')
    axes[0].plot(*nicer_filtered_points.T, '.r', label='gap fixed points')
    axes[0].plot(*on_points.T, '.b', label='on points')

    first_stroke = np.array(strokes[0])
    second_stroke = np.array(strokes[1])
    third_stroke = np.array(strokes[2])

    axes[1].imshow(bin_finished_image)
    axes[1].plot(*first_stroke.T, '.r', label='new on points')
    axes[1].plot(*second_stroke.T, '.r', label='new on points')
    axes[1].plot(*third_stroke.T, '.r', label='new on points')

    online_recognizer = OnlineInkRequester().online_ink(strokes)

    plt.show()
