import math
import os
from ast import literal_eval

import cv2
import matplotlib
import numpy as np
import pandas as pd

from imgproc.heatmap_image_binarizer import HeatmapImageBinarizer
from imgproc.heatmap_keypoint_extractor import HeatmapKeypointExtractor, MaxFilterPeakExtraction
from imgproc.imgproc_utils import resize_image_array_pil
from prototype.camera_streamer import DummyCameraStreamer, VideoFrameStreamer
from visual.video_analysis.binary_analysis import BinaryAnalyzer

matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
from util.dollar import Recognizer


# needs merged tip/tail heatmaps with 396w x 236h
def get_tip_path_from_heatmaps(heatmap_dir, rescale_shape=None):
    heatmap_stream = DummyCameraStreamer(heatmap_dir)
    heatmap = heatmap_stream.get()
    tips_x = []
    tips_y = []
    tails_x = []
    tails_y = []
    while heatmap is not None:
        print(heatmap_stream.get_current_image_name())
        heatmap_tip = heatmap[:, :396]
        heatmap_tail = heatmap[:, 396:]
        tip = HeatmapKeypointExtractor(heatmap_tip, MaxFilterPeakExtraction).run()
        tail = HeatmapKeypointExtractor(heatmap_tail, MaxFilterPeakExtraction).run()
        if rescale_shape is not None:
            rescale_w = rescale_shape[0]
            rescale_h = rescale_shape[1]
            if tip[1] > 0 and tip[0] > 0:
                tips_x.append(tip[1] / 396 * rescale_w)
                tips_y.append(tip[0] / 236 * rescale_h)
                tails_x.append(tail[1] / 396 * rescale_w)
                tails_y.append(tail[0] / 236 * rescale_h)
        else:
            if tip[1] > 0 or tip[0] > 0:
                tips_x.append(tip[1])
                tips_y.append(tip[0])
                tails_x.append(tail[1])
                tails_y.append(tail[0])
        heatmap = heatmap_stream.get()
    return tips_x, tips_y, tails_x, tails_y


# needs rough tip scaled to image size already
def get_fine_tip_from_heatmap(heatmap, orig_side_length_hm, rough_tip):
    patch_radius = orig_side_length_hm / 2
    scaled_x = int(rough_tip[0])
    scaled_y = int(rough_tip[1])
    scaled_tip_heatmap = resize_image_array_pil(heatmap, (orig_side_length_hm, orig_side_length_hm))
    fine_tip = HeatmapKeypointExtractor(scaled_tip_heatmap, MaxFilterPeakExtraction).run()
    fine_x = scaled_x - patch_radius + fine_tip[1]
    fine_y = scaled_y - patch_radius + fine_tip[0]
    return fine_x, fine_y


# needs rough tips scaled to image size already
def get_fine_tip_path_from_heatmaps(tiplet_heatmap_dir, tip_path, orig_side_length_hm):
    heatmap_stream = DummyCameraStreamer(tiplet_heatmap_dir)
    heatmap = heatmap_stream.get()
    fine_tips_x = []
    fine_tips_y = []
    tip_path_counter = 0
    while heatmap is not None:
        print(heatmap_stream.get_current_image_name())
        current_rough_tip = tip_path[tip_path_counter]
        if current_rough_tip[1] > 0 and current_rough_tip[0] > 0:
            fine_tip_x, fine_tip_y = get_fine_tip_from_heatmap(heatmap, orig_side_length_hm, current_rough_tip)
            fine_tips_x.append(fine_tip_x)
            fine_tips_y.append(fine_tip_y)
            heatmap = heatmap_stream.get()
            tip_path_counter += 1
    return fine_tips_x, fine_tips_y


def filter_outliers_from_tip_stream(tips_x, tips_y, threshold):
    my_coords = list(zip(tips_x, tips_y))
    prev_coord = None
    recorded_indices = []

    for index, coord in enumerate(my_coords):
        if index > 0:
            dist = math.sqrt((coord[0] - prev_coord[0]) ** 2 + (coord[1] - prev_coord[1]) ** 2)
            # dist = np.linalg.norm(np.array(my_coords)-np.array(prev_coord))
            if dist > threshold:
                recorded_indices.append(index)
            else:
                prev_coord = coord
        else:
            prev_coord = coord

    for elem in sorted(recorded_indices, reverse=True):
        del tips_x[elem]
        del tips_y[elem]

    return tips_x, tips_y


class VideoFrameViewer():
    def __init__(self, anno_dataframe, base_dir, start_image=0):
        fig, axes = plt.subplots(ncols=2, nrows=2, figsize=(20, 14))
        self.fig = fig
        self.axes = axes
        self.index = 0
        self.marked_images = []
        self.anno_dataframe = anno_dataframe

        # self.camera_stream = DummyCameraStreamer(os.path.join(base_dir, "frames\\"), counter=start_image)
        self.camera_stream = VideoFrameStreamer("L:\\video_analysis\\gesture_test1\\gesture_test.mp4")

        self.heatmap_stream = DummyCameraStreamer(os.path.join(base_dir, "keypoints\\"), counter=start_image)
        # self.bin_stream = DummyCameraStreamer(os.path.join(base_dir, "bin\\"), counter=start_image)
        # self.patch_stream = DummyCameraStreamer(os.path.join(base_dir, "tip_patches\\"), counter=start_image)
        # self.tiplet_stream = DummyCameraStreamer(os.path.join(base_dir, "tip_inferences\\"), counter=start_image)

        self.frame = None
        self.heatmap = None
        self.frame_bin = None
        self.tip_patch = None
        self.tip_heatmap = None

        self.tips_x = []
        self.tips_y = []

        # self.tips_x, self.tips_y, _, _ = get_tip_path_from_heatmaps(os.path.join(base_dir, "keypoints\\"), rescale_shape=(1920, 1080))

        # self.tips_x, self.tips_y = filter_outliers_from_tip_stream(self.tips_x, self.tips_y, 100)

        self.anno_tips_x = []
        self.anno_tips_y = []

        self.fine_tips_x = []
        self.fine_tips_y = []

        self.binary_analyzer = BinaryAnalyzer(binz_dir=os.path.join(base_dir, "binz\\"), frame_count=1300)
        self.binz_sum = self.binary_analyzer.get_binz_frame_sum()

        # self.fine_tips_x, self.fine_tips_y = get_fine_tip_path_from_heatmaps(
        #    "L:\\video_analysis\\6_0\\tip_inferences\\",
        #    list(zip(self.tips_x, self.tips_y)),
        #    48)

        fig.canvas.mpl_connect('key_press_event', self.on_key)
        fig.canvas.mpl_connect('button_press_event', self.on_click)

        self._load_current_frame_and_results()
        pass

    def _load_current_frame_and_results(self):
        self.frame = self.camera_stream.get()
        self.heatmap = self.heatmap_stream.get()
        # self.frame_bin = self.bin_stream.get()
        # self.tip_patch = self.patch_stream.get()
        # self.tip_heatmap = self.tiplet_stream.get()
        return

    def _clear_axes(self):
        for ax in self.axes.flatten():
            ax.cla()
        # self.axes.cla()
        return

    def _view_axes2(self):
        self._clear_axes()
        resized_image = resize_image_array_pil(self.frame, (1920, 1080))

        ################################################
        # frame
        ################################################

        self.axes[0][0].imshow(resized_image)

        ################################################
        # heatmap overlay and binarization heatmap
        ################################################

        overlay = np.zeros((236, 396, 4), dtype="float32")
        heatmap_tip = self.heatmap[:, :396]
        heatmap_tail = self.heatmap[:, 396:]

        cmap = plt.cm.jet
        overlay = cmap(heatmap_tip)
        overlay[:, :, 3] = heatmap_tip / 255.0

        self.axes[0][1].imshow(self.frame_bin, cmap='cividis', interpolation='nearest', )
        self.axes[0][1].imshow(overlay)

        ################################################
        # binarized image
        ################################################

        binary_image = HeatmapImageBinarizer(self.frame_bin, resized_image, 40).run()
        self.axes[1][1].imshow(binary_image, cmap='cividis', interpolation='nearest', )

        ################################################
        # tiplet heatmap on frame
        ################################################

        tip = HeatmapKeypointExtractor(heatmap_tip, MaxFilterPeakExtraction).run()
        patch_radius = 24
        background = np.zeros((1080, 1920, 4))
        scaled_x = int(tip[1] / 396 * 1920)
        scaled_y = int(tip[0] / 236 * 1080)
        scaled_patch = resize_image_array_pil(self.tip_patch, (48, 48))
        scaled_tip_heatmap = resize_image_array_pil(self.tip_heatmap, (48, 48))
        cmap_tip_heatmap = cmap(scaled_tip_heatmap)
        # flip_patch = np.flipud(self.tip_patch)

        background[scaled_y - patch_radius:scaled_y + patch_radius, scaled_x - patch_radius:scaled_x + patch_radius,
        0:3] = cmap_tip_heatmap[:, :, 0:3]  # scaled_patch/255.
        background[scaled_y - patch_radius:scaled_y + patch_radius, scaled_x - patch_radius:scaled_x + patch_radius,
        3] = scaled_tip_heatmap / 255.  # 1
        self.axes[0][0].imshow(background)

        ################################################
        # annotated keypoints on frame
        ################################################

        current_image_name = "6_0_" + self.camera_stream.get_current_image_name()
        if current_image_name in self.anno_dataframe:
            current_tip_anno_row = self.anno_dataframe[current_image_name]
            current_tip_anno = current_tip_anno_row['ptip']
            self.anno_tips_x.append(current_tip_anno[0] / 3840 * 1920)
            self.anno_tips_y.append(current_tip_anno[1] / 2160 * 1080)

        self.axes[0][0].plot(self.anno_tips_x, self.anno_tips_y, 'rx')

        ################################################
        # detected paths on frame and binarization
        ################################################

        self.axes[0][0].plot(self.tips_x, self.tips_y, 'r-')
        self.axes[1][1].plot(self.tips_x, self.tips_y, 'r-')
        self.axes[0][0].plot(self.fine_tips_x, self.fine_tips_y, 'g-')
        self.axes[1][1].plot(self.fine_tips_x, self.fine_tips_y, 'g-')

    def _view_axes_tips(self):
        self._clear_axes()
        start = 0
        stop = 300
        self.axes[0][0].imshow(self.frame)
        self.axes[0][0].plot(self.tips_x[start:stop], self.tips_y[start:stop], 'r-')

        filter_tips = list(zip(self.tips_x[start:stop], self.tips_y[start:stop]))
        filter_tips_x = [elem[0] for elem in filter_tips if elem[0] < 960]
        filter_tips_y = [elem[1] for elem in filter_tips if elem[0] < 960]

        reco = Recognizer()
        result = reco.Recognize(list(zip(filter_tips_x, filter_tips_y)))
        print(result.Name)
        print(result.Score)

        self.axes[1][1].imshow(self.binz_3d)
        # check = reco.Templates[-1]
        # check_x, check_y = zip(*check.Points)
        # cross = reco.Templates[-2]
        # cross_x, cross_y = zip(*cross.Points)
        # self.axes[0][1].plot(check_x, check_y)
        # self.axes[1][1].plot(cross_x, cross_y)

    def _view_axes(self):
        self._clear_axes()
        # self.axes[0][0].imshow(self.binz_sum)
        xx, yy, z = self.binary_analyzer.get_binz_3d_surface_data()
        self.axes[0][0] = self.fig.gca(projection='3d')
        self.axes[0][0].plot_surface(xx, yy, z, rstride=1, cstride=1,
                                     cmap=plt.cm.viridis, linewidth=0)

    def on_click(self, event):

        return

    def on_key(self, event):
        if event.key == 'e':
            self.next()
        elif event.key == 'w':
            self.prev()

    def view(self):
        self._view_axes()
        plt.show()
        pass

    def next(self):
        self.index += 1
        self._load_current_frame_and_results()
        self.view()
        return

    def prev(self):
        if self.index > 0:
            self.index -= 1
        self._load_current_frame_and_results()
        self.view()
        return


def create_3d_binz_data(binz_dir):
    # binz 3d plot
    # data: (1920, 1080, framescount)
    binz_stream = DummyCameraStreamer(binz_dir, counter=0)

    current_binz = binz_stream.get()
    frames = []
    binz_3d = np.zeros((current_binz.shape[0], current_binz.shape[1]))
    print(current_binz.shape)
    while current_binz is not None:
        frames.append(current_binz)
        # graybinz = cv2.cvtColor(current_binz, cv2.COLOR_RGB2GRAY)
        binz_3d += current_binz / 255.
        current_binz = binz_stream.get()
    binz_3d = binz_3d / np.max(binz_3d) * 255.
    return binz_3d


def create_and_save_tip_paths():
    print("creating tip path from heatmaps")
    tips_x, tips_y, tails_x, tails_y = get_tip_path_from_heatmaps("L:\\video_analysis\\6_0\\keypoints\\",
                                                                  rescale_shape=(1920, 1080))
    np.save('tips_x.npy', np.array(tips_x))
    np.save('tips_y.npy', np.array(tips_y))
    np.save('tails_x.npy', np.array(tails_x))
    np.save('tails_y.npy', np.array(tails_y))

    print("creating fine tip path from heatmaps")
    fine_tips_x, fine_tips_y = get_fine_tip_path_from_heatmaps(
        "L:\\video_analysis\\6_0\\tip_inferences\\",
        list(zip(tips_x, tips_y)),
        48,
        (1920, 1080))

    np.save('fine_tips_x.npy', np.array(fine_tips_x))
    np.save('fine_tips_y.npy', np.array(fine_tips_y))


def write_video_file_of_images(filename, images, size):
    out = cv2.VideoWriter(filename, cv2.VideoWriter_fourcc(*'MJPG'), 60, size)

    for i in range(len(images)):
        out.write(images[i])
    out.release()
    return


def create_resized_video_of_tiptail_heatmaps(filename, heatmap_dir):
    hm_stream = DummyCameraStreamer(heatmap_dir, counter=0)
    # bin_stream = DummyCameraStreamer(bin_dir, counter=0)

    current_heatmap = hm_stream.get()
    # current_bin = bin_stream.get()
    frames = []

    while current_heatmap is not None:
        img = np.zeros((236, 396, 3), dtype='uint8')
        img[:, :, 1] = current_heatmap[:, :396]
        img[:, :, 0] = current_heatmap[:, 396:]
        # current_merged_hm = np.array(current_heatmap[:, :396])
        # current_merged_hm += np.array(current_heatmap[:, 396:])
        # current_merged_hm = current_merged_hm / np.max(current_merged_hm) * 255.
        # bgr_hm = cv2.cvtColor(current_merged_hm.astype('uint8'),cv2.COLOR_GRAY2BGR)
        bgr_hm = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
        frames.append(bgr_hm)
        current_heatmap = hm_stream.get()
    write_video_file_of_images(filename, frames, (396, 236))


def create_resized_video_of_bin_heatmaps(filename, heatmap_dir, size):
    hm_stream = DummyCameraStreamer(heatmap_dir, counter=0)
    # bin_stream = DummyCameraStreamer(bin_dir, counter=0)

    current_heatmap = hm_stream.get()
    # current_bin = bin_stream.get()
    frames = []

    while current_heatmap is not None:
        bgr_hm = cv2.cvtColor(current_heatmap.astype('uint8'), cv2.COLOR_GRAY2BGR)
        frames.append(bgr_hm)
        current_heatmap = hm_stream.get()
    write_video_file_of_images(filename, frames, size)


if __name__ == '__main__':
    current_annos = pd.read_csv('updated_all_old.csv', index_col=0,
                                dtype={'subject': np.int32,
                                       'task': np.int32,
                                       'frame': np.int32},
                                converters={
                                    'ptip': literal_eval,
                                    'ptail': literal_eval})

    viewer = VideoFrameViewer(current_annos, "L:\\video_analysis\\combined\\", start_image=0)
    viewer.view()
    # tips_x, tips_y, tails_x, tails_y = get_tip_path_from_heatmaps("L:\\video_analysis\\6_0\\keypoints\\", rescale_shape=(1920, 1080))
    #
    # np.save('tips_x.npy', np.array(tips_x))
    # np.save('tips_y.npy', np.array(tips_y))
    # create_resized_video_of_tiptail_heatmaps("combined_keypoints.avi", "L:\\video_analysis\\combined\\keypoints\\")
    # create_resized_video_of_bin_heatmaps("combined_binz.avi", "L:\\video_analysis\\combined\\binz\\", (1920,1080))
