import numpy as np
import pyvista as pv
from scipy.special import softmax

from imgproc.imgproc_utils import resize_image_array_pil
from prototype.camera_streamer import DummyCameraStreamer


def split_hm(hm):
    tip_hm = hm[:, :396]
    tail_hm = hm[:, 396:]
    return tip_hm, tail_hm


def create_bin_volume(do_softmax=False):
    bin_stream = DummyCameraStreamer("L:\\video_analysis\\static\\bin\\")
    hm = bin_stream.get()

    hm_count = 0
    # hm_sum = np.zeros_like(tip_hm).astype('float32')
    if do_softmax:
        hm_volume = softmax(hm.astype('float32'))
    else:
        hm_volume = np.copy(hm)

    while hm is not None:
        hm_count += 1
        if do_softmax:
            hm = softmax(hm.astype('float32'))
        hm_volume = np.dstack((hm_volume, hm))
        hm = bin_stream.get()

    return hm_volume


def create_binz_volume(do_softmax=False):
    bin_stream = DummyCameraStreamer("L:\\video_analysis\\static\\binz\\")
    hm = bin_stream.get()
    hm = resize_image_array_pil(hm, (396, 236))

    hm_count = 0
    # hm_sum = np.zeros_like(tip_hm).astype('float32')
    if do_softmax:
        hm_volume = softmax(hm.astype('float32'))
    else:
        hm_volume = np.copy(hm)

    while hm is not None:
        hm = resize_image_array_pil(hm, (396, 236))
        hm_count += 1
        if do_softmax:
            hm = softmax(hm.astype('float32'))
        hm_volume = np.dstack((hm_volume, hm))
        hm = bin_stream.get()

    return hm_volume


def create_hm_volume(heatmap_path, do_softmax=False):
    heatmap_stream = DummyCameraStreamer(heatmap_path)
    hm = heatmap_stream.get()
    tip_hm, tail_hm = split_hm(hm)

    hm_count = 0
    # hm_sum = np.zeros_like(tip_hm).astype('float32')
    if do_softmax:
        hm_volume = softmax(tip_hm.astype('float32'))
    else:
        hm_volume = np.copy(tip_hm)

    while hm is not None:
        hm_count += 1
        tip_hm, tail_hm = split_hm(hm)
        if do_softmax:
            tip_hm = softmax(tip_hm.astype('float32'))
        hm_volume = np.dstack((hm_volume, tip_hm))
        hm = heatmap_stream.get()

    return hm_volume


def save_volume(filename, hm_volume):
    print("saving volume")
    np.save(filename, hm_volume)


def load_volume(filename):
    print("loading volume")
    return np.load(filename)


# bin_volume = load_volume("bin_volume_softmax.npy")
hm_volume = create_hm_volume("L:\\video_analysis\\static\\patch_keypoints\\")
# binz_volume = load_volume("binz_volume.npy")
# save_volume("binz_volume.npy", binz_volume)
save_volume("hm_volume_patch224.npy", hm_volume)

# hm_volume = load_volume()
# Volume rendering is not supported with Panel yet
pv.rcParams["use_panel"] = False

# Download a volumetric dataset
# vol = examples.download_knee_full()
cpos = [(-381.74, -46.02, 216.54), (74.8305, 89.2905, 100.0), (0.23, 0.072, 0.97)]

p = pv.Plotter()
p.add_volume(hm_volume, cmap="bone", opacity="sigmoid")
# p.add_volume(hm_volume, cmap="viridis", opacity="sigmoid")
p.camera_position = cpos
p.show()
